@@ -2,11 +2,9 @@
 # _accessmanager.py
 #
 # $Id: _accessmanager.py,v 1.11 2004/10/04 19:57:06 zmsdev Exp $
-# $Name:  $
+# $Name:$
 # $Author: zmsdev $
 # $Revision: 1.11 $
-#
-# Implementation of class AccessManager (see below).
 # 
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -29,6 +27,7 @@ from Globals import HTMLFile
 from AccessControl.User import UserFolder
 from types import StringTypes
 import copy
+import sys
 import time
 import urllib
 # Product Imports.
@@ -136,7 +135,7 @@ def deleteUser(self, id, REQUEST):
 ################################################################################
 ################################################################################
 ###
-###   C l a s s   A c c e s s a b l e O b j e c t
+###   Class AccessableObject
 ###
 ################################################################################
 ################################################################################
@@ -172,7 +171,7 @@ class AccessableObject:
             for right in rights:
               access = access or auth_user.has_role( right)
               access = access or auth_user.has_permission( right, self)
-        if access and self.meta_type == 'ZMSCustom':
+        if access:
           ob_access = self.getObjProperty('manage_access',REQUEST)
           access = access and ((not type(ob_access) is dict) or (ob_access.get( 'edit') is None) or (len( self.intersection_list( self.concat_list( ob_access.get( 'edit'), [ 'Manager']), self.getUserRoles(auth_user))) > 0))
         access = access and auth_user.has_permission( 'ZMS Author', self) in [ 1, True]
@@ -195,7 +194,11 @@ class AccessableObject:
         pass
       nodes = self.getUserAttr(userObj,'nodes',{})
       ob = self
+      depth = 0
       while ob is not None:
+        if depth > sys.getrecursionlimit():
+          raise "Maximum recursion depth exceeded"
+        depth = depth + 1
         nodekey = self.getRefObjPath(ob)
         if nodekey in nodes.keys():
           roles = self.concat_list(roles,nodes[nodekey]['roles'])
@@ -217,7 +220,11 @@ class AccessableObject:
         pass
       nodes = self.getUserAttr(userObj, 'nodes', {})
       ob = self
+      depth = 0
       while ob is not None:
+        if depth > sys.getrecursionlimit():
+          raise "Maximum recursion depth exceeded"
+        depth = depth + 1
         nodekey = self.getRefObjPath(ob)
         if nodekey in nodes.keys():
           langs = nodes[nodekey]['langs']
@@ -229,14 +236,12 @@ class AccessableObject:
       return langs
 
 
-    """
     ############################################################################
     ###
-    ###   Public Access (Subscribers)
+    ###  Public Access (Subscribers)
     ###
     ############################################################################
-    """
-    
+
     # --------------------------------------------------------------------------
     #  AccessableObject.viewPermission:
     #
@@ -277,7 +282,7 @@ class AccessableObject:
       # AccessableObject and ZMSContainerObject is inherited from 
       # AccessableContainer!
       restricted = self.hasRestrictedAccess()
-      if self.meta_type == 'ZMSLinkElement' and self.isEmbedded( self.REQUEST):
+      if self.meta_id == 'ZMSLinkElement' and self.isEmbedded( self.REQUEST):
         ob = self.getRefObj()
         if ob is not None:
           for item in ob.breadcrumbs_obj_path():
@@ -293,20 +298,18 @@ class AccessableObject:
           self.grantPublicAccess()
       
 
-    """
     ############################################################################
     ###
-    ###   P r o p e r t i e s
+    ###  Properties
     ###
     ############################################################################
-    """
 
     ############################################################################
     #  AccessableObject.manage_user:
     #
     #  Change user.
     ############################################################################
-    manage_userForm = HTMLFile('dtml/zms/manage_user', globals())
+    manage_userForm = HTMLFile('dtml/ZMS/manage_user', globals())
     def manage_user(self, btn, lang, REQUEST, RESPONSE):
       """ AccessManager.manage_user """
       message = ''
@@ -338,12 +341,12 @@ class AccessableObject:
 ################################################################################
 ################################################################################
 ###
-###   C l a s s   A c c e s s a b l e C o n t a i n e r
+###   Class AccessableContainer
 ###
 ################################################################################
 ################################################################################
 class AccessableContainer(AccessableObject): 
-    
+
     # --------------------------------------------------------------------------
     #  AccessableContainer.restrictAccess:
     # --------------------------------------------------------------------------
@@ -392,7 +395,7 @@ class AccessableContainer(AccessableObject):
 ################################################################################
 ################################################################################
 ###
-###   C l a s s   A c c e s s M a n a g e r
+###   Class AccessManager
 ###
 ################################################################################
 ################################################################################
@@ -537,9 +540,9 @@ class AccessManager(AccessableContainer):
       userFldrs = homeElmnt.objectValues(user_folder_meta_types)
       if len(userFldrs)==0:
         portalMaster = self.getPortalMaster()
-	if portalMaster is not None:
-	  userFldr = portalMaster.getUserFolder()
-	else:
+        if portalMaster is not None:
+          userFldr = portalMaster.getUserFolder()
+        else:
           userFldr = UserFolder()
           homeElmnt._setObject(userFldr.id, userFldr)
       else:
@@ -553,8 +556,13 @@ class AccessManager(AccessableContainer):
     def getUserFolders(self):
       userFolders = []
       ob = self
-      while 1:
-        if ob is None: break
+      depth = 0
+      while True:
+        if depth > sys.getrecursionlimit():
+          raise "Maximum recursion depth exceeded"
+        depth = depth + 1
+        if ob is None: 
+          break
         try:
           localUserFolders = ob.objectValues(user_folder_meta_types)
           if len(localUserFolders)==1:
@@ -567,13 +575,11 @@ class AccessManager(AccessableContainer):
       return userFolders
 
 
-    """
     ############################################################################
     ###
-    ###   L o c a l   U s e r s
+    ###  Local Users
     ###
     ############################################################################
-    """
 
     # --------------------------------------------------------------------------
     #  AccessManager.setLocalUser:
@@ -611,18 +617,16 @@ class AccessManager(AccessableContainer):
         _globals.writeException(self,'[delLocalUser]: node=%s does not exist!'%node)
 
 
-    """
     ############################################################################
     ###
-    ###   P r o p e r t i e s
+    ###  Properties
     ###
     ############################################################################
-    """
 
     # Management Interface.
     # ---------------------
-    manage_users = HTMLFile('dtml/zms/manage_users', globals())
-    manage_users_sitemap = HTMLFile('dtml/zms/manage_users_sitemap', globals())
+    manage_users = HTMLFile('dtml/ZMS/manage_users', globals())
+    manage_users_sitemap = HTMLFile('dtml/ZMS/manage_users_sitemap', globals())
 
     ############################################################################
     #  AccessManager.manage_roleProperties:
@@ -717,7 +721,7 @@ class AccessManager(AccessableContainer):
       # -------
       if btn in [ self.getZMILangStr('BTN_CANCEL'), self.getZMILangStr('BTN_BACK')]:
         id = ''
-
+      
       # Insert.
       # -------
       if btn == self.getZMILangStr('BTN_INSERT'):
@@ -825,7 +829,7 @@ class AccessManager(AccessableContainer):
             self.sendMail(mto,msubject,mbody,REQUEST)
             #-- Assemble message.
             message = self.getZMILangStr('MSG_CHANGED')
-
+      
       # Return with message.
       if RESPONSE:
         message = urllib.quote(message)
