@@ -2,12 +2,10 @@
 # _blobfields.py
 #
 # $Id: _blobfields.py,v 1.10 2004/11/30 20:03:16 zmsdev Exp $
-# $Name:  $
+# $Name:$
 # $Author: zmsdev $
 # $Revision: 1.10 $
 #
-# Implementation of classes MyFile, MyImage, etc. (see below).
-# 
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
 # as published by the Free Software Foundation; either version 2
@@ -27,7 +25,9 @@
 from __future__ import nested_scopes
 from webdav.common import rfc1123_date
 from zExceptions import Unauthorized 
+from DateTime.DateTime import DateTime
 from ZPublisher import HTTPRangeSupport
+from OFS.CopySupport import absattr
 from OFS.Image import Image, File
 from cStringIO import StringIO
 from mimetools import choose_boundary
@@ -38,6 +38,8 @@ import _fileutil
 import _globals
 import _mimetypes
 
+__all__= ['MyBlob','MyImage','MyFile']
+
 
 # ------------------------------------------------------------------------------
 #  _blobfields.StringType:
@@ -45,45 +47,31 @@ import _mimetypes
 StringType=type('')
 
 
-# ------------------------------------------------------------------------------
-#  _blobfields.hex2bin:
-# ------------------------------------------------------------------------------
-def hex2bin(m):
-  return ''.join(map(lambda x: chr(16*int('0x%s'%m[x*2],0)+int('0x%s'%m[x*2+1],0)),range(len(m)/2)))
-
-
-# ------------------------------------------------------------------------------
-#  _blobfields.bin2hex:
-# ------------------------------------------------------------------------------
-def bin2hex(m):
-  return ''.join(map(lambda x: hex(ord(x)/16)[-1]+hex(ord(x)%16)[-1],m))
-
-
 # ------------------------------------------------------------------------------
 #  _blobfields.recurse_downloadRessources:
 #
 #  Download from ZODB to file-system during Export.
 # ------------------------------------------------------------------------------
 def recurse_downloadRessources(self, base_path, REQUEST, incl_embedded):
+  ressources = []
   # Check Constraints.
   root = getattr( self, '__root__', None)
-  if root is not None or self.meta_type == 'ZMSTrashcan':
-    return []
+  if root is not None:
+    return ressources
   ob = self
-  if ob.meta_type != 'ZMS':
-    base_path += self.id + '/'
-  if ob.meta_type == 'ZMSLinkElement' and ob.isEmbedded( REQUEST) and incl_embedded:
+  if ob.meta_id == 'ZMSLinkElement' and ob.isEmbedded( REQUEST) and incl_embedded:
     ob = ob.getRefObj()
-  ressources = []
   # Attributes.
-  keys = ob.getObjAttrs().keys()
-  for key in keys:
+  langs = ob.getLangIds()
+  prim_lang = ob.getPrimaryLanguage()
+  obj_attrs = ob.getObjAttrs()
+  for key in obj_attrs.keys():
     obj_attr = ob.getObjAttr(key)
     datatype = obj_attr['datatype_key']
     if datatype in _globals.DT_BLOBS:
-      for lang in ob.getLangIds():
+      for lang in langs:
         try:
-          if obj_attr['multilang']==1 or lang==ob.getPrimaryLanguage() or (obj_attr['multilang']==0 and lang!=ob.getPrimaryLanguage()):
+          if obj_attr['multilang'] or lang==prim_lang or (obj_attr['multilang']==0 and lang!=prim_lang):
             req = {'lang':lang,'preview':'preview'}
             obj_vers = ob.getObjVersion(req)
             blob = ob._getObjAttrValue(obj_attr,obj_vers,lang)
@@ -96,41 +84,36 @@ def recurse_downloadRessources(self, base_path, REQUEST, incl_embedded):
               ressources.append( { 'filepath':filename, 'content_type':blob.getContentType()})
         except:
           _globals.writeException(ob,"[recurse_downloadRessources]: Can't export %s"%key)
-    elif datatype == _globals.DT_LIST and obj_attr.get('type') in ['image','file']:
-      for lang in ob.getLangIds():
+    elif datatype == _globals.DT_LIST:
+      for lang in langs:
         try:
-          if obj_attr['multilang']==1 or lang==ob.getPrimaryLanguage() or (obj_attr['multilang']==0 and lang!=ob.getPrimaryLanguage()):
+          if obj_attr['multilang'] or lang==prim_lang or (obj_attr['multilang']==0 and lang!=prim_lang):
             req = {'lang':lang,'preview':'preview'}
             obj_vers = ob.getObjVersion(req)
-            blobs = ob._getObjAttrValue(obj_attr,obj_vers,lang)
+            v = ob._getObjAttrValue(obj_attr,obj_vers,lang)
             i = 0
-            for blob in blobs:
-              filename = blob.getFilename()
-              filename = getLangFilename(ob,filename,lang)
-              filename = '%s@%i/%s'%(base_path,i,filename)
-              filename = _fileutil.getOSPath(filename)
-              _fileutil.exportObj(blob,filename)
-              ressources.append( { 'filepath':filename, 'content_type':blob.getContentType()})
-              i += 1
+            for r in v:
+              uu = []
+              if type(r) is dict:
+                for k in r.keys():
+                  u = r[k]
+                  if isinstance(u,MyImage) or isinstance(u,MyFile):
+                    uu.append( u)
+              elif isinstance(r,MyImage) or isinstance(r,MyFile):
+                uu.append( r)
+              for u in uu:
+                filename = u.getFilename()
+                filename = getLangFilename(ob,filename,lang)
+                filename = '%s@%i/%s'%(base_path,i,filename)
+                filename = _fileutil.getOSPath(filename)
+                _fileutil.exportObj(u,filename)
+                ressources.append( { 'filepath':filename, 'content_type':u.getContentType()})
+              i = i + 1
         except:
           _globals.writeException(ob,"[recurse_downloadRessources]: Can't export %s"%key)
-  if ob.meta_type == 'ZMSCustom':
-    metaObjId = ob.meta_id
-    for metaObjAttrId in ob.getMetaobjAttrIds( metaObjId):
-      metaObjAttr = ob.getMetaobjAttr( metaObjId, metaObjAttrId)
-      if metaObjAttr['type'] == 'resource':
-        try:
-          blob = metaObjAttr['custom']
-          filename = blob.getFilename()
-          filename = '%s%s'%(base_path,filename)
-          filename = _fileutil.getOSPath(filename)
-          _fileutil.exportObj(blob,filename)
-          ressources.append( { 'filepath':filename, 'content_type':blob.getContentType()})
-        except:
-          _globals.writeException( ob, "[recurse_downloadRessources]: Can't export %s.%s"%(metaObjId,metaObjAttrId))
   # Process children.
   for child in ob.getChildNodes():
-    ressources.extend( recurse_downloadRessources( child, base_path, REQUEST, incl_embedded))
+    ressources.extend( recurse_downloadRessources( child, base_path+child.id+'/', REQUEST, incl_embedded))
   # Return list of ressources.
   return ressources
 
@@ -157,14 +140,16 @@ def recurse_uploadRessources(self, folder='.', mediadbStorable=True):
 #  Upload blob-fields from file-system to ZODB during import.
 # ------------------------------------------------------------------------------
 def uploadRessources(self, folder='.', mediadbStorable=True):
+  langs = self.getLangIds()
+  prim_lang = self.getPrimaryLanguage()
   obj_attrs = self.getObjAttrs()
   for key in obj_attrs.keys():
     obj_attr = self.getObjAttr(key)
     datatype = obj_attr['datatype_key']
     if datatype in _globals.DT_BLOBS:
-      for lang in self.getLangIds():
+      for lang in langs:
         try:
-          if obj_attr['multilang']==1 or lang==self.getPrimaryLanguage():
+          if obj_attr['multilang'] or lang==prim_lang:
             req = {'lang':lang,'preview':'preview'}
             obj_vers = self.getObjVersion(req)
             blob = self._getObjAttrValue(obj_attr,obj_vers,lang)
@@ -263,13 +248,11 @@ def getLangFilename(self, filename, lang):
   return name
 
 
-"""
 ################################################################################
 ###
-###   T H U M B N A I L S
+###  THUMBNAILS
 ###
 ################################################################################
-"""
 
 # ------------------------------------------------------------------------------
 #  _blobfields.thumbnailImageFields:
@@ -312,7 +295,11 @@ def thumbnailImage(self, hiresKey, loresKey, maxdim, lang, REQUEST):
 
 ################################################################################
 ################################################################################
-
+###
+###   Class
+###
+################################################################################
+################################################################################
 class MyBlob:
 
     # Documentation string.
@@ -373,7 +360,7 @@ class MyBlob:
         if_range = REQUEST.get_header('If-Range', None)
         if range is not None:
             ranges = HTTPRangeSupport.parseRange(range)
-
+            
             if if_range is not None:
                 # Only send ranges if the data isn't modified, otherwise send
                 # the whole object. Support both ETags and Last-Modified dates!
@@ -562,9 +549,12 @@ class MyBlob:
 
 
     # --------------------------------------------------------------------------
-    # 	MyBlob.equals:
+    #  MyBlob.equals
     # --------------------------------------------------------------------------
     def equals(self, ob):
+      """
+      Indicates whether some other MyBlob-object is "equal to" this one.
+      """
       b = ob is not None
       try:
         if b:
@@ -605,8 +595,21 @@ class MyBlob:
         RESPONSE = REQUEST.RESPONSE
         parent = self.aq_parent
         
-        if not parent.hasAccess( REQUEST) and not parent.getConfProperty( 'ZMS.blobfields.grant_public_access', 0) == 1:
-          # Raise unauthorized error.
+        access = parent.hasAccess( REQUEST) or parent.getConfProperty( 'ZMS.blobfields.grant_public_access', 0) == 1
+        # Hook for custom access rules: return True/False, return 404 (Forbidden) if you want to perform redirect
+        if access:
+          try:
+            name = 'hasCustomAccess'
+            if hasattr(parent,name):
+              v = getattr(parent,name)(context=parent,REQUEST=REQUEST)
+              if type( v) is bool:
+                access = access and v
+              elif type( v) is int and v == 404:
+                return ''
+          except:
+            _globals.writeException(parent,'[__call__]: can\'t %s'%name)
+        # Raise unauthorized error.
+        if not access:
           raise Unauthorized
         
         if self._if_modified_since_request_handler(REQUEST, RESPONSE):
@@ -630,12 +633,25 @@ class MyBlob:
         accept_ranges = parent.getConfProperty('ZMS.blobfields.accept_ranges','bytes')
         if len( accept_ranges) > 0:
           RESPONSE.setHeader('Accept-Ranges', accept_ranges)
-        public_access = parent.hasPublicAccess()
-        cached_page = parent.isCachedPage( REQUEST)
-        if not public_access and not cached_page:
+        cacheable = parent.hasPublicAccess() or parent.isCachedPage( REQUEST)
+        # Hook for custom cacheable rules: return True/False
+        if cacheable:
+          try:
+            name = 'hasCustomPublicAccess'
+            if hasattr(parent,name):
+              v = getattr(parent,name)(context=parent,REQUEST=REQUEST)
+              if type( v) is bool:
+                cacheable = cacheable and v
+          except:
+            _globals.writeException(parent,'[__call__]: can\'t %s'%name)
+        if not cacheable:
           RESPONSE.setHeader('Expires', '-1')
           RESPONSE.setHeader('Cache-Control', 'no-cache')
-          RESPONSE.setHeader('Pragma', 'no-cache')
+          # IE6 SSL Download bug:
+          # http://support.microsoft.com/kb/812935/en-us
+          # http://support.microsoft.com/kb/323308/en-us
+          if not REQUEST.get('URL','').startswith( 'https://'):
+            RESPONSE.setHeader('Pragma', 'no-cache')
         
         if self.ZCacheable_isCachingEnabled():
             result = self.ZCacheable_get(default=None)
@@ -675,6 +691,9 @@ class MyBlob:
     # --------------------------------------------------------------------------
     getData__roles__ = None
     def getData(self, parent=None):
+      """
+      Returns data.
+      """
       data = ''
       mediadbfile = self.getMediadbfile()
       if mediadbfile is not None:
@@ -696,6 +715,11 @@ class MyBlob:
     # --------------------------------------------------------------------------
     getHref__roles__ = None
     def getHref(self, REQUEST):
+      """
+      Returns absolute url.
+      @var REQUEST: the triggering request
+      @type REQUEST: ZPublisher.HTTPRequest
+      """
       parent = self.aq_parent
       key = self.key
       rownum = ''
@@ -727,23 +751,30 @@ class MyBlob:
 
     # --------------------------------------------------------------------------
     #  MyBlob.getMediadbfile: 
-    #
-    #  Returns mediadb-filename.
     # --------------------------------------------------------------------------
     getMediadbfile__roles__ = None
     def getMediadbfile(self):
+      """
+      Returns mediadb-filename.
+      @rtype: C{string}
+      """
       return getattr(self,'mediadbfile',None)
 
 
     # --------------------------------------------------------------------------
     #  MyBlob.getFilename: 
-    #
-    #  Returns filename.
     # --------------------------------------------------------------------------
     getFilename__roles__ = None
     def getFilename(self):
+      """
+      Returns filename.
+      @rtype: C{string}
+      """
       filename = self.filename
-      filename = _globals.filename_quote(filename)
+      while filename.startswith( '_'):
+        filename = filename[1:]
+      for ch in [ '+', '%', ' ', '!', '?', '#', '"']:
+        filename = filename.replace(ch,'')
       if filename != self.filename and len( self.data) > 0: 
         self.filename = filename
       return filename
@@ -751,11 +782,13 @@ class MyBlob:
     
     # --------------------------------------------------------------------------
     #  MyBlob.get_real_size: 
-    #
-    #  Returns real size in ZODB.
     # --------------------------------------------------------------------------
     get_real_size__roles__ = None
     def get_real_size(self):
+      """
+      Returns real size in ZODB.
+      @rtype: C{int}
+      """
       if self.mediadbfile is None:
         return self.get_size()
       else:
@@ -764,40 +797,50 @@ class MyBlob:
 
     # --------------------------------------------------------------------------
     #  MyBlob.getDataSizeStr: 
-    #
-    #  Returns display string of file-size (KB).
     # --------------------------------------------------------------------------
     getDataSizeStr__roles__ = None
     def getDataSizeStr(self):
+      """
+      Returns display string for file-size (KB).
+      @rtype: C{string}
+      @deprecated: Use ZMSGlobals.getDataSizeStr(len) instead!
+      """
       return _fileutil.getDataSizeStr(self.get_size())
 
 
     # --------------------------------------------------------------------------
     #  MyBlob.getContentType:
-    #
-    #  MIME-type.
     # --------------------------------------------------------------------------
     getContentType__roles__ = None
     def getContentType(self):
+      """
+      Returns MIME-type (e.g. image/gif, text/xml).
+      @rtype: C{string}
+      """
       return self.content_type
 
 
     # --------------------------------------------------------------------------
     #  MyBlob.getMimeTypeIconSrc:
-    #
-    #  Image source for MIME-type.
     # --------------------------------------------------------------------------
     getMimeTypeIconSrc__roles__ = None
     def getMimeTypeIconSrc(self):
+      """
+      Returns the absolute-url of an icon representing the MIME-type of this MyBlob-object.
+      @rtype: C{string}
+      @deprecated: Use ZMSGlobals.getMimeTypeIconSrc(mt) instead!
+      """
       return '/misc_/zms/' + _mimetypes.dctMimeType.get( self.getContentType(), _mimetypes.content_unknown)
 
 
     # --------------------------------------------------------------------------
     #  MyBlob.xmlGetTagName: 
-    #
-    #  Returns <XML> Tag-Name.
     # --------------------------------------------------------------------------
     def xmlGetTagName(self):
+      """
+      Returns <XML> Tag-Name.
+      @rtype: C{string}
+      """
       return 'data'
 
 
@@ -831,7 +874,7 @@ class MyImage(MyBlob,Image):
 
 
     # --------------------------------------------------------------------------
-    # 	MyImage.toXml:
+    #  MyImage.toXml:
     # --------------------------------------------------------------------------
     def toXml(self, sender=None, base_path='', data2hex=True):
       data = ''
@@ -841,7 +884,7 @@ class MyImage(MyBlob,Image):
         if getattr(self,'content_type','').find('text/') == 0:
           data = '<![CDATA[%s]]>'%str(self.getData( sender))
         else:
-          data = bin2hex(self.getData( sender))
+          data = _globals.bin2hex(self.getData( sender))
         objtype = ' type="image"'
       else:
         filename = self.getFilename()
@@ -858,7 +901,7 @@ class MyImage(MyBlob,Image):
 
 
     # --------------------------------------------------------------------------
-    # 	MyImage.getWidth:
+    #  MyImage.getWidth:
     # --------------------------------------------------------------------------
     getWidth__roles__ = None
     def getWidth(self):
@@ -866,7 +909,7 @@ class MyImage(MyBlob,Image):
 
 
     # --------------------------------------------------------------------------
-    # 	MyImage.getHeight:
+    #  MyImage.getHeight:
     # --------------------------------------------------------------------------
     getHeight__roles__ = None
     def getHeight(self):
@@ -882,7 +925,7 @@ class MyFile(MyBlob,File):
     __doc__ = """ZMS product module."""
     # Version string. 
     __version__ = '0.1' 
-    
+
 
     __obj_attrs__  = ['content_type','size','data','filename','mediadbfile']
     __xml_attrs__  = ['content_type']
@@ -903,7 +946,7 @@ class MyFile(MyBlob,File):
 
 
     # --------------------------------------------------------------------------
-    # 	MyFile.toXml:
+    #  MyFile.toXml:
     # --------------------------------------------------------------------------
     def toXml(self, sender=None, base_path='', data2hex=True):
       data = ''
@@ -913,7 +956,7 @@ class MyFile(MyBlob,File):
         if getattr(self,'content_type','').find('text/') == 0:
           data = '<![CDATA[%s]]>'%str(self.getData( sender))
         else:
-          data = bin2hex(self.getData( sender))
+          data = _globals.bin2hex(self.getData( sender))
         objtype = ' type="file"'
       else:
         filename = self.getFilename()
@@ -926,4 +969,44 @@ class MyFile(MyBlob,File):
       xml += '</%s>'%self.xmlGetTagName()
       return xml
 
+
+################################################################################
+################################################################################
+
+class MyBlobWrapper:
+
+    # Documentation string.
+    __doc__ = """ZMS product module."""
+    # Version string. 
+    __version__ = '0.1' 
+
+    # --------------------------------------------------------------------------
+    #  MyBlobWrapper.__init__:
+    #
+    #  Constructor
+    # --------------------------------------------------------------------------
+    def __init__(self, f):
+      self.f = f
+
+    # --------------------------------------------------------------------------
+    #  MyBlobWrapper.getHref:
+    # --------------------------------------------------------------------------
+    getHref__roles__ = None
+    def getHref(self, REQUEST):
+      return self.f.absolute_url()
+
+    # --------------------------------------------------------------------------
+    #  MyBlobWrapper.getFilename: 
+    # --------------------------------------------------------------------------
+    getFilename__roles__ = None
+    def getFilename(self):
+      return absattr( self.f.id)
+
+    # --------------------------------------------------------------------------
+    #  MyBlobWrapper.getData:
+    # --------------------------------------------------------------------------
+    getData__roles__ = None
+    def getData(self, parent=None):
+      return self.f.data
+
 ################################################################################
