@@ -2,7 +2,7 @@
 # _builder.py
 #
 # $Id: _builder.py,v 1.6 2004/11/24 21:02:52 zmsdev Exp $
-# $Name:  $
+# $Name:$
 # $Author: zmsdev $
 # $Revision: 1.6 $
 #
@@ -24,9 +24,9 @@
 ################################################################################
 
 # Imports
-try: # Zope >=2.9
+try: # Zope >= 2.9
   import pyexpat
-except: # Zope <2.9
+except: # Zope < 2.9
   from Shared.DC.xml import pyexpat
 import time
 import Globals
@@ -144,7 +144,7 @@ class Builder:
         #### parsing ####
         if _globals.debug( self):
           _globals.writeLog( self, "#### parsing ####")
-        if type(input) is type(''):
+        if type(input) is str:
           # input is a string!
           rv = p.Parse(input, 1)
         else:
@@ -160,7 +160,7 @@ class Builder:
             if not rv:
               break 
         
-        # raise parser exception           
+        # raise parser exception
         if not rv:
             raise ParseError('%s at line %s' % (pyexpat.ErrorString(p.ErrorCode), p.ErrorLineNumber))
         ####
@@ -188,11 +188,16 @@ class Builder:
           
         name = _globals.unencode( name)
         attrs = _globals.unencode( attrs)
-          
+        
         #-- Zope < 2.9
         if type( attrs) is list:
           attrs = self.dict_list( attrs)
         
+        # handle alias
+        if name in self.getMetaobjIds(sort=0) and not self.dGlobalAttrs.has_key(name):
+          attrs['meta_id'] = name
+          name = 'ZMSCustom'
+        
         if self.bInRootTag or \
            self.oRoot == None or \
            (self.oRoot.id == self.getDocumentElement().id and \
@@ -233,11 +238,11 @@ class Builder:
                   newNode = self.dGlobalAttrs[name]['obj_class'](id,sort_id)
                   self.oCurrNode._setObject(newNode.id, newNode)
                   newNode = getattr(self.oCurrNode,newNode.id)
-
+                
                 ##### Meta Objects ####
                 if newNode.meta_type == 'ZMSCustom':
                   meta_id = attrs.get( 'meta_id')
-                  if meta_id not in self.getMetaobjIds():
+                  if meta_id not in self.getMetaobjIds( sort=0):
                     raise ParseError("Unknown meta_id (" + meta_id + "): no special object available!")  # no special object available!
                   newNode.meta_id = meta_id
                 ##### Object State ####
@@ -261,7 +266,7 @@ class Builder:
                 if self.oRoot is None:   # root object set?
                     self.oRoot = newNode # -> set root node
                 
-                # notify new node                
+                # notify new node
                 newNode.xmlOnStartElement(name, attrs, self.oCurrNode, self.oRoot)
                 
                 # set new node as current node
@@ -289,7 +294,7 @@ class Builder:
             
             # set root node as current node
             self.oCurrNode = self.oRoot
-
+        
         # we are inside the XML root now!
         self.bInRootTag=1
         if self.oRootNode is None:
@@ -308,23 +313,26 @@ class Builder:
         """ Builder.OnEndElement """
         if _globals.debug( self): 
           _globals.writeLog( self, "[Builder.OnEndElement(" + str(name) + ")]")
-      
+        
         # do we have a current node?
         if self.oCurrNode==None:
             raise ParseError("Unmatching end tag (" + name + ")")
-
-        # is this the right tag name?
-        if name==self.oCurrNode.xmlGetTagName():
         
-            # notify current node
-            self.oCurrNode.xmlOnEndElement()
-      
-            parent = self.oCurrNode.xmlGetParent()
-
-            # set parent node as current node
-            self.oCurrNode = parent
-            
-
+        # is this the right tag name?
+        if name==self.oCurrNode.xmlGetTagName() or name==self.oCurrNode.meta_type:
+          
+          ##### VersionManager ####
+          self.oCurrNode.resetObjStates()
+          
+          # notify current node
+          self.oCurrNode.xmlOnEndElement()
+          
+          parent = self.oCurrNode.xmlGetParent()
+          
+          # set parent node as current node
+          self.oCurrNode = parent
+          
+        
         else:
             if self.dGlobalAttrs.has_key(name) and \
                self.dGlobalAttrs[name]['obj_class'] is not None:
@@ -332,14 +340,14 @@ class Builder:
                    self.dGlobalAttrs[self.oCurrNode.xmlGetTagName()]['obj_class'] is not None:
                     # tag name is known, but not valid at this place!
                     raise ParseError("Unmatching end tag (/" + name + "), expected(/" + self.oCurrNode.xmlGetTagName() + ")")
-
+            
             else:
-                # tag name is unknown -> offer it to current object       
+                # tag name is unknown -> offer it to current object
                 if not self.oCurrNode.xmlOnUnknownEndTag(name):
                     if name == self._unknownTagName:
                       self._unknownTagName = None
                     _globals.writeLog( self, "[Builder.OnEndElement]: Unknown end-tag (/" + name + ")")  # current object did not accept tag!
-                    # raise ParseError("Unknown end-tag (" + name + ")")  # current object did not accept tag!         
+                    # raise ParseError("Unknown end-tag (" + name + ")")  # current object did not accept tag!
 
 
     ############################################################################
