@@ -2,12 +2,10 @@
 # _confmanager.py
 #
 # $Id: _confmanager.py,v 1.9 2004/11/30 20:03:17 zmsdev Exp $
-# $Name:  $
+# $Name:$
 # $Author: zmsdev $
 # $Revision: 1.9 $
 #
-# Implementation of class Configuration-Manager (see below).
-# 
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
 # as published by the Free Software Foundation; either version 2
@@ -25,6 +23,7 @@
 
 # Imports.
 from __future__ import nested_scopes
+from zope.interface import implements
 from cStringIO import StringIO
 from App.Common import package_home
 from Globals import HTMLFile
@@ -34,18 +33,14 @@ import os
 import stat
 import urllib
 # Product imports.
-import _charformatmanager
+import IZMSMetamodelProvider, IZMSFormatProvider
 import _globals
 import _fileutil
 import _filtermanager
 import _mediadb
-import _metadictmanager
 import _metacmdmanager
-import _metaobjmanager
-import _metapatternmanager
 import _multilangmanager
 import _sequence
-import _textformatmanager
 import _workflowmanager
 import zmslog
 
@@ -86,8 +81,11 @@ def initCSS(self):
     if id not in folder.objectIds():
       folder.manage_addDTMLMethod( id, title, data)
     self.manage_delObjects( ids=excl_ids)
-    cb_copy_data = self.manage_cutObjects(move_ids,REQUEST)
-    trashcan.manage_pasteObjects(cb_copy_data=None,REQUEST=REQUEST)
+    try:
+      cb_copy_data = self.manage_cutObjects(move_ids,self.REQUEST)
+      folder.manage_pasteObjects(cb_copy_data=None,REQUEST=self.REQUEST)
+    except:
+      pass
 
 
 # ------------------------------------------------------------------------------
@@ -102,6 +100,7 @@ def initConf(self, profile, REQUEST):
         self.importConfPackage(filename,REQUEST,createIfNotExists)
       elif filename.find('.xml') > 0:
         self.importConf(filename,REQUEST,createIfNotExists)
+      self.synchronizeObjAttrs()
 
 
 # ------------------------------------------------------------------------------
@@ -116,40 +115,39 @@ def updateConf(self, REQUEST):
 ################################################################################
 ################################################################################
 ###
-###   class ConfManager:
+###   Class
 ###
 ################################################################################
 ################################################################################
 class ConfManager(
 	_multilangmanager.MultiLanguageManager,		# Languages
-	_metadictmanager.MetadictManager,		# Metadata
-	_metaobjmanager.MetaobjManager,			# Spec. Objects
-	_metapatternmanager.MetapatternManager,		# Modules
 	_metacmdmanager.MetacmdManager,			# Actions
 	_workflowmanager.WorkflowManager,		# Workflow
-	_textformatmanager.TextFormatManager,		# Text-Formats
-	_charformatmanager.CharFormatManager,		# Char-Formats
 	_filtermanager.FilterManager,			# Filters (XML Im-/Export)
 	):
-        
+    implements(
+      IZMSMetamodelProvider.IZMSMetamodelProvider,
+      IZMSFormatProvider.IZMSFormatProvider)
+
     # Management Interface.
     # ---------------------
-    manage_customize = HTMLFile('dtml/zms/manage_customize', globals()) 
-    manage_customizeDesignForm = HTMLFile('dtml/zms/manage_customizedesignform', globals()) 
+    manage_customize = HTMLFile('dtml/ZMS/manage_customize', globals()) 
+    manage_customizeDesignForm = HTMLFile('dtml/ZMS/manage_customizedesignform', globals()) 
 
 
     # --------------------------------------------------------------------------
     #  ConfManager.importConfPackage:
     # --------------------------------------------------------------------------
     def importConfPackage(self, file, REQUEST, createIfNotExists=0):
-      if type( file) is type(''):
+      if type( file) is str:
         filename = file
         filepath = package_home(globals())+'/import/'
         file = open(_fileutil.getOSPath(filepath+filename),'rb')
       files = _fileutil.getZipArchive( file)
       for f in files:
         self.importConf(f,REQUEST,createIfNotExists)
-      
+      self.synchronizeObjAttrs()
+
 
     # --------------------------------------------------------------------------
     #  ConfManager.importConf:
@@ -162,20 +160,21 @@ class ConfManager(
         filename = file
         filepath = package_home(globals())+'/import/'
         xmlfile = open(_fileutil.getOSPath(filepath+filename),'rb')
+      zms_system = 1
       if filename.find('.charfmt.') > 0:
-        _charformatmanager.importXml(self, xmlfile, REQUEST, 1, createIfNotExists)
+        self.format_manager.importCharformatXml(xmlfile, REQUEST, zms_system, createIfNotExists)
       elif filename.find('.filter.') > 0:
-        _filtermanager.importXml(self, xmlfile, REQUEST, 1, createIfNotExists)
+        _filtermanager.importXml(self, xmlfile, REQUEST, zms_system, createIfNotExists)
       elif filename.find('.metadict.') > 0:
-        _metadictmanager.importXml(self, xmlfile, REQUEST, 1, createIfNotExists)
+        self.metaobj_manager.importMetadictXml(xmlfile, REQUEST, zms_system, createIfNotExists)
       elif filename.find('.metaobj.') > 0:
-        _metaobjmanager.importXml(self, xmlfile, REQUEST, 1, createIfNotExists)
+        self.metaobj_manager.importMetaobjXml(xmlfile, REQUEST, zms_system, createIfNotExists)
       elif filename.find('.metacmd.') > 0:
-        _metacmdmanager.importXml(self, xmlfile, REQUEST, 1, createIfNotExists)
+        _metacmdmanager.importXml(self, xmlfile, REQUEST, zms_system, createIfNotExists)
       elif filename.find('.langdict.') > 0:
-        _multilangmanager.importXml(self, xmlfile, REQUEST, 1, createIfNotExists)
+        _multilangmanager.importXml(self, xmlfile, REQUEST, zms_system, createIfNotExists)
       elif filename.find('.textfmt.') > 0:
-        _textformatmanager.importXml(self, xmlfile, REQUEST, 1, createIfNotExists)
+        self.format_manager.importTextformatXml(xmlfile, REQUEST, zms_system, createIfNotExists)
       xmlfile.close()
 
 
@@ -198,11 +197,11 @@ class ConfManager(
 
 
     """
-    ############################################################################    
+    ############################################################################
     ###
     ###   Configuration-Properties Getters
     ###
-    ############################################################################    
+    ############################################################################
     """
 
     # --------------------------------------------------------------------------
@@ -213,7 +212,8 @@ class ConfManager(
     def getSequence(self):
       id = 'acl_sequence'
       if id not in self.objectIds(['Sequence']):
-        _sequence.manage_addSequence(self)
+        sequence = _sequence.Sequence()
+        self._setObject(sequence.id, sequence)
       ob = getattr(self,id)
       return ob
 
@@ -261,40 +261,39 @@ class ConfManager(
                 else:
                   obs.append( ob)
       return obs
-      
+
 
     """
-    ############################################################################    
+    ############################################################################
     ###
     ###   Configuration-Tab Options
     ###
-    ############################################################################    
+    ############################################################################
     """
-    
+
     # --------------------------------------------------------------------------
     #  ConfManager.customize_manage_options:
     # --------------------------------------------------------------------------
-    def customize_manage_options(self, lang, REQUEST):
-      options = []
-      options.append({'label':self.getZMILangStr('TAB_SYSTEM'),'action':'manage_customize'})
-      options.append({'label':self.getZMILangStr('TAB_LANGUAGES'),'action':'manage_customizeLanguagesForm'})
-      options.append({'label':self.getZMILangStr('TAB_METADATA'),'action':'manage_customizeMetadictForm'})
-      options.append({'label':self.getZMILangStr('TAB_METAOBJ'),'action':'manage_customizeMetaobjForm'})
-      options.append({'label':self.getZMILangStr('TAB_METAPATTERN'),'action':'manage_customizeMetapatternForm'})
-      options.append({'label':self.getZMILangStr('TAB_METACMD'),'action':'manage_customizeMetacmdForm'})
-      options.append({'label':self.getZMILangStr('TAB_WORKFLOW'),'action':'manage_customizeWorkflowForm'})
-      options.append({'label':self.getZMILangStr('TAB_TEXTFORMAT'),'action':'manage_customizeTextFormatForm'})
-      options.append({'label':self.getZMILangStr('TAB_FILTER'),'action':'manage_customizeFilterForm'})
-      options.append({'label':self.getZMILangStr('TAB_DESIGN'),'action':'manage_customizeDesignForm'})
-      return options
+    def customize_manage_options(self):
+      l = []
+      l.append({'label':'TAB_SYSTEM','action':'manage_customize'})
+      l.append({'label':'TAB_LANGUAGES','action':'manage_customizeLanguagesForm'})
+      for ob in self.objectValues(['ZMSMetamodelProvider','ZMSFormatProvider']):
+        for d in ob.manage_sub_options:
+          l.append(self.operator_setitem(d.copy(),'action',ob.id+'/'+d['action']))
+      l.append({'label':'TAB_METACMD','action':'manage_customizeMetacmdForm'})
+      l.append({'label':'TAB_WORKFLOW','action':'manage_customizeWorkflowForm'})
+      l.append({'label':'TAB_FILTER','action':'manage_customizeFilterForm'})
+      l.append({'label':'TAB_DESIGN','action':'manage_customizeDesignForm'})
+      return l
 
 
     """
-    ############################################################################    
+    ############################################################################
     ###
     ###   Configuration-Properties
     ###
-    ############################################################################    
+    ############################################################################
     """
 
     # --------------------------------------------------------------------------
@@ -305,6 +304,19 @@ class ConfManager(
     def __get_attr_conf_dict__(self):
       return getattr( self, '__attr_conf_dict__', {})
 
+    # --------------------------------------------------------------------------
+    #  ConfManager.getConfProperty:
+    #
+    #  Removes property from configuration.
+    #
+    #  @param key	The key.
+    # --------------------------------------------------------------------------
+    def delConfProperty(self, key):
+      try:
+        del self.__get_attr_conf_dict__()[key]
+      except:
+        pass
+
     # --------------------------------------------------------------------------
     #  ConfManager.getConfProperty:
     #
@@ -338,11 +350,11 @@ class ConfManager(
 
 
     """
-    ############################################################################    
+    ############################################################################
     ###
     ###   Configuration-System
     ###
-    ############################################################################    
+    ############################################################################
     """
 
     ############################################################################
@@ -356,25 +368,19 @@ class ConfManager(
       message = ''
       params = []
       
-      ##### ASP ####
-      if key == 'ASP':
-        self.setConfProperty('ZMSAdministrator.email',REQUEST.get('zmsadministrator_email',''))
-        self.setConfProperty('ASP.ip_or_domain',REQUEST.get('asp_ip_or_domain',''))
-        message = self.getZMILangStr('MSG_CHANGED')
-      
       ##### Import ####
-      elif key == 'Import':
+      if key == 'Import':
         if btn == 'Import':
           f = REQUEST['file']
           createIfNotExists = 1
-	  if f:
-	    filename = f.filename
-	    self.importConfPackage( f, REQUEST, createIfNotExists)
-	  else:
+          if f:
+            filename = f.filename
+            self.importConfPackage( f, REQUEST, createIfNotExists)
+          else:
             filename = REQUEST['init']
-	    self.importConfPackage( filename, REQUEST, createIfNotExists)
+            self.importConfPackage( filename, REQUEST, createIfNotExists)
           message = self.getZMILangStr('MSG_IMPORTED')%('<i>%s</i>'%filename)
-
+      
       ##### History ####
       elif key == 'History':
         old_active = self.getConfProperty('ZMS.Version.active',0)
@@ -447,25 +453,20 @@ class ConfManager(
           self.setConfProperty('InstalledProducts.pil.hires.thumbnail.max',REQUEST.get('pil_hires_thumbnail_max',0))
           message = self.getZMILangStr('MSG_CHANGED')
       
-      ##### StandardObjects ####
-      elif key == 'StandardObjects':
-        meta_types = []
-        meta_types.extend(self.dGlobalAttrs.keys())
-        meta_types.extend(self.getMetaobjIds())
-        for meta_type in meta_types:
-          self.setConfProperty('%s.enabled'%meta_type,meta_type in REQUEST.get('meta_types',[]))
-        message = self.getZMILangStr('MSG_CHANGED')
-      
-      ##### Log ####
-      elif key == 'Log':
-        if btn == 'Create':
-          obj = zmslog.ZMSLog()
-          self._setObject(obj.id, obj)
-          message = 'Log created'
+      ##### Manager ####
+      elif key == 'Manager':
+        if btn == 'Add':
+          meta_type = REQUEST.get('meta_type','')
+          if meta_type == 'ZMSLog':
+            obj = zmslog.ZMSLog()
+            self._setObject(obj.id, obj)
+            message = 'Added '+meta_type
         elif btn == 'Remove':
-          self.manage_delObjects(ids=self.objectIds(['ZMS Log']))
-          message = 'Log removed'
-
+          meta_types = REQUEST.get('meta_types',[])
+          if len( meta_types) > 0:
+            self.manage_delObjects(ids=self.objectIds(meta_types))
+            message = 'Removed '+', '.join(meta_types)
+      
       # Return with message.
       d = {
       	'lang': lang,
@@ -477,11 +478,11 @@ class ConfManager(
 
 
     """
-    ############################################################################    
+    ############################################################################
     ###
     ###   Configuration-Design
     ###
-    ############################################################################    
+    ############################################################################
     """
 
     ############################################################################
@@ -546,4 +547,48 @@ class ConfManager(
       message = urllib.quote(message)
       return RESPONSE.redirect('manage_customizeDesignForm?lang=%s&manage_tabs_message=%s&cssId=%s'%(lang,message,cssId))
 
+
+    ############################################################################
+    ###
+    ###   Interface IZMSMetamodelProvider: delegate to metaobj_manager
+    ###
+    ############################################################################
+
+    def getMetaobjId(self, name):
+      return self.metaobj_manager.getMetaobjId( name)
+
+    def getMetaobjIds(self, sort=1, excl_ids=[]):
+      return self.metaobj_manager.getMetaobjIds( sort, excl_ids)
+
+    def getMetaobj(self, id):
+      return self.metaobj_manager.getMetaobj( id)
+
+    def getMetaobjAttrIds(self, meta_id, types=[]):
+      return self.metaobj_manager.getMetaobjAttrIds( meta_id, types)
+
+    def getMetaobjAttr(self, meta_id, key):
+      return self.metaobj_manager.getMetaobjAttr( meta_id, key)
+
+    def getMetaobjAttrIdentifierId(self, meta_id):
+      return self.metaobj_manager.getMetaobjAttrIdentifierId( meta_id)
+
+
+    ############################################################################
+    ###
+    ###   Interface IZMSFormatProvider: delegate to format_manager
+    ###
+    ############################################################################
+
+    def getTextFormatDefault(self):
+      return self.format_manager.getTextFormatDefault()
+
+    def getTextFormat(self, id, REQUEST):
+      return self.format_manager.getTextFormat(id, REQUEST)
+
+    def getTextFormats(self, REQUEST):
+      return self.format_manager.getTextFormats(REQUEST)
+
+    def getCharFormats(self):
+      return self.format_manager.getCharFormats()
+
 ################################################################################
