@@ -1,13 +1,11 @@
 ################################################################################
 # _exportable.py
 #
-# $Id: _exportable.py,v 1.9 2004/11/30 20:03:17 zmsdev Exp $
-# $Name:  $
-# $Author: zmsdev $
-# $Revision: 1.9 $
+# $Id:$
+# $Name:$
+# $Author:$
+# $Revision:$
 #
-# Implementation of class Exportable (see below).
-# 
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
 # as published by the Free Software Foundation; either version 2
@@ -31,6 +29,7 @@ import urllib
 import tempfile
 import os
 import re
+import sys
 # Product Imports.
 import _blobfields
 import _fileutil
@@ -42,22 +41,43 @@ import _xmllib
 # ------------------------------------------------------------------------------
 #  _exportable.exportFolder:
 # ------------------------------------------------------------------------------
-def exportFolder(self, root, path, id, REQUEST):
+def exportFolder(self, root, path, id, REQUEST, depth=0):
   if hasattr(root,id):
     folder = getattr(root,id)
-    if folder.meta_type == 'Folder':
-      for ob in folder.objectValues():
-        if ob.meta_type == 'Folder':
-          ob_id = ob.id
-          exportFolder(self, ob,'%s/%s'%(path,id),ob_id,REQUEST)
-        else:
+    for ob in folder.objectValues():
+      if ob.meta_type == 'Folder':
+        ob_id = ob.id
+        exportFolder(self, ob,'%s/%s'%(path,id),ob_id,REQUEST,depth+1)
+      else:
+        try:
+          ob_id = ob.id()
+        except:
+          ob_id = str(ob.id)
+        if ob.meta_type in [ 'DTML Document', 'DTML Method']:
           try:
-            ob_id = ob.id()
+            html = ob.raw
+            html = _globals.dt_html( self, html, REQUEST)
+            html = localHtml( self, ob, html, REQUEST)
+            html = localIndexHtml( self, ob, len(ob.absolute_url().split('/'))-len(root.absolute_url().split('/'))+depth, html, REQUEST)
+            ob = html
           except:
-            ob_id = str(ob.id)
-          if ob.meta_type in [ 'DTML Document', 'DTML Method']:
-            ob = _globals.dt_html(self,ob.raw,REQUEST)
-          _fileutil.exportObj(ob,'%s/%s/%s'%(path,id,ob_id))
+            pass
+        _fileutil.exportObj(ob,'%s/%s/%s'%(path,id,ob_id))
+
+
+# ------------------------------------------------------------------------------
+#  _exportable.exportMetaobjManager:
+# ------------------------------------------------------------------------------
+def exportMetaobjManager(self, root, path):
+  id = 'metaobj_manager'
+  if hasattr(root,id):
+    folder = getattr(root,id)
+    for ob in folder.objectValues(['File']):
+      try:
+        ob_id = ob.id()
+      except:
+        ob_id = str(ob.id)
+      _fileutil.exportObj(ob,'%s/%s/%s'%(path,id,ob_id))
 
 
 # ------------------------------------------------------------------------------
@@ -87,13 +107,32 @@ def rfindDelimiter(s, delimiters=['"',"'"]):
 
 # ------------------------------------------------------------------------------
 #  _exportable.localHtml:
+#
+#  Process encoding.
 # ------------------------------------------------------------------------------
 def localHtml(self, obj, html, REQUEST):
-  
-  # Process relative links to JavaScript.
-  for js in [ 'comlib', 'datelib', 'formlib', 'zmilib', 'styleswitcher']:
-    html = re.sub( '"(.*?)\_js(|\?lang=(.*?))"', '"\\1.js"', html)
-  
+  try:
+    default_charset = 'utf-8'
+    charset = REQUEST.get('ZMS_CHARSET', default_charset)
+    if charset != default_charset:
+      html = unicode( html, default_charset).encode( charset)
+  except ( UnicodeDecodeError, UnicodeEncodeError):
+    _globals.writeException( obj, "[localHtml]")
+    v = str(sys.exc_value)
+    STR_POSITION = ' position '
+    i = v.find(STR_POSITION)
+    if i > 0:
+      v = v[i+len(STR_POSITION):]
+      if v.find('-') > 0:
+        l = int( v[:v.find('-')])
+        h = int( v[v.find('-')+1:v.find(':')])
+      else:
+        l = int( v[:v.find(':')])
+        h = l
+      ln = max( l - 20, 0)
+      hn = min( h + 20, len(html))
+      print ">>>>>",html[ln:hn]
+      print ">>>>>"," "*(l-ln)+"^"*(h-l+1)
   return html
 
 
@@ -105,11 +144,24 @@ def localIndexHtml(self, obj, level, html, REQUEST):
    sRoot = ''
    for i in range(level):
      sRoot = '../%s'%sRoot
-     
-   # Process absolute URLs
+   
+   # Process aliases.
+   doc_url = self.getDocumentElement().absolute_url()
+   url = doc_url
+   url = url[url.find('://')+3:]
+   url = url[url.find('/'):]
+   base_url = doc_url
+   base_url = base_url[ : base_url.find(url)]
+   html = re.sub( '"([^("\')]*?)'+url+'([^("\')]*?)"', '"'+base_url+url+'\\2"', html)
+   
+   # Process absolute URLs.
    s_new = '%s'%sRoot
    s_old = '%s/'%self.absolute_url()
-   html = html.replace( s_old, s_new)
+   if level != 0:
+     html = html.replace( s_old, s_new)
+   if self.getConfProperty('ZMS.pathhandler',0) != 0:
+     s_old = '%s/'%self.getDeclUrl( REQUEST)
+     html = html.replace( s_old, s_new)
    s_old = '%s/'%self.getDocumentElement().absolute_url()
    html = html.replace( s_old, s_new)
    s_old = '%s/'%self.getHome().absolute_url()
@@ -150,10 +202,10 @@ def localIndexHtml(self, obj, level, html, REQUEST):
      html = html.replace( oldTmp, newTmp)
      # Replace 'index' in declarative URLs
      pageexts = ['.html']
-     if 'attr_pageext' in self.getMetadictAttrs():
-       metadictAttr = self.getMetadictAttr('attr_pageext')
-       if metadictAttr.has_key('keys') and len(metadictAttr.get('keys')) > 0:
-         pageexts = metadictAttr.get('keys')
+     if 'attr_pageext' in self.getObjAttrs().keys():
+       obj_attr = self.getObjAttr('attr_pageext')
+       if obj_attr.has_key('keys') and len(obj_attr.get('keys')) > 0:
+         pageexts = obj_attr.get('keys')
      for pageext in pageexts:
        s_new = pageext
        s_old = '/index_%s%s'%(REQUEST['lang'],pageext)
@@ -173,17 +225,17 @@ def localIndexHtml(self, obj, level, html, REQUEST):
 ################################################################################
 ################################################################################
 ###
-###   class Exportable
+###   Class
 ###
 ################################################################################
 ################################################################################
 class Exportable:
 
-    ############################################################################    
+    ############################################################################
     #  Exportable.manage_export:
     #
     #  Exports ZMS-object.
-    ############################################################################    
+    ############################################################################
     def manage_export(self, export_format, lang, REQUEST, RESPONSE):
       """ Exportable.manage_export """
       
@@ -244,11 +296,11 @@ class Exportable:
         RESPONSE.redirect( url)
 
 
-    ############################################################################    
+    ############################################################################
     #  Exportable.pub_export:
     #
     #  Exports ZMS-object.
-    ############################################################################    
+    ############################################################################
     def pub_export(self, export_format, lang, REQUEST, RESPONSE):
       """ Exportable.pub_export """
       self.f_standard_html_request( self, REQUEST)
@@ -262,7 +314,7 @@ class Exportable:
     # --------------------------------------------------------------------------
     def getObjToXml(self):
       xml = []
-      method = getattr( self, 'getObjToXml_%s'%self.meta_id_or_type(), None)
+      method = getattr( self, 'getObjToXml_%s'%self.meta_id, None)
       if method is not None:
         xml.append( method( context=self))
       return ''.join(xml)
@@ -283,7 +335,7 @@ class Exportable:
     # --------------------------------------------------------------------------
     #  Exportable.toXhtml:
     #
-    #  (X)HTNL
+    #  (X)HTML
     # --------------------------------------------------------------------------
     def toXhtml(self, REQUEST, deep=True):
       if _globals.debug( self):
@@ -325,46 +377,31 @@ class Exportable:
     #
     #  Returns list of exported resources (Images, StyleSheets, etc.)
     # --------------------------------------------------------------------------
-    def exportRessources(self, tempfolder, REQUEST, from_content=True, from_zms=False, from_common=False, incl_embedded=False):
+    def exportRessources(self, tempfolder, REQUEST, from_content=True, from_zms=False, from_home=False, incl_embedded=False):
       ressources = []
       
       if from_zms:
         folder = 'misc_/zms'
         for obj_id in self.misc_.zms._d.keys():
           _fileutil.exportObj(self.misc_.zms[obj_id],'%s/%s/%s'%(tempfolder,folder,obj_id))
+        exportMetaobjManager( self, self.getDocumentElement(), tempfolder)
       
-      if from_common:
-        for id in [ 'common', 'instance']:
-          exportFolder( self, self.getHome(), tempfolder, id, REQUEST)
+      if from_home:
+        root = self.getHome()
+        for id in root.objectIds(['Folder']):
+          exportFolder( self, root, tempfolder, id, REQUEST)
       
       if from_content:
-        ressources.extend( self.exportContentRessources( tempfolder, REQUEST, incl_embedded))
+        base_path = tempfolder+'/'
+        if not self.meta_id == 'ZMS' and not REQUEST.get( 'ZMS_FILTER', False):
+          base_path += self.id+'/'
+        ressources.extend( _blobfields.recurse_downloadRessources( self, base_path, REQUEST, incl_embedded))
       
       return ressources
 
 
     # --------------------------------------------------------------------------
-    #  Exportable.exportContentRessources:
-    #
-    #  Returns list of exported resources (Images, StyleSheets, etc.)
-    # --------------------------------------------------------------------------
-    def exportContentRessources(self, tempfolder, REQUEST, incl_embedded):
-      
-      if REQUEST is not None:
-      
-        #-- JavaScript
-        for js in [ 'comlib', 'formlib', 'datelib', 'formlib', 'zmilib', 'styleswitcher']:
-          data = getattr(self,'%s_js'%(js))(self,REQUEST)
-          filename = '%s.js'%js
-          ressource = '%s/%s'%(tempfolder,filename)
-          _fileutil.exportObj( data, ressource)
-      
-      # Download ressources (images & files)
-      return _blobfields.recurse_downloadRessources( self, tempfolder+'/', REQUEST, incl_embedded)
-
-
-    # --------------------------------------------------------------------------
-    #	Exportable.exportExternalResources
+    #  Exportable.exportExternalResources
     # --------------------------------------------------------------------------
     def exportExternalResources(self, obj, html, path, REQUEST):
       domains = []
@@ -386,14 +423,14 @@ class Exportable:
           if (html[ t + 1: t + 4].lower() == 'img' and html[ k - 3: k].lower() == 'src') \
               or (html[ t + 1].lower() == 'a' and html[ k - 4: k].lower() == 'href'):
             l = findDelimiter(html[ d + 1:])
-            url = html[ d + 1: d + l + 1]        
+            url = html[ d + 1: d + l + 1]
             for domain in domains:
               if domain in url:
                 try:
                   _globals.writeLog( self, '[exportExternalResources]: url=%s'%url)
                   s_new = s_old = url
                   for repl in ':/%&?;=':
-                    s_new = s_new.replace(repl, '_')                  
+                    s_new = s_new.replace(repl, '_')
                   # test if extension is a real extension at the end ?
                   # http://host:port/uri.gif?a=x&b=k => http___host_port_uri.gif_a_x_b_k.gif
                   # http://host:port/uri.gif => http__host_port_uri.gif
@@ -419,7 +456,7 @@ class Exportable:
 
 
     # --------------------------------------------------------------------------
-    #	Exportable.recurse_downloadHtmlPages:
+    #  Exportable.recurse_downloadHtmlPages:
     # --------------------------------------------------------------------------
     def recurse_downloadHtmlPages(self, obj, path, lang, REQUEST):
       try:
@@ -428,7 +465,14 @@ class Exportable:
         pass
       
       level = obj.getLevel()
-      dctOp = {'index':'','sitemap':'sitemap','index_print':'print'}
+      dctOp = {}
+      if obj.meta_id.find('ZMSSys') == 0:
+        exportFolder( obj, obj.getParentNode(), path[:path.rfind('/')], obj.id, REQUEST)
+      else:
+        dctOp['index'] = ''
+        dctOp['index_print'] = 'print'
+        dctOp['sitemap'] = 'sitemap'
+      
       for key in dctOp.keys():
         
         # Get html.
@@ -436,17 +480,17 @@ class Exportable:
         REQUEST.set('ZMS_PATH_HANDLER', True)
         
         try:
-
+          
           # Remember others.
           others = copy.copy(REQUEST.other.keys())
-
+          
           root = getattr( obj, '__root__', None)
           if root is not None:
             REQUEST.set('ZMS_PROXY_%s'%root.id,obj)
             html = root.index_html( root, REQUEST)
           else:
             html = obj.index_html( obj, REQUEST)
-
+          
           # Remove new others.
           for rk in REQUEST.other.keys():
             if rk not in others:
@@ -455,6 +499,7 @@ class Exportable:
               except:
                 pass
           
+          # Localize html.
           html = localHtml( self, obj, html, REQUEST)
           
           # Save html to file.
@@ -471,29 +516,28 @@ class Exportable:
               pageext = obj.getPageExt( REQUEST)
             html = localIndexHtml( self, obj, level - self.getLevel(), html, REQUEST)
             filename = '%s/%s_%s%s'%( path, key, lang, pageext)
-
+          
           html = self.exportExternalResources( obj, html, path, REQUEST)
-
+          
           f = open( filename, 'w')
           f.write( html)
           f.close()
           
           # Root folder requires and defaults to "index.html" at most systems.
-          if key == 'index' and \
-             level == 0 and lang == self.getPrimaryLanguage():
+          if key == 'index' and lang == self.getPrimaryLanguage():
             filename = '%s/%s%s'%( path, key, obj.getPageExt( REQUEST))
             f = open(filename,'w')
             f.write(html)
             f.close()
-          
+        
         except:
-          _globals.writeException( self, "[recurse_downloadHtmlPages]: Can't get html '%s'"%key)
+          _globals.writeException( obj, "[recurse_downloadHtmlPages]: Can't get html '%s'"%key)
         
       # Process DTML-methods of meta-objects.
-      for metadictAttrId in self.getMetadictAttrs( obj.meta_type):
+      for metadictAttrId in self.getMetaobjAttrIds( obj.meta_id):
         try:
-          metadictAttr = self.getMetadictAttr( metadictAttrId)
-          if metadictAttr['type'] in self.getMetaobjIds():
+          metadictAttr = self.getMetaobjAttr( obj.meta_id, metadictAttrId)
+          if metadictAttr is not None and metadictAttr['meta_type'] and metadictAttr['type'] in self.getMetaobjIds( sort=0):
             metaObj = self.getMetaobj( metadictAttr['type'])
             if metaObj['type'] == 'ZMSResource':
               for metadictObj in obj.getObjChildren( metadictAttr['id'], REQUEST):
@@ -514,18 +558,18 @@ class Exportable:
 
 
     # --------------------------------------------------------------------------
-    #	Exportable.toZippedHtml:
+    #  Exportable.toZippedHtml:
     # --------------------------------------------------------------------------
     def toZippedHtml(self, REQUEST, get_data=True):
+      REQUEST.set('ZMS_INDEX_HTML',1)
+      REQUEST.set('ZMS_HTML_EXPORT',1)
       
       #-- Create temporary folder.
       tempfolder = tempfile.mktemp()
-      ressources = self.exportRessources( tempfolder, REQUEST, from_zms=self.getLevel()==0, from_common=True)
+      ressources = self.exportRessources( tempfolder, REQUEST, from_zms=self.getLevel()==0, from_home=True)
       
       #-- Download HTML-pages.
       for lang in self.getLangIds():
-        REQUEST.set('ZMS_INDEX_HTML',1)
-        REQUEST.set('ZMS_HTML_EXPORT',1)
         REQUEST.set('lang',lang)
         REQUEST.set('preview',None)
         self.recurse_downloadHtmlPages( self, tempfolder, lang, REQUEST)
@@ -554,7 +598,7 @@ class Exportable:
       xml = self.toXml(REQUEST,incl_embedded)
       
       #-- Write xml-export to file.
-      xmlfilename = _fileutil.getOSPath('%s/%s_%s.xml'%(tempfolder,self.getHome().id,self.meta_type))
+      xmlfilename = _fileutil.getOSPath('%s/%s_%s.xml'%(tempfolder,self.getHome().id,self.meta_id))
       _fileutil.exportObj(xml,xmlfilename)
       
       #-- Get zip-file.
