@@ -2,7 +2,7 @@
 # _fileutil.py
 #
 # $Id: _fileutil.py,v 1.9 2004/11/30 20:03:17 zmsdev Exp $
-# $Name:  $
+# $Name:$
 # $Author: zmsdev $
 # $Revision: 1.9 $
 #
@@ -33,7 +33,6 @@ import os
 import shutil
 import stat
 import tempfile
-import urllib
 import zipfile
 
 
@@ -48,7 +47,7 @@ def importZexp(self, path, filename):
   if src_filename != dst_filename:
     try: 
       os.stat(getOSPath(dst_filename)) 
-    except OSError: 
+    except OSError:
       shutil.copy(src_filename,dst_filename)
   self.manage_importObject(filename)
   remove(dst_filename)
@@ -92,7 +91,7 @@ def getOSPath(path, sep=None, chs=range(32)+[34,60,62,63,127]):
   path = path.replace('/',sep)
   if type( path) is type( ''):
     path = unicode(path, 'latin-1')
-  path = path.encode('ascii', 'replace') # replace => uncodierbare Zeichen durch ? (63) ersetzen 
+  path = path.encode('ascii', 'replace') # replace uncodable characters by ? (63)
   if len( chs) > 0:
     for ch in chs:
       path = path.replace(chr(ch),'')
@@ -182,7 +181,10 @@ def readPath(path, data=True, recursive=True):
             d['content_type']=mt
             d['encoding']=enc
           else:
-            mt, enc = guess_content_type( local_filename)
+            try:
+              mt, enc = guess_content_type( local_filename)
+            except:
+              mt, enc = 'content/unknown', ''
             d['content_type']=mt
             d['encoding']=enc
           fsize = os.path.getsize( local_filename)
@@ -202,14 +204,19 @@ def readPath(path, data=True, recursive=True):
 def readFile(filename, mode='b', threshold=2 << 16):
   size = os.path.getsize( filename)
   if size < threshold or -1 == threshold:
+    f = None
     try:
       f = open( filename, 'r'+mode)
       data = f.read()
     finally:
-      f.close()
+      if f is not None:
+        f.close()
   else:
     data = filestream_iterator( filename, 'r'+mode)
-  mt, enc  = guess_content_type( filename, data)
+  try:
+    mt, enc  = guess_content_type( filename, data)
+  except:
+    mt, enc = 'content/unknown', ''
   return data, mt, enc, size
 
 
@@ -302,9 +309,9 @@ def exportObj(obj, filename, filetype='b'):
   filepath = getFilePath(filename)
   mkDir(filepath)
         
-  #-- Get object data.   
+  #-- Get object data.
   data = None
-  try: # ImageFile        
+  try: # ImageFile
     f = open(obj.path,'r%s'%filetype)
     data = f.read()
     f.close()
@@ -369,22 +376,20 @@ def readDir(path):
   return map(lambda ob: ob[1],obs)
 
 
-"""
 ################################################################################
 ###
-###  P I L   I M A G I N G 
+###  PIL (Python Imaging Library)
 ###
 ################################################################################
-"""
 
 # ------------------------------------------------------------------------------
 #  _fileutil.createThumbnail:
 #
 #  Creates thumbnail of given image.
-#  IN:  img
-#  OUT: thumb
+#  @param img
+#  @return thumb
 # ------------------------------------------------------------------------------
-def createThumbnail(img, maxdim=100):
+def createThumbnail(img, maxdim=100, qual=75):
   
   # Save image in temp-folder.
   tempfolder = tempfile.mktemp()
@@ -392,7 +397,7 @@ def createThumbnail(img, maxdim=100):
   exportObj(img,filename)
   
   # Call PIL to generate thumbnail.
-  command = 'python ' + getOSPath(package_home(globals())+'/conf/pil/img_conv.py') + ' ' + str(maxdim) + ' ' + filename
+  command = 'python ' + getOSPath(package_home(globals())+'/conf/pil/img_conv.py') + ' ' + str(maxdim) + ' ' + filename + ' ' + str(qual)
   os.system(command)
   
   # Read thumbnail from file-system.
@@ -414,13 +419,109 @@ def createThumbnail(img, maxdim=100):
   return thumb
 
 
-"""
+# ------------------------------------------------------------------------------
+#  _fileutil.pil_img_resize:
+# ------------------------------------------------------------------------------
+def pil_img_resize(img, size, mode='resize', sffx='none', qual=75):
+  
+  # Save image in temp-folder.
+  tempfolder = tempfile.mktemp()
+  filename = getOSPath('%s/%s'%(tempfolder,img.filename))
+  exportObj(img,filename)
+  
+  # Call PIL to generate resized image.
+  command = 'python ' + getOSPath(package_home(globals())+'/conf/pil/img_resize.py') + ' ' + str(size[0]) + ' ' + str(size[1]) + ' ' + filename + ' ' + mode + ' ' + str(qual)
+  os.system(command)
+  
+  # Read resized image from file-system.
+
+  if sffx == 'none':
+      f = open(filename,'rb')
+      resized_data = f.read()
+      resized_filename = extractFilename(filename)
+      resized = {'data':resized_data,'filename':resized_filename}
+      f.close()
+  else:
+      # Read thumbnail from file-system.
+      lang_sffx = '_' + img.lang
+      thumb_sffx = '_thumbnail'
+      newfilename = filename[:-(len(extractFileExt(filename))+1)]
+      f = open(filename,'rb')
+      thumb_data = f.read()
+      thumb_filename = filename
+      if len(thumb_filename)>len(lang_sffx) and thumb_filename[-len(lang_sffx):]==lang_sffx: thumb_filename = thumb_filename[:-len(lang_sffx)]
+      thumb_filename = extractFilename(newfilename + thumb_sffx + lang_sffx + '.jpg')
+      resized = {'data':thumb_data,'filename':thumb_filename}
+      f.close()
+  
+  # Remove temp-folder and images.
+  remove(tempfolder,deep=1)
+  
+  # Returns resized image.
+  return resized
+
+
+# ------------------------------------------------------------------------------
+#  _fileutil.pil_img_crop:
+# ------------------------------------------------------------------------------
+def pil_img_crop(img, box, qual=75):
+  
+  # Save image in temp-folder.
+  tempfolder = tempfile.mktemp()
+  filename = getOSPath('%s/%s'%(tempfolder,img.filename))
+  exportObj(img,filename)
+  
+  # Call PIL to generate resized image.
+  command = 'python ' + getOSPath(package_home(globals())+'/conf/pil/img_crop.py') + ' ' + str(box[0]) + ' ' + str(box[1]) + ' ' + str(box[2]) + ' ' + str(box[3]) + ' ' + filename + ' ' + str(qual)
+  os.system(command)
+  
+  # Read resized image from file-system.
+  f = open(filename,'rb')
+  cropped_data = f.read()
+  cropped_filename = extractFilename(filename)
+  cropped = {'data':cropped_data,'filename':cropped_filename}
+  f.close()
+  
+  # Remove temp-folder and images.
+  remove(tempfolder,deep=1)
+  
+  # Returns cropped image.
+  return cropped
+
+
+# ------------------------------------------------------------------------------
+#  _fileutil.pil_img_rotate:
+# ------------------------------------------------------------------------------
+def pil_img_rotate(img, direction, qual=75):
+
+    # Save image in temp-folder.
+    tempfolder = tempfile.mktemp()
+    filename = getOSPath('%s/%s'%(tempfolder,img.filename))
+    exportObj(img,filename)
+
+    # Call PIL to generate resized image.
+    command = 'python ' + getOSPath(package_home(globals())+'/conf/pil/img_rotate.py') + ' ' + str(direction) + ' ' + filename + ' ' + str(qual)
+    os.system(command)
+
+    # Read resized image from file-system.
+    f = open(filename,'rb')
+    rot_data = f.read()
+    rot_filename = extractFilename(filename)
+    rotated = {'data':rot_data,'filename':rot_filename}
+    f.close()
+
+    # Remove temp-folder and images.
+    remove(tempfolder,deep=1)
+
+    # Returns cropped image.
+    return rotated
+
+
 ################################################################################
 ###
-###  Z I P   A R C H I V E 
+###  ZIP
 ###
 ################################################################################
-"""
 
 # ------------------------------------------------------------------------------
 #  _fileutil.getZipArchive:
