@@ -2,12 +2,10 @@
 # _filtermanager.py
 #
 # $Id: _filtermanager.py,v 1.8 2004/11/24 21:02:52 zmsdev Exp $
-# $Name:  $
+# $Name:$
 # $Author: zmsdev $
 # $Revision: 1.8 $
 #
-# Implementation of class FilterManager (see below).
-#
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
 # as published by the Free Software Foundation; either version 2
@@ -437,10 +435,10 @@ def processCommand(self, filename, command):
       _globals.writeLog( self, '[processCommand]: rename %s to %s'%( tmpoutfilename, outfilename))
     try:
       os.remove( outfilename)
-    except OSError: 
+    except OSError:
       pass
     os.rename( tmpoutfilename, outfilename)
-  except OSError: 
+  except OSError:
     outfilename = infilename
   # Remove input file if it is the result of a transformation of output file.
   if outfilename != infilename:
@@ -510,19 +508,21 @@ def importFilter(self, filename, id, REQUEST):
 #  _filtermanager.exportFilter:
 # ------------------------------------------------------------------------------
 def exportFilter(self, id, REQUEST):
+  # Set environment variables.
+  instance_home = INSTANCE_HOME
+  software_home = os.path.join(SOFTWARE_HOME, '..%s..' % os.sep)
+  software_home = os.path.normpath(software_home)  
+  REQUEST.set( 'ZMS_FILTER', True)
+  REQUEST.set( 'ZMS_FILTER_SOFTWARE_HOME', software_home)
+  REQUEST.set( 'ZMS_FILTER_INSTANCE_HOME', instance_home)
+  REQUEST.set( 'ZMS_FILTER_PACKAGE_HOME', _fileutil.getOSPath(package_home(globals())))
+  # Set local variables.
   ob_filter = self.getFilter(id)
   ob_filter_format = ob_filter.get('format','')
   incl_embedded = ob_filter_format == 'XML_incl_embedded'
   # Create temporary folder.
   tempfolder = tempfile.mktemp()
-  ressources = self.exportRessources( tempfolder, REQUEST, from_content=True, from_zms=ob_filter_format=='XHTML', from_common=ob_filter_format=='XHTML', incl_embedded=incl_embedded)
-  # Set environment variables.
-  instance_home = INSTANCE_HOME
-  software_home = os.path.join(SOFTWARE_HOME, '..%s..' % os.sep)
-  software_home = os.path.normpath(software_home)  
-  REQUEST.set( 'ZMS_FILTER_SOFTWARE_HOME', software_home)
-  REQUEST.set( 'ZMS_FILTER_INSTANCE_HOME', instance_home)
-  REQUEST.set( 'ZMS_FILTER_PACKAGE_HOME', _fileutil.getOSPath(package_home(globals())))
+  ressources = self.exportRessources( tempfolder, REQUEST, from_content=True, from_zms=ob_filter_format=='XHTML', from_home=ob_filter_format=='XHTML', incl_embedded=incl_embedded)
   # Export data to file.
   if ob_filter_format == 'export':
     outfilename = _fileutil.getOSPath('%s/INDEX0'%tempfolder)
@@ -576,7 +576,7 @@ class FilterManager:
 
     # Management Interface.
     # ---------------------
-    manage_customizeFilterForm = HTMLFile('dtml/zms/manage_customizefilterform', globals())
+    manage_customizeFilterForm = HTMLFile('dtml/ZMS/manage_customizefilterform', globals())
 
 
     # --------------------------------------------------------------------------
