@@ -2,12 +2,10 @@
 # _ftpmanager.py
 #
 # $Id: _ftpmanager.py,v 1.3 2004/11/24 21:02:52 zmsdev Exp $
-# $Name:  $
+# $Name:$
 # $Author: zmsdev $
 # $Revision: 1.3 $
 #
-# Implementation of class _ftpmanager.FtpManager (see below).
-# 
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
 # as published by the Free Software Foundation; either version 2
@@ -93,7 +91,7 @@ def ftpToProvider(self, lang, REQUEST, RESPONSE):
     
     # Create temporary local-folder.
     tempfolder = tempfile.mktemp()
-    ressources = self.exportRessources( tempfolder, REQUEST, from_content=self.getLevel() == 0, from_zms=True, from_common=True)
+    ressources = self.exportRessources( tempfolder, REQUEST, from_content=self.getLevel() == 0, from_zms=True, from_home=True)
     
     # Download HTML-pages (to temporary local-folder).
     for lang in self.getLangIds():
@@ -136,7 +134,7 @@ class FtpManager:
 
     # Management Interface.
     # ---------------------
-    manage_importexportFtp = HTMLFile('dtml/zmscontainerobject/manage_importexportftp', globals()) 
+    manage_importexportFtp = HTMLFile('dtml/ZMSContainerObject/manage_importexportftp', globals()) 
 
 
     # --------------------------------------------------------------------------
