@@ -2,12 +2,10 @@
 # _globals.py
 #
 # $Id: _globals.py,v 1.12 2004/11/24 21:02:52 zmsdev Exp $
-# $Name:  $
+# $Name:$
 # $Author: zmsdev $
 # $Revision: 1.12 $
 #
-# Implementation of Global Functions.
-# 
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
 # as published by the Free Software Foundation; either version 2
@@ -24,10 +22,8 @@
 ################################################################################
 
 # Imports.
-from Acquisition import Implicit
 from App.Common import package_home
 from DateTime.DateTime import DateTime
-from Globals import Persistent
 from httplib import HTTP
 from types import StringTypes
 from traceback import format_exception
@@ -44,9 +40,6 @@ import urllib
 
 """ Globals. """
 
-# DEPRECATED!
-class MyDict(Persistent,Implicit): pass    
-
 # Constants.
 # ----------
 
@@ -55,7 +48,7 @@ DT_UNKNOWN = 0
 DT_BOOLEAN = 1
 DT_DATE = 2
 DT_DATETIME = 3
-DT_DICTIONARY = 4
+DT_DICT = 4
 DT_FILE = 5
 DT_FLOAT = 6
 DT_IMAGE = 7
@@ -99,6 +92,17 @@ dtMapping = [
   [ 'color',''],
 ]
 
+# German umlaute.
+umlautMapping = {
+	unicode('\xe4','latin-1').encode('utf-8'):'ae',
+	unicode('\xf6','latin-1').encode('utf-8'):'oe',
+	unicode('\xfc','latin-1').encode('utf-8'):'ue',
+	unicode('\xc4','latin-1').encode('utf-8'):'Ae',
+	unicode('\xd6','latin-1').encode('utf-8'):'Oe',
+	unicode('\xdc','latin-1').encode('utf-8'):'Ue',
+	unicode('\xdf','latin-1').encode('utf-8'):'ss',
+}
+
 
 # ------------------------------------------------------------------------------
 #  _globals.datatype_key:
@@ -120,15 +124,6 @@ def format_sort_id(i_sort_id):
   return sort_id
 
 
-# ------------------------------------------------------------------------------
-#  _globals.filename_quote:
-# ------------------------------------------------------------------------------
-def filename_quote(filename):
-  for ch in ['+','%',' ','!','?','#','"']:
-    filename = filename.replace(ch,'')
-  return filename
-
-
 # ------------------------------------------------------------------------------
 #  _globals.html_quote:
 # ------------------------------------------------------------------------------
@@ -136,6 +131,20 @@ def html_quote(v, name='(Unknown name)', md={}):
   return cgi.escape(str(v), 1)
 
 
+# ------------------------------------------------------------------------------
+#  _globals.bin2hex:
+# ------------------------------------------------------------------------------
+def bin2hex(m):
+  return ''.join(map(lambda x: hex(ord(x)/16)[-1]+hex(ord(x)%16)[-1],m))
+
+
+# ------------------------------------------------------------------------------
+#  _globals.hex2bin:
+# ------------------------------------------------------------------------------
+def hex2bin(m):
+  return ''.join(map(lambda x: chr(16*int('0x%s'%m[x*2],0)+int('0x%s'%m[x*2+1],0)),range(len(m)/2)))
+
+
 # ------------------------------------------------------------------------------
 #  _globals.unencode:
 #
@@ -162,24 +171,18 @@ def unencode( p, enc='utf-8'):
 # ------------------------------------------------------------------------------
 #  _globals.id_quote:
 # ------------------------------------------------------------------------------
-def id_quote(s):
-  mapping = {
-  	'\x20':'_',
-  	'-':'_',
-  	'/':'_',
-  	unicode('\xe4','latin-1').encode('utf-8'):'ae',
-  	unicode('\xf6','latin-1').encode('utf-8'):'oe',
-  	unicode('\xfc','latin-1').encode('utf-8'):'ue',
-  	unicode('\xc4','latin-1').encode('utf-8'):'Ae',
-  	unicode('\xd6','latin-1').encode('utf-8'):'Oe',
-  	unicode('\xdc','latin-1').encode('utf-8'):'Ue',
-  	unicode('\xdf','latin-1').encode('utf-8'):'ss',
-  }
+def id_quote(s, mapping={
+		'\x20':'_',
+		'-':'_',
+		'/':'_',
+}):
+  map( lambda x: operator.setitem( mapping, x, umlautMapping[x]), umlautMapping.keys())
   for key in mapping.keys():
     s = s.replace(key,mapping[key])
-  valid = [ord('_')] + range(ord('0'),ord('9')+1) + range(ord('A'),ord('Z')+1) + range(ord('a'),ord('z')+1)
+  valid = map( lambda x: ord(x[0]), mapping.values()) + [ord('_')] + range(ord('0'),ord('9')+1) + range(ord('A'),ord('Z')+1) + range(ord('a'),ord('z')+1)
   s = filter( lambda x: ord(x) in valid, s)
-  while len(s) > 0 and s[0] == '_': s = s[1:]
+  while len(s) > 0 and s[0] == '_':
+    s = s[1:]
   s = s.lower()
   return s
 
@@ -302,11 +305,18 @@ def qs_append(qs, p, v):
 
 # ------------------------------------------------------------------------------
 #  _globals.nvl:
-#
-#  Returns its first argument if it is not None, otherwise it returns its second argument.
 # ------------------------------------------------------------------------------
-def nvl(a1, a2):
-  if a1 is not None:
+def nvl(a1, a2, n=None):
+  """
+  Returns its first argument if it is not equal to third argument (None), 
+  otherwise it returns its second argument.
+  @param a1: 1st argument
+  @type a1: C{any}
+  @param a2: 2nd argument
+  @type a2: C{any}
+  @rtype: C{any}
+  """
+  if (n is None and a1 is not None) or (type(n) is str and a1 != n) or (type(n) is list and a1 not in n):
     return a1
   else:
     return a2
@@ -361,11 +371,18 @@ def getPageWithElements(self, REQUEST):
 #  _globals.unescape:
 # ------------------------------------------------------------------------------
 def unescape(s):
-  while 1:
+  while True:
     i = s.find('%')
     if i < 0: break
-    old = s[i:i+3]
-    new = chr(int('0x'+s[i+1:i+3],0))
+    if s[i+1] == 'u':
+      old = s[i:i+6]
+      if old == '%u21B5':
+        new = '&crarr;'
+      else:
+        new = ''
+    else:
+      old = s[i:i+3]
+      new = chr(int('0x'+s[i+1:i+3],0))
     s = s.replace(old,new)
   return s
 
@@ -391,7 +408,8 @@ def http_import(self, url, method='GET', auth=None, parse_qs=0):
 
   # Remove HTTP-Prefix.
   http_prefix = 'http://'
-  url = url[ len( http_prefix):]
+  if url.startswith( http_prefix):
+    url = url[ len( http_prefix):]
 
   # Get Query-String.
   qs = ''
@@ -399,7 +417,7 @@ def http_import(self, url, method='GET', auth=None, parse_qs=0):
   if i > 0:
     qs = url[i+1:]
     url = url[:i]
-    
+  
   # Get Host and Port.
   i = url.find('/')
   if i > 0:
@@ -409,16 +427,15 @@ def http_import(self, url, method='GET', auth=None, parse_qs=0):
     host = url
     url = '/'
   i = host.find(':',max(0,host.find('@')))
+  port = 80
   if i > 0:
     port = int(host[i+1:])
     host = host[:i]
-  else:
-    port = 80
-     
+  
   # Open HTTP connection.
   writeBlock( self, "[http_import.%s]: %s:%i%s?%s"%(method,host,port,url,qs))
   req = HTTP(host,port)
-        
+  
   # Set request-headers.
   if method.upper() == 'GET':
     if len( qs) > 0:
@@ -434,11 +451,11 @@ def http_import(self, url, method='GET', auth=None, parse_qs=0):
     authtobasic(auth,req)
     req.putheader('Accept', '*/*')
     req.putheader('Content-type', 'application/x-www-form-urlencoded')
-    req.putheader('Content-length', '%d' % len(qs))            
+    req.putheader('Content-length', '%d' % len(qs))
     req.endheaders()
     # Send query string
-    req.send(qs)  
-        
+    req.send(qs)
+  
   # Send request.
   reply_code, message, headers = req.getreply()
   
@@ -461,7 +478,10 @@ def http_import(self, url, method='GET', auth=None, parse_qs=0):
         # return string
         rtn = content
     else:
+      # return string
       rtn = content
+      if port != 80:
+        rtn = rtn.replace( '%s%s/'%(http_prefix,host), '%s%s:%i/'%(http_prefix,host,port))
     return rtn
   else:
     result = '['+str(reply_code)+']: '+str(message)
@@ -527,7 +547,7 @@ def writeLog(self, info):
     zms_log = getattr( self, 'zms_log', None)
     if 'DEBUG' in zms_log.logged_entries:
       severity = logging.DEBUG
-      info = "[%s@%s]"%(self.meta_type,self.absolute_url()[len(self.REQUEST['SERVER_URL']):]) + info
+      info = "[%s@%s]"%(self.meta_id,self.absolute_url()[len(self.REQUEST['SERVER_URL']):]) + info
       zms_log.LOG( severity, info)
   except:
     pass
@@ -540,7 +560,7 @@ def writeBlock(self, info):
     zms_log = getattr( self, 'zms_log', None)
     if 'INFO' in zms_log.logged_entries:
       severity = logging.INFO
-      info = "[%s@%s]"%(self.meta_type,self.absolute_url()[len(self.REQUEST['SERVER_URL']):]) + info
+      info = "[%s@%s]"%(self.meta_id,self.absolute_url()[len(self.REQUEST['SERVER_URL']):]) + info
       zms_log.LOG( severity, info)
   except:
     pass
@@ -561,7 +581,7 @@ def writeException(self, etc=''):
     if etc: etc += '\n'
     severity = logging.ERROR
     info = etc+''.join(format_exception(t, v, tb))
-    info = "[%s@%s]"%(self.meta_type,self.absolute_url()[len(self.REQUEST['SERVER_URL']):]) + info
+    info = "[%s@%s]"%(self.meta_id,self.absolute_url()[len(self.REQUEST['SERVER_URL']):]) + info
     zms_log = getattr( self, 'zms_log', None)
     if 'ERROR' in zms_log.logged_entries:
       zms_log.LOG( severity, info)
@@ -680,7 +700,7 @@ def compareDate(t0, t1, accuracy_time=1):
 # ------------------------------------------------------------------------------
 def parseLangFmtDate(s):
   value = None
-  for fmt in ['%d.%m.%Y %H:%M:%S','%Y/%m/%d %H:%M:%S','%H:%M:%S']:
+  for fmt in ['%d.%m.%Y %H:%M:%S','%Y/%m/%d %H:%M:%S','%Y-%m-%d %H:%M:%S','%H:%M:%S']:
     if value is None:
       try:
         if isinstance(s,DateTime):
@@ -688,21 +708,32 @@ def parseLangFmtDate(s):
         if s is not None and len(str(s)) > 0:
           dctTime = {'Y':0,'m':0,'d':0,'H':0,'M':0,'S':0}
           v = str(s)
-          while 1:
+          while True:
             i = fmt.find('%')+1
-            if i == 0: break
+            if i == 0:
+              break
             dirkey = fmt[i]
             fmt = fmt[i+1:]
             j = strip_int(v)
-            if j == 0: break
+            if j == 0:
+              break
             dirval = int(v[:j])
             v = v[j:]
             dctTime[dirkey] = dirval
-            if len(v) == 0: break
-            if fmt[0] != v[0]: 
+            if len(v) == 0:
+              break
+            if fmt[0] != v[0]:
               raise "Exception"
             fmt = fmt[1:]
             v = v[1:]
+          if dctTime['Y'] + dctTime['m'] + dctTime['d'] == 0:
+            dctTime['Y'] = 1980
+            dctTime['m'] = 1
+            dctTime['d'] = 1
+          if dctTime['Y'] in range( 0, 50): 
+            dctTime['Y'] = dctTime['Y'] + 2000 
+          if dctTime['Y'] in range( 50, 100): 
+            dctTime['Y'] = dctTime['Y'] + 1900 
           if len(v.strip())>0 or \
              (dctTime['Y']+dctTime['m']+dctTime['d']!=0 and dctTime['Y']-1 not in range(1900,2100)) or \
              (dctTime['Y']+dctTime['m']+dctTime['d']!=0 and dctTime['m']-1 not in range(12)) or \
@@ -747,7 +778,8 @@ class MySectionizer:
     #
     #  Constructor.
     # --------------------------------------------------------------------------
-    def __init__(self):
+    def __init__(self, levelnfc='0'):
+      self.levelnfc = levelnfc
       self.sections = []
 
     # --------------------------------------------------------------------------
@@ -758,7 +790,12 @@ class MySectionizer:
     def __str__(self):
       s = ''
       for i in range(len(self.sections)):
-        s += '%i.'%self.sections[i]
+        if self.levelnfc == '0':
+          s += str(self.sections[i]) + '.'
+        elif self.levelnfc == '1':
+          s += chr(self.sections[i] - 1 + ord('A')) + '.'
+        elif self.levelnfc == '2':
+          s += chr(self.sections[i] - 1 + ord('a')) + '.'
       return s
 
     # --------------------------------------------------------------------------
@@ -767,7 +804,7 @@ class MySectionizer:
     #  Creates and returns a copy of this object.
     # --------------------------------------------------------------------------
     def clone(self):
-      ob = MySectionizer()
+      ob = MySectionizer(self.levelnfc)
       ob.sections = copy.deepcopy(self.sections)
       return ob
 
