@@ -2,7 +2,7 @@
 # _multilangmanager.py
 #
 # $Id: _multilangmanager.py,v 1.9 2004/11/30 20:03:17 zmsdev Exp $
-# $Name:  $
+# $Name:$
 # $Author: zmsdev $
 # $Revision: 1.9 $
 #
@@ -37,8 +37,9 @@ def _importXml(self, item, zms_system=0, createIfNotExists=1):
     key = item['key']
     lang_dict = self.get_lang_dict()
     lang_dict[key] = {}
-    for s_lang in self.getLangIds():
-      lang_dict[key][s_lang] = item[s_lang]
+    for langId in self.getLangIds():
+      if item.has_key(langId):
+        lang_dict[key][langId] = item[langId]
     self.setConfProperty('ZMS.custom.langs.dict',lang_dict.copy())
 
 def importXml(self, xml, REQUEST=None, zms_system=0, createIfNotExists=1):
@@ -157,11 +158,11 @@ class MultiLanguageManager:
 
     # Management Interface.
     # ---------------------
-    manage_customizeLanguagesForm = HTMLFile('dtml/zms/manage_customizelanguagesform', globals()) 
+    manage_customizeLanguagesForm = HTMLFile('dtml/ZMS/manage_customizelanguagesform', globals()) 
 
     """
     ############################################################################
-    #	P R I M A R Y   L A N G U A G E
+    #  PRIMARY LANGUAGE
     ############################################################################
     """
 
@@ -376,9 +377,6 @@ class MultiLanguageManager:
       
       #-- Set/Add Standard DTML-Methods.
       self.setLangMethods(lang)
-      
-      #-- Recreate catalog.
-      self.recreateCatalog()
 
 
     # --------------------------------------------------------------------------
@@ -388,10 +386,10 @@ class MultiLanguageManager:
     # --------------------------------------------------------------------------
     def setLangMethods(self, lang):
       pageexts = ['.html']
-      if 'attr_pageext' in self.getMetadictAttrs():
-        metadictAttr = self.getMetadictAttr('attr_pageext')
-        if metadictAttr.has_key('keys') and len(metadictAttr.get('keys')) > 0:
-          pageexts = metadictAttr.get('keys')
+      if 'attr_pageext' in self.getObjAttrs().keys():
+        obj_attr = self.getObjAttr('attr_pageext')
+        if obj_attr.has_key('keys') and len(obj_attr.get('keys')) > 0:
+          pageexts = obj_attr.get('keys')
       for pageext in pageexts:
         setMethod(self,'index_%s%s'%(lang,pageext),'<dtml-var index_html>')
         setMethod(self,'sitemap_%s%s'%(lang,pageext),'<dtml-call "REQUEST.set(\'op\',\'sitemap\')"><dtml-var index_html>')
@@ -416,17 +414,14 @@ class MultiLanguageManager:
       
       #-- Delete Standard DTML-Methods.
       pageexts = ['.html']
-      if 'attr_pageext' in self.getMetadictAttrs():
-        metadictAttr = self.getMetadictAttr('attr_pageext')
-	pageexts = metadictAttr.get('keys',pageexts)
+      if 'attr_pageext' in self.getObjAttrs().keys():
+        obj_attr = self.getObjAttr('attr_pageext')
+        pageexts = obj_attr.get('keys',pageexts)
       for pageext in pageexts:
         delMethod(self,'index_%s%s'%(lang,pageext))
         delMethod(self,'sitemap_%s%s'%(lang,pageext))
         delMethod(self,'index_print_%s%s'%(lang,pageext))
         delMethod(self,'search_%s%s'%(lang,pageext))
-      
-      #-- Recreate catalog.
-      self.recreateCatalog()
 
 
     ############################################################################
@@ -435,39 +430,40 @@ class MultiLanguageManager:
     #  Change languages.
     ############################################################################
     def manage_changeLanguages(self, lang, REQUEST, RESPONSE):
-        """ MultiLanguageManager.manage_changeLanguages """
-        
-        # Delete.
-        # -------
-        if REQUEST['btn'] == self.getZMILangStr('BTN_DELETE'):
-          ids = REQUEST.get('ids',[])
-          for id in ids:
-            self.delLanguage(id) 
-        
-        # Change.
-        # -------
-        elif REQUEST['btn'] == self.getZMILangStr('BTN_CHANGE'):
-          for id in self.getLangIds():
-            newLabel = REQUEST.get('%s_label'%id).strip()
-            newParent = REQUEST.get('%s_parent'%id).strip()
-            newManage = REQUEST.get('%s_manage'%id).strip()
-            self.setLanguage(id, newLabel, newParent, newManage)
-        
-        # Insert.
-        # -------
-        elif REQUEST['btn'] == self.getZMILangStr('BTN_INSERT'):
-          id = REQUEST['language_id']
-          newLabel = REQUEST.get('language_label').strip()
-          if len(self.getLangIds()) == 0:
-            newParent = ''
-          else:
-            newParent = REQUEST.get('language_parent').strip()
-          newManage = REQUEST.get('language_manage').strip()
+      """ MultiLanguageManager.manage_changeLanguages """
+      
+      # Delete.
+      # -------
+      if REQUEST['btn'] == self.getZMILangStr('BTN_DELETE'):
+        ids = REQUEST.get('ids',[])
+        for id in ids:
+          self.delLanguage(id) 
+      
+      # Change.
+      # -------
+      elif REQUEST['btn'] == self.getZMILangStr('BTN_CHANGE'):
+        self.initLangStr()
+        for id in self.getLangIds():
+          newLabel = REQUEST.get('%s_label'%id).strip()
+          newParent = REQUEST.get('%s_parent'%id).strip()
+          newManage = REQUEST.get('%s_manage'%id).strip()
           self.setLanguage(id, newLabel, newParent, newManage)
-        
-        # Return with message.
-        message = urllib.quote(self.getZMILangStr('MSG_CHANGED'))
-        return RESPONSE.redirect('manage_customizeLanguagesForm?lang=%s&manage_tabs_message=%s'%(lang,message))
+      
+      # Insert.
+      # -------
+      elif REQUEST['btn'] == self.getZMILangStr('BTN_INSERT'):
+        id = REQUEST['language_id']
+        newLabel = REQUEST.get('language_label').strip()
+        if len(self.getLangIds()) == 0:
+          newParent = ''
+        else:
+          newParent = REQUEST.get('language_parent').strip()
+        newManage = REQUEST.get('language_manage').strip()
+        self.setLanguage(id, newLabel, newParent, newManage)
+      
+      # Return with message.
+      message = urllib.quote(self.getZMILangStr('MSG_CHANGED'))
+      return RESPONSE.redirect('manage_customizeLanguagesForm?lang=%s&manage_tabs_message=%s'%(lang,message))
 
 
     # --------------------------------------------------------------------------
@@ -479,17 +475,19 @@ class MultiLanguageManager:
       d = {}
       portalMaster = self.getPortalMaster()
       if portalMaster is not None:
-        masterLangDict = portalMaster.get_lang_dict()
-        for key in masterLangDict.keys():
-          d[key] = masterLangDict[key].copy()
-          d[key]['acquired'] = self.concat_list(d[key].get('acquired',[]),d[key].keys())
-      langDict = self.getConfProperty('ZMS.custom.langs.dict',{})
-      for key in langDict.keys():
+        lang_dict = portalMaster.get_lang_dict()
+        for key in lang_dict.keys():
+          d[key] = lang_dict[key].copy()
+          lang_ids = lang_dict[key].keys()
+          d[key]['acquired'] = self.concat_list(d[key].get('acquired',[]),lang_ids)
+      lang_dict = self.getConfProperty('ZMS.custom.langs.dict',{})
+      for key in lang_dict.keys():
         if d.has_key(key):
-          for langKey in langDict[key].keys():
-            d[key][langKey] = langDict[key][langKey]
+          lang_ids = lang_dict[key].keys()
+          for lang_id in lang_ids:
+            d[key][lang_id] = lang_dict[key][lang_id]
         else:
-          d[key] = langDict[key].copy()
+          d[key] = lang_dict[key].copy()
       return d
 
 
@@ -535,11 +533,11 @@ class MultiLanguageManager:
           d = self.get_lang_dict()
           lang_dict = {}
           for key in d.keys():
-            lang_dict[key] = {}
-            for s_lang in self.getLangIds():
-              enabled = s_lang not in d[key].get('acquired',[])
-	      if enabled:
-                lang_dict[key][s_lang] = REQUEST['%s_value_%s'%(key,s_lang)].strip()
+            for lang_id in self.getLangIds():
+              lang_dict[key] = lang_dict.get(key,{})
+              enabled = lang_id not in d[key].get('acquired',[])
+              if enabled:
+                lang_dict[key][lang_id] = REQUEST['%s_value_%s'%(key,lang_id)].strip()
           self.setConfProperty('ZMS.custom.langs.dict',lang_dict.copy())
         
         # Insert.
@@ -549,8 +547,8 @@ class MultiLanguageManager:
           key = REQUEST['%s_key'%(id)].strip()
           lang_dict = self.get_lang_dict()
           lang_dict[key] = {}
-          for s_lang in self.getLangIds():
-            lang_dict[key][s_lang] = REQUEST['%s_value_%s'%(id,s_lang)].strip()
+          for lang_id in self.getLangIds():
+            lang_dict[key][lang_id] = REQUEST['%s_value_%s'%(id,lang_id)].strip()
           self.setConfProperty('ZMS.custom.langs.dict',lang_dict.copy())
         
         # Export.
@@ -577,13 +575,13 @@ class MultiLanguageManager:
         # -------
         elif REQUEST['btn'] == self.getZMILangStr('BTN_IMPORT'):
           f = REQUEST['file']
-	  if f:
-	    filename = f.filename
+          if f:
+            filename = f.filename
             importXml(self,xml=f)
-	  else:
+          else:
             filename = REQUEST['init']
-	    createIfNotExists = 1
-	    self.importConf(filename, REQUEST, createIfNotExists)
+            createIfNotExists = 1
+            self.importConf(filename, REQUEST, createIfNotExists)
           message = self.getZMILangStr('MSG_IMPORTED')%('<i>%s</i>'%filename)
         
         # Return with message.
