@@ -2,12 +2,10 @@
 # _sequence.py
 #
 # $Id: _sequence.py,v 1.1 2003/08/31 13:29:23 zmsdev Exp $
-# $Name:  $
+# $Name:$
 # $Author: zmsdev $
 # $Revision: 1.1 $
 #
-# Implementation of class Sequence (see below).
-# 
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
 # as published by the Free Software Foundation; either version 2
@@ -24,83 +22,70 @@
 ################################################################################
 
 # Imports.
-from Globals import HTMLFile, Persistent   
-import OFS.SimpleItem
-import Acquisition
-import AccessControl.Role
+from Globals import HTMLFile
 import urllib
-
-
-################################################################################
-################################################################################
-###   
-###   C o n s t r u c t o r ( s )
-###   
-################################################################################
-################################################################################
-
-def manage_addSequence(self, startvalue=0, REQUEST=None, RESPONSE=None):
-  """ manage_addSequence """
-  
-  # Create sequence.
-  obj = Sequence(startvalue)
-  self._setObject(obj.id, obj)
-  
-  # Return.
-  if RESPONSE is not None:
-    RESPONSE.redirect('%s/manage_main'%self.absolute_url())
-
+# Product Imports.
+import ZMSItem
 
 
 ################################################################################
 ################################################################################
 ###
-###   C l a s s   S e q u e n c e 
+###   Class
 ###
 ################################################################################
 ################################################################################
-class Sequence(
-    OFS.SimpleItem.Item,
-    Persistent,
-    Acquisition.Implicit,
-    AccessControl.Role.RoleManager
-    ): 
+class Sequence(ZMSItem.ZMSItem):
 
     # Properties.
     # -----------
     meta_type = 'Sequence'
+    icon = "misc_/zms/Sequence.gif"
 
     # Management Options.
     # -------------------
     manage_options = (
-	{'label': 'Properties','action': 'manage_properties'},
-        ) 
+	{'label': 'TAB_CONFIGURATION','action': '../manage_customize'},
+	) 
+
+    # Management Permissions.
+    # -----------------------
+    __administratorPermissions__ = (
+		'manage_changeProperties', 'manage_main',
+		)
+    __ac_permissions__=(
+		('ZMS Administrator', __administratorPermissions__),
+		)
+
+    # Management Interface.
+    # ---------------------
+    manage_main = HTMLFile('dtml/Sequence/manage_main', globals())
 
-    manage_properties = HTMLFile('dtml/acl_sequence/manage_properties', globals())
 
     """
-    ############################################################################    
-    #
-    #   CONSTRUCTOR
-    #
-    ############################################################################    
+    ############################################################################
+    ###
+    ###   Constructor
+    ###
+    ############################################################################
     """
 
     ############################################################################
     #  Sequence.__init__: 
     #
-    #  Initialise a new instance of Sequence.
+    #  Initialise a new instance.
     ############################################################################
     def __init__(self, startvalue=0):
       self.id = 'acl_sequence'
       self.value = startvalue
 
+
     """
-    ############################################################################    
-    #
-    #   FUNCTIONS
-    #
-    ############################################################################    
+    ############################################################################
+    ###
+    ###   Functions
+    ###
+    ############################################################################
     """
 
     # --------------------------------------------------------------------------
@@ -118,11 +103,11 @@ class Sequence(
 
 
     """
-    ############################################################################    
+    ############################################################################
     ###
-    ###   P r o p e r t i e s
+    ###   Properties
     ###
-    ############################################################################    
+    ############################################################################
     """
 
     ############################################################################
@@ -146,6 +131,6 @@ class Sequence(
 
       # Return.
       if RESPONSE is not None:
-        RESPONSE.redirect('manage_properties?manage_tabs_message=%s'%urllib.quote(message))
+        RESPONSE.redirect('%s?manage_tabs_message=%s'%(REQUEST[ 'HTTP_REFERER'],urllib.quote(message)))
 
 ################################################################################
