@@ -2,12 +2,10 @@
 # _versionmanager.py
 #
 # $Id: _versionmanager.py,v 1.12 2004/11/24 21:02:52 zmsdev Exp $
-# $Name:  $
+# $Name:$
 # $Author: zmsdev $
 # $Revision: 1.12 $
 #
-# Implementation of class VersionManager (see below).
-# 
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
 # as published by the Free Software Foundation; either version 2
@@ -44,17 +42,21 @@ import _zmsattributecontainer
 #  Hook for trigger of custom event (if there is one)
 # ------------------------------------------------------------------------------
 def triggerEvent(self, name, REQUEST):
-    if self.meta_type == 'ZMSCustom':
-      metaObjAttr = self.getMetaobjAttr(self.meta_id,name)
-      if metaObjAttr is not None:
-        try: REQUEST.set('preview','preview')
-        except: REQUEST['preview'] = 'preview'
-        v = self.getObjProperty(name,REQUEST)
-    for ob_id in self.getHome().objectIds():
-      if ob_id.find( name) == 0:
+  metaObjAttr = self.getMetaobjAttr(self.meta_id,name)
+  if metaObjAttr is not None:
+    try: REQUEST.set('preview','preview')
+    except: REQUEST['preview'] = 'preview'
+    v = self.getObjProperty(name,REQUEST)
+  ob = self
+  ob_ids = []
+  while ob is not None:
+    for ob_id in ob.getHome().objectIds():
+      if ob_id not in ob_ids and ob_id.find( name) == 0:
         try: REQUEST.set('preview','preview')
         except: REQUEST['preview'] = 'preview'
         v = getattr(self,ob_id)(context=self,REQUEST=REQUEST)
+        ob_ids.append(ob_id)
+    ob = ob.getPortalMaster()
 
 
 # ------------------------------------------------------------------------------
@@ -147,12 +149,12 @@ def doAutocommit(self, REQUEST):
   
   ##### Auto-Commit ####
   if len( getObjStates( self)) > 0:
-  
+    
     if self.inObjStates(['STATE_DELETED'],REQUEST):
        parent = self.getParentNode()
        parent.moveObjsToTrashcan([self.id], REQUEST)
        return
-
+    
     if self.version_work_id is not None:
       if self.version_live_id is not None and \
          self.version_live_id in self.objectIds( self.dGlobalAttrs.keys()):
@@ -197,7 +199,7 @@ class VersionItem:
         if checkPending:
           #-- Check for pending changes.
           if self.getObjStateNames(REQUEST):
-            raise "Can't tagObjVersions: %s@%s has pending changes %s"%(self.meta_type,self.absolute_url(),str(self.getObjStateNames(REQUEST)))
+            raise "Can't tagObjVersions: %s@%s has pending changes %s"%(self.meta_id,self.absolute_url(),str(self.getObjStateNames(REQUEST)))
         else:
           #-- Tag master-version.
           obj_version = None
@@ -243,10 +245,18 @@ class VersionItem:
       states = getObjStates(self)
       lang = REQUEST.get('lang',None)
       obj_states = []
-      for obj_state in self.getWfActivitiesIds():
-          obj_state_name = getObjStateName(obj_state,lang)
-          if obj_state_name in states:
-            obj_states.append(obj_state)
+      wfActivitiesIds = self.getWfActivitiesIds()
+      for obj_state in wfActivitiesIds:
+        obj_state_name = getObjStateName(obj_state,lang)
+        if obj_state_name in states:
+          obj_states.append(obj_state)
+      if len( obj_states) == 0 and \
+         len( wfActivitiesIds) > 0 and \
+         lang is not None and \
+        ( 'STATE_NEW_%s'%lang.upper() in states or \
+          'STATE_MODIFIED_%s'%lang.upper() in states or \
+          'STATE_DELETED_%s'%lang.upper() in states):
+        obj_states.append( wfActivitiesIds[0])
       return obj_states
 
 
@@ -256,21 +266,15 @@ class VersionItem:
     #  Returns all version-items.
     # --------------------------------------------------------------------------
     def getVersionItems(self, REQUEST, recursive=False):
+      pc = self.isPageContainer()
       children = []
-      if self.meta_type == 'ZMSCustom':
-        for metaobjAttrId in self.getMetaobjAttrIds( self.meta_id):
-          for child in self.getObjChildren( metaobjAttrId , REQUEST):
+      types = self.getMetaobjIds(sort=0)+['*']
+      for metaobjAttrId in self.getMetaobjAttrIds( self.meta_id, types=types):
+        for child in self.getObjChildren( metaobjAttrId , REQUEST):
+          if recursive or not pc or not child.isPage():
             children.append( child)
-            if recursive:
-              children.extend( child.getVersionItems( REQUEST))
-      elif self.meta_type == 'ZMSLinkContainer':
-        children.extend( self.getObjChildren( '', REQUEST))
-      else:
-        for child in self.getObjChildren( '', REQUEST):
-          if not child.isPage() or child.meta_type in [ 'ZMSLinkElement']:
-            children.append( child)
-            if not child.isPageElement():
-              children.extend( child.getVersionItems( REQUEST))
+            if not pc:
+              children.extend( child.getVersionItems( REQUEST, recursive=True))
       return children
 
 
@@ -347,7 +351,7 @@ class VersionItem:
     # --------------------------------------------------------------------------
     def isCommitted(self, REQUEST):
       if _globals.isPreviewRequest( REQUEST):
-        committed = not self.inObjStates( [ 'STATE_DELETED'], REQUEST) or self.getConfProperty( 'ZMS.workflow.highlightChanges')
+        committed = not self.inObjStates( [ 'STATE_DELETED'], REQUEST)
       else:
         committed = not self.inObjStates( [ 'STATE_NEW'], REQUEST)
       return committed
@@ -395,8 +399,14 @@ class VersionItem:
     #
     #  Resets object-state of this object.
     # --------------------------------------------------------------------------
-    def resetObjStates(self, REQUEST):
-      self.delObjStates( [ 'STATE_NEW', 'STATE_MODIFIED', 'STATE_DELETED'], REQUEST)
+    def resetObjStates(self, REQUEST=None):
+      if REQUEST is None:
+        self.version_live_id = self.version_work_id
+        self.version_work_id = None
+        self.__work_state__.states = []
+        self.__work_state__ = copy.deepcopy(self.__work_state__)
+      else:
+        self.delObjStates( [ 'STATE_NEW', 'STATE_MODIFIED', 'STATE_DELETED'], REQUEST)
 
 
     # --------------------------------------------------------------------------
@@ -417,6 +427,23 @@ class VersionItem:
       return False
 
 
+    # --------------------------------------------------------------------------
+    #  VersionItem.filteredObjStates:
+    #
+    #  Checks if given states are in object-states of this object.
+    # --------------------------------------------------------------------------
+    def filteredObjStates(self, REQUEST):
+      obj_states = []
+      states = getObjStates(self)
+      if len(states) > 0:
+        lang = REQUEST.get('lang',self.getPrimaryLanguage())
+        for obj_state in [ 'STATE_NEW', 'STATE_MODIFIED', 'STATE_DELETED']:
+          obj_state_name = getObjStateName(obj_state,lang)
+          if obj_state_name in states:
+            obj_states.append( obj_state)
+      return obj_states
+
+
     # --------------------------------------------------------------------------
     #  VersionItem.getObjStateNames:
     #
@@ -747,7 +774,7 @@ class VersionItem:
       
       ##### Trigger custom afterRollbackObjChanges-Event (if there is one) ####
       triggerEvent( self, 'afterRollbackObjChangesEvt', REQUEST)
-
+      
       # Return flag for deleted objects.
       return delete
 
@@ -827,12 +854,12 @@ class VersionItem:
 
     # --------------------------------------------------------------------------
     #  VersionItem.ajaxBodyContentObjHistory:
-    #
-    #  Returns ajax-xml with body-content of object-history for given version-nr.
     # --------------------------------------------------------------------------
     def ajaxBodyContentObjHistory(self, version_nr, REQUEST):
-      """ VersionItem.ajaxBodyContentObjHistory """
-
+      """
+      Returns ajax-xml with body-content of object-history for given version-nr.
+      """
+      
       #-- Get versions.
       master_version = int( version_nr[ :version_nr.find( '.')])
       major_version = int( version_nr[ version_nr.find( '.')+1: version_nr.rfind( '.')])
@@ -848,7 +875,7 @@ class VersionItem:
              item.get( 'major_version', 0) <= major_version:
             version_dt = item[ 'version_dt']
             break
-
+      
       #-- Build xml.
       RESPONSE = REQUEST.RESPONSE
       content_type = 'text/xml; charset=utf-8'
@@ -860,14 +887,14 @@ class VersionItem:
       self.f_standard_html_request( self, REQUEST)
       xml = self.getXmlHeader()
       xml += "<BodyContentObjHistory version_nr=\""+version_nr+"\">\n"
-      obj_version = self.getObjHistory( version_nr, REQUEST, False)
-      obj_history = self.getObjHistory( version_nr, REQUEST)
       if self.meta_type == 'ZMSCustom':
+        obj_version = self.getObjHistory( version_nr, REQUEST, False)
         REQUEST.set( 'ZMS_VERSION_%s'%self.id, obj_version.id)
         xml += '<ObjHistory id="' + obj_version.id + '">'
         xml += '<![CDATA[' + obj_version.getBodyContent( REQUEST) + ']]>'
         xml += '</ObjHistory>\n'
       else:
+        obj_history = self.getObjHistory( version_nr, REQUEST)
         for history_version in obj_history:
           if history_version.isActive(REQUEST):
             xml += '<ObjHistory id="' + history_version.id + '">'
@@ -1001,7 +1028,7 @@ class VersionItem:
         # sort object-items
         obs.sort()
         obs.reverse()
-        # truncate sort-id from sorted object-items
+        # truncate version-nr from sorted object-items
         obs = map( lambda ob: ob[1], obs)
         # return object-items
         return obs
@@ -1078,17 +1105,24 @@ class VersionItem:
 class VersionManagerContainer: 
 
     # --------------------------------------------------------------------------
-    #  VersionManagerContainer.getVersionManagerContainer
+    #  VersionItem.isVersionContainer
     # --------------------------------------------------------------------------
-    def getVersionManagerContainer(self):
-      parent = self.getParentNode()
-      if (self.meta_type == 'ZMSTeaserContainer') or \
-         (self.meta_type == 'ZMSTeaserElement'):
-        return parent.getVersionManagerContainer()
-      elif (self.meta_type == 'ZMSLinkContainer') or \
-           (self.meta_type == 'ZMSCustom' and (parent.meta_type == 'ZMSCustom' or not self.isPage())):
-         return parent
-      return self
+    def isVersionContainer(self):
+      b = False
+      b = b or (self.isPage() and self.isPageContainer())
+      b = b or (self.isPage() and self.meta_id == 'ZMSLinkElement' and self.isEmbedded(self.REQUEST))
+      if not b:
+        parent = self.getParentNode()
+        b = parent is not None and parent.isPageContainer() and self.isPage()
+      return b
+
+    # --------------------------------------------------------------------------
+    #  VersionManagerContainer.getVersionContainer
+    # --------------------------------------------------------------------------
+    def getVersionContainer(self):
+      if self.isVersionContainer():
+        return self
+      return self.getParentNode().getVersionContainer()
 
 
     """
@@ -1153,14 +1187,9 @@ class VersionManagerContainer:
       if _globals.debug( self):
         _globals.writeLog( self, "[autoWfTransition]")
       lang = REQUEST['lang']
-      # Special Objects.
-      parent = self.getParentNode()
-      if (self.meta_type == 'ZMSTeaserElement') or \
-         (self.meta_type == 'ZMSTeaserContainer') or \
-         (self.meta_type == 'ZMSLinkElement') or \
-         (self.meta_type == 'ZMSLinkContainer') or \
-         (self.meta_type == 'ZMSCustom' and (parent.meta_type == 'ZMSCustom' or not self.isPage())):
-        parent.autoWfTransition(REQUEST)
+      # Enter Container.
+      if not self.isVersionContainer():
+        return self.getVersionContainer().autoWfTransition(REQUEST)
         return
       # Enter Workflow.
       enter = len(self.getWfStates(REQUEST)) == 0
@@ -1235,8 +1264,8 @@ class VersionManagerContainer:
         work_dt = _globals.getDateTime( time.time())
         work_uid = str(REQUEST.get('AUTHENTICATED_USER'))
         work_desc = REQUEST.get('work_desc','')
-        self.setObjProperty('work_dt',work_dt,lang)
         self.setObjProperty('work_uid',work_uid,lang)
+        self.setObjProperty('work_dt',work_dt,lang)
         # Log Protocol.
         log = ''
         log = log + self.getLangFmtDate(work_dt,lang,'%Y-%m-%d %H:%M:%S') + '\t'
@@ -1253,9 +1282,9 @@ class VersionManagerContainer:
 
 
     """
-    ############################################################################    
+    ############################################################################
     #  Tasks 
-    ############################################################################    
+    ############################################################################
     """
 
     # Management Interface.
