@@ -1,13 +1,11 @@
-###################################################################################################
+################################################################################
 # _workflowmanager.py
 #
 # $Id: _workflowmanager.py,v 1.8 2004/11/23 23:26:37 zmsdev Exp $
-# $Name:  $
+# $Name:$
 # $Author: zmsdev $
 # $Revision: 1.8 $
 #
-# Implementation of class WorkflowManager (see below).
-#
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
 # as published by the Free Software Foundation; either version 2
@@ -21,7 +19,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-###################################################################################################
+################################################################################
 
 # Imports.
 from __future__ import nested_scopes
@@ -39,11 +37,11 @@ import _versionmanager
 
 
 """
-################################################################################################
+################################################################################
 #
 #   K E Y S
 #
-################################################################################################
+################################################################################
 """
 
 CONF_TRANSITIONS	= "ZMS.workflow.transitions"
@@ -58,16 +56,16 @@ CONF_CHANGE_UID		= "ZMS.workflow.change_uid"
 
 
 """
-################################################################################################
+################################################################################
 #
 #   X M L   I M / E X P O R T
 #
-################################################################################################
+################################################################################
 """
 
-# -------------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 #  _workflowmanager.initConf:
-# -------------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 def initConf(self, filename, REQUEST):
   filepath = package_home(globals())+'/import/'
   xmlfile = open(_fileutil.getOSPath(filepath+filename),'rb')
@@ -75,9 +73,9 @@ def initConf(self, filename, REQUEST):
   # Return filename.
   return filename
 
-# -------------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 #  _workflowmanager.importXml
-# -------------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 def importXml(self, xml, REQUEST):
   v = self.parseXmlString(xml)
   self.setConfProperty(CONF_CHANGE_DT, v.get('change_dt',_globals.getDateTime(time.time())))
@@ -95,9 +93,9 @@ def importXml(self, xml, REQUEST):
     btn = self.getZMILangStr('BTN_INSERT')
     self.manage_roleProperties(btn, key, lang, REQUEST)
 
-# -------------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 #   _workflowmanager.exportXml
-# -------------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 def exportXml(self, REQUEST, RESPONSE):
   value = {}
   value['change_dt'] = self.getConfProperty(CONF_CHANGE_DT,None)
@@ -112,16 +110,16 @@ def exportXml(self, REQUEST, RESPONSE):
   return export
 
 """
-################################################################################################
+################################################################################
 #
 #   T R A N S I T I O N S
 #
-################################################################################################
+################################################################################
 """
 
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 #  _workflowmanager.setWfTransition
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 def setWfTransition(self, id, newId, newName, newFrom, newTo, newPerformer=[], newDtml=''):
   message = ''
   obs = self.getConfProperty(CONF_TRANSITIONS,[])
@@ -156,16 +154,16 @@ def setWfTransition(self, id, newId, newName, newFrom, newTo, newPerformer=[], n
 
 
 """
-################################################################################################
+################################################################################
 #
 #   A C T I V I T I E S
 #
-################################################################################################
+################################################################################
 """
 
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 #  _workflowmanager.setWfActivity
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 def setWfActivity(self, id, newId, newName, newIcon=None, newEditable=1):
   obs = self.getConfProperty(CONF_ACTIVITIES,[])
   # Remove exisiting entry.
@@ -189,16 +187,16 @@ def setWfActivity(self, id, newId, newName, newIcon=None, newEditable=1):
 
 
 """
-################################################################################################
+################################################################################
 #
 #   I T E M
 #
-################################################################################################
+################################################################################
 """
 
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 #  _workflowmanager.delWfItem
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 def delWfItem(self, id, conf_key):
   obs = self.getConfProperty(conf_key,[])
   # Update attribute.
@@ -210,9 +208,9 @@ def delWfItem(self, id, conf_key):
   # Return with empty id.
   return ''
 
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 #  _workflowmanager.moveWfItem
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 def moveWfItem(self, id, pos, conf_key):
   obs = self.getConfProperty(conf_key,[])
   # Move.
@@ -228,16 +226,16 @@ def moveWfItem(self, id, pos, conf_key):
 
 
 """
-################################################################################################
+################################################################################
 #
 #   P R O T O C O L
 #
-################################################################################################
+################################################################################
 """
 
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 #  _workflowmanager.writeProtocol
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 def writeProtocol(self, entry):
   keep_entries = self.getConfProperty(CONF_PROTOCOL_KEEP,0)
   if keep_entries > 0:
@@ -247,9 +245,9 @@ def writeProtocol(self, entry):
     log.insert(0,entry)
     self.setConfProperty(CONF_PROTOCOL,log)
 
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 #  _workflowmanager.readProtocol
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 def readProtocol(self):
   log = ''
   for entry in self.getConfProperty(CONF_PROTOCOL,[]):
@@ -257,20 +255,20 @@ def readProtocol(self):
   return log
 
 
-###################################################################################################
-###################################################################################################
+################################################################################
+################################################################################
 ###
 ###   class WorkflowItem
 ###
-###################################################################################################
-###################################################################################################
+################################################################################
+################################################################################
 class WorkflowItem: 
 
-    # ---------------------------------------------------------------------------------------------
+    # --------------------------------------------------------------------------
     #  WorkflowItem.getAutocommit
     #
     #  Returns true if auto-commit is active (workflow is inactive), false otherwise.
-    # ---------------------------------------------------------------------------------------------
+    # --------------------------------------------------------------------------
     def getAutocommit(self): 
       autocommit = False
       if not autocommit:
@@ -291,19 +289,21 @@ class WorkflowItem:
       return autocommit
 
 
-    # ---------------------------------------------------------------------------------------------
+    # --------------------------------------------------------------------------
     #  WorkflowItem.filtered_workflow_actions:
-    # ---------------------------------------------------------------------------------------------
-    def filtered_workflow_actions(self, path, REQUEST):
+    # --------------------------------------------------------------------------
+    def filtered_workflow_actions(self, path=''):
       actions = []
+      REQUEST = self.REQUEST
       lang = REQUEST['lang']
       auth_user = REQUEST['AUTHENTICATED_USER']
       
       #-- Workflow.
-      if not self.getAutocommit() and (self.meta_type == 'ZMSCustom' or not self.isPageElement()) and 'work_uid' in self.getObjAttrs().keys():
+      if not self.getAutocommit() and self.isVersionContainer():
         wfStates = self.getWfStates(REQUEST)
+        wfTransitions = self.getWfTransitions()
         roles = self.getUserRoles(auth_user)
-        for wfTransition in self.getWfTransitions():
+        for wfTransition in wfTransitions:
           wfFrom = wfTransition.get('from',[])
           wfPerformer = wfTransition.get('performer',[])
           wfTo = wfTransition.get('to',[])
@@ -313,39 +313,41 @@ class WorkflowItem:
           append = append and (len(self.intersection_list(roles,wfPerformer)) > 0 or auth_user.has_permission('Manager',self))
           if append:
             actions.append((wfTransition['name'],path+'manage_wfTransition'))
-
+      
       #-- Headline,
       if len( actions) > 0:
         actions.insert(0,('----- %s -----'%self.getZMILangStr('TAB_WORKFLOW'),''))
-
+      
       # Return action list.
       return actions
 
 
-###################################################################################################
-###################################################################################################
+################################################################################
+################################################################################
 ###
 ###   class WorkflowManager
 ###
-###################################################################################################
-###################################################################################################
+################################################################################
+################################################################################
 class WorkflowManager: 
 
     # Management Interface.
     # ---------------------
-    manage_customizeWorkflowForm = HTMLFile('dtml/zms/manage_customizeworkflowform', globals())
+    manage_customizeWorkflowForm = HTMLFile('dtml/ZMS/manage_customizeworkflowform', globals())
 
-    # ---------------------------------------------------------------------------------------------
+
+    # --------------------------------------------------------------------------
     #  WorkflowManager.doAutocommit:
     #
     #  Auto-Commit ZMS-tree.
-    # ---------------------------------------------------------------------------------------------
+    # --------------------------------------------------------------------------
     def doAutocommit(self, lang, REQUEST):
       _versionmanager.doAutocommit(self,REQUEST)
 
-    # ---------------------------------------------------------------------------------------------
+
+    # --------------------------------------------------------------------------
     #  WorkflowManager.getWfTransitions
-    # ---------------------------------------------------------------------------------------------
+    # --------------------------------------------------------------------------
     def getWfTransitions(self):
       acquired = self.getConfProperty(CONF_ACQUIRE,0)
       if acquired:
@@ -362,23 +364,26 @@ class WorkflowManager:
           wfTransitions.append(wfTransition)
       return wfTransitions
 
-    # ---------------------------------------------------------------------------------------------
+
+    # --------------------------------------------------------------------------
     #  WorkflowManager.getWfTransitionsIds
-    # ---------------------------------------------------------------------------------------------
+    # --------------------------------------------------------------------------
     def getWfTransitionsIds(self):
       obs = self.getWfTransitions()
       return map(lambda x: x['id'], obs) 
 
-    # ---------------------------------------------------------------------------------------------
+
+    # --------------------------------------------------------------------------
     #  WorkflowManager.getWfTransition
-    # ---------------------------------------------------------------------------------------------
+    # --------------------------------------------------------------------------
     def getWfTransition(self, id):
       ob = filter(lambda x: x['id']==id, self.getWfTransitions())[0]
       return ob
 
-    # ---------------------------------------------------------------------------------------------
+
+    # --------------------------------------------------------------------------
     #  WorkflowManager.getWfActivities
-    # ---------------------------------------------------------------------------------------------
+    # --------------------------------------------------------------------------
     def getWfActivities(self): 
       acquired = self.getConfProperty(CONF_ACQUIRE,0)
       if acquired:
@@ -395,23 +400,26 @@ class WorkflowManager:
           workflowItms.append(workflowItm)
       return workflowItms
 
-    # ---------------------------------------------------------------------------------------------
+
+    # --------------------------------------------------------------------------
     #  WorkflowManager.getWfActivitiesIds
-    # ---------------------------------------------------------------------------------------------
+    # --------------------------------------------------------------------------
     def getWfActivitiesIds(self):
       obs = self.getWfActivities()
       return map(lambda x: x['id'], obs) 
 
-    # ---------------------------------------------------------------------------------------------
+
+    # --------------------------------------------------------------------------
     #  WorkflowManager.getWfActivity
-    # ---------------------------------------------------------------------------------------------
+    # --------------------------------------------------------------------------
     def getWfActivity(self, id):
       ob = filter(lambda x: x['id']==id, self.getWfActivities())[0]
       return ob
 
-    # ---------------------------------------------------------------------------------------------
+
+    # --------------------------------------------------------------------------
     #  WorkflowManager.getWfActivityDetails
-    # ---------------------------------------------------------------------------------------------
+    # --------------------------------------------------------------------------
     def getWfActivityDetails(self, id):
       ids = self.getWfActivitiesIds()
       froms = []
@@ -434,11 +442,11 @@ class WorkflowManager:
       return {'froms':froms, 'tos': tos, 'idxs': idxs, 'idx': idx}
 
 
-    ###############################################################################################
+    ############################################################################
     #  WorkflowManager.manage_changeWorkflow:
     #
     #  Customize workflow.
-    ###############################################################################################
+    ############################################################################
     def manage_changeWorkflow(self, lang, key='', btn='', REQUEST=None, RESPONSE=None):
       """ WorkflowManager.manage_changeWorkflow """
       message = ''
@@ -458,10 +466,10 @@ class WorkflowManager:
         newAcquire = REQUEST.get('acquire',0) == 1
         self.setConfProperty(CONF_ACQUIRE,newAcquire)
         message = self.getZMILangStr('MSG_CHANGED')
-	# Init.
+        # Init.
         filename = REQUEST.get('filename','')
         self.setConfProperty(CONF_INIT,filename)
-
+      
       # Protocol.
       # ---------
       elif key == 'protocol':
@@ -490,10 +498,10 @@ class WorkflowManager:
       # -------
       elif key == 'import' and btn == self.getZMILangStr('BTN_IMPORT'):
         f = REQUEST['file']
-	if f:
-	  filename = f.filename
+        if f:
+          filename = f.filename
           importXml(self, xml=f, REQUEST=REQUEST)
-	else:
+        else:
           filename = REQUEST.get('init')
           self.setConfProperty(CONF_INIT,filename)
           filename = initConf(self, filename, REQUEST)
@@ -504,11 +512,11 @@ class WorkflowManager:
       return RESPONSE.redirect('manage_customizeWorkflowForm?lang=%s&manage_tabs_message=%s#_%s'%(lang,message,key))
 
 
-    ###############################################################################################
+    ############################################################################
     #  WorkflowManager.manage_changeWfActivities:
     #
     #  Customize workflow-activities.
-    ###############################################################################################
+    ############################################################################
     def manage_changeWfActivities(self, lang, btn='', REQUEST=None, RESPONSE=None):
       """ WorkflowManager.manage_changeWfActivities """
       message = ''
@@ -567,11 +575,11 @@ class WorkflowManager:
       return RESPONSE.redirect('manage_customizeWorkflowForm?lang=%s&manage_tabs_message=%s#_Activities'%(lang,message))
 
 
-    ###############################################################################################
+    ############################################################################
     #  WorkflowManager.manage_changeWfTransitions:
     #
     #  Customize workflow-transitions.
-    ###############################################################################################
+    ############################################################################
     def manage_changeWfTransitions(self, lang, btn='', REQUEST=None, RESPONSE=None):
       """ WorkflowManager.manage_changeWfTransitions """
       message = ''
@@ -625,4 +633,4 @@ class WorkflowManager:
       message = urllib.quote(message)
       return RESPONSE.redirect('manage_customizeWorkflowForm?id=%s&lang=%s&manage_tabs_message=%s#_Transitions'%(id,lang,message))
 
-###################################################################################################
+################################################################################
