@@ -1,13 +1,11 @@
-###############################################################################################
+################################################################################
 # _xmllib.py
 #
 # $Id: _xmllib.py,v 1.13 2004/11/30 20:03:17 zmsdev Exp $
-# $Name:  $
+# $Name:$
 # $Author: zmsdev $
 # $Revision: 1.13 $
 #
-# Implementation of XML-Library (see below).
-# 
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
 # as published by the Free Software Foundation; either version 2
@@ -21,18 +19,20 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-###############################################################################################
+################################################################################
 
 # Imports.
 from __future__ import nested_scopes
-try: # Zope >=2.9
+try: # Zope >= 2.9
   import pyexpat
-except: # Zope <2.9
+except: # Zope < 2.9
   from Shared.DC.xml import pyexpat
 from App.Common import package_home
+from OFS.Image import File
 import Globals
 import copy
 import os
+import re
 import tempfile
 import time
 # Product Imports.
@@ -43,16 +43,16 @@ import _fileutil
 
 
 """
-###############################################################################################
+################################################################################
 #
 #  Datatypes
 #
-###############################################################################################
+################################################################################
 """
 
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 #  _xmllib.getXmlType:
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 def getXmlType(v):
   t = ''
   if type(v) is float:
@@ -72,12 +72,12 @@ def getXmlType(v):
   return t
 
 
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 #  _xmllib.getXmlTypeSaveValue:
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 def getXmlTypeSaveValue(v, attrs):
   # Strip.
-  if type(v) is type(''): 
+  if type(v) is str:
     while len(v) > 0 and v[0] <= ' ':
       v = v[1:]
     while len(v) > 0 and v[-1] <= ' ':
@@ -103,33 +103,33 @@ def getXmlTypeSaveValue(v, attrs):
 
 
 """
-###############################################################################################
+################################################################################
 #
 #  XML-Encoding
 #
-###############################################################################################
+################################################################################
 """
 
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 #  _xmllib.xml_header:
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 def xml_header(encoding='utf-8'):
   return '<?xml version="1.0" encoding="%s"?>\n'%encoding
 
 
 """
-###############################################################################################
+################################################################################
 #
 #  Import
 #
-###############################################################################################
+################################################################################
 """
 
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 #  _xmllib.xmlInitObjProperty:
 #
 #  Only for internal use: init specified property with value.
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 def xmlInitObjProperty(self, key, value, lang=None):
   
   #-- DEFINITION
@@ -144,15 +144,15 @@ def xmlInitObjProperty(self, key, value, lang=None):
   if value is not None:
     #-- Date-Fields
     if datatype in _globals.DT_DATETIMES:
-      if type(value) is type('') and len(value) > 0:
+      if type(value) is str and len(value) > 0:
         value = self.parseLangFmtDate(value)
     #-- Integer-Fields
     elif datatype in _globals.DT_INTS:
-      if type(value) is type('') and len(value) > 0:
+      if type(value) is str and len(value) > 0:
         value = int(value)
     #-- Float-Fields
     elif datatype == _globals.DT_FLOAT:
-      if type(value) is type('') and len(value) > 0:
+      if type(value) is str and len(value) > 0:
         value = float(value)
     #-- String-Fields
     elif datatype in _globals.DT_STRINGS:
@@ -163,9 +163,9 @@ def xmlInitObjProperty(self, key, value, lang=None):
     setattr(ob,attr,value)
 
 
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 #  _xmllib.xmlOnCharacterData:
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 def xmlOnCharacterData(self, sData, bInCData):
 
   #-- TAG-STACK
@@ -178,11 +178,11 @@ def xmlOnCharacterData(self, sData, bInCData):
   return 1  # accept any character data
 
 
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 #  _xmllib.xmlOnUnknownStartTag:
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 def xmlOnUnknownStartTag(self, sTagName, dTagAttrs):
-        
+  
   #-- TAG-STACK
   tag = {'name':sTagName,'attrs':dTagAttrs,'cdata':''}
   tag['dValueStack'] = self.dValueStack.size()
@@ -201,9 +201,9 @@ def xmlOnUnknownStartTag(self, sTagName, dTagAttrs):
   return 1  # accept any unknown tag
 
 
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 #  _xmllib.xmlOnUnknownEndTag:
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 def xmlOnUnknownEndTag(self, sTagName):
   
   #-- TAG-STACK
@@ -213,7 +213,7 @@ def xmlOnUnknownEndTag(self, sTagName):
     
     attrs = _globals.unencode( tag['attrs'])
     cdata = _globals.unencode( tag['cdata'])
-
+    
     #-- Zope < 2.9
     if type( attrs) is list:
       attrs = self.dict_list( attrs)
@@ -247,7 +247,7 @@ def xmlOnUnknownEndTag(self, sTagName):
         if content_type.find('text/') == 0:
           data = cdata
         else:
-          data = _blobfields.hex2bin(cdata)
+          data = _globals.hex2bin(cdata)
         value['data'] = data
       self.dValueStack.push(value)
     
@@ -271,7 +271,6 @@ def xmlOnUnknownEndTag(self, sTagName):
     #-- OBJECT-ATTRIBUTES --
     #-----------------------
     elif sTagName in self.getObjAttrs().keys():
-    
       obj_attr = self.getObjAttr(sTagName)
       
       #-- DATATYPE
@@ -299,58 +298,65 @@ def xmlOnUnknownEndTag(self, sTagName):
               self.setObjProperty('change_dt',time.time(),s_lang)
               xmlInitObjProperty(self,sTagName,value,s_lang)
       
-      #-- Complex Attributes (Dictionary|List).
-      elif self.dValueStack.size() > 0:
-        value = self.dValueStack.pop()
-        # Data
-        if datatype in _globals.DT_BLOBS:
-          if type(value) is dict and len(value.keys()) > 0:
-            ob = _blobfields.createBlobField(self,datatype)
-            for key in value.keys():
-              setattr(ob,key,value[key])
-            xmlInitObjProperty(self,sTagName,ob)
-        # Others
-        else:
-          if self.meta_type == 'ZMSCustom' and \
-             self.getType() == 'ZMSRecordSet':
-            if type( value) is list:
-              for item in value:
-                if type( item) is dict:
-                  for key in item.keys():
-                    item_obj_attr = self.getObjAttr( key)
-                    item_datatype = item_obj_attr['datatype_key']
-                    if item_datatype in _globals.DT_BLOBS:
-                      item_data = item[ key]
-                      if type( item_data) is dict:
-                        blob = _blobfields.createBlobField( self, item_datatype, item_data)
-                        item[ key] = blob
-          #-- Convert multilingual to monolingual attributes.
-          if obj_attr['multilang']==0 and \
-             type(value) is dict and \
-             len(value.keys()) == 1 and \
-             value.keys()[0] == self.getPrimaryLanguage():
-            value = value[value.keys()[0]]
-          xmlInitObjProperty(self,sTagName,value)
-        if self.dValueStack.size() > 0:
-          raise "Items on self.dValueStack=%s"%self.dValueStack
-      
-      #-- Simple Attributes (String, Integer, etc.)
       else:
-        value = cdata
-        #-- OPTIONS
-        if obj_attr.has_key('options'):
-          options = obj_attr['options']
-          if type(options) is list:
-            try:
-              i = options.index(int(value))
-              if i%2==1: value = options[i-1]
-            except:
-              try: 
-                i = options.index(str(value))
+        #-- Complex Attributes (Blob|Dictionary|List).
+        value = None
+        if self.dValueStack.size() > 0:
+          value = self.dValueStack.pop()
+        if value is not None and \
+           ( datatype in _globals.DT_BLOBS or \
+             datatype == _globals.DT_LIST or \
+             datatype == _globals.DT_DICT):
+          # Data
+          if datatype in _globals.DT_BLOBS:
+            if type(value) is dict and len(value.keys()) > 0:
+              ob = _blobfields.createBlobField(self,datatype)
+              for key in value.keys():
+                setattr(ob,key,value[key])
+              xmlInitObjProperty(self,sTagName,ob)
+          # Others
+          else:
+            if self.getType() == 'ZMSRecordSet':
+              if type( value) is list:
+                for item in value:
+                  if type( item) is dict:
+                    for key in item.keys():
+                      item_obj_attr = self.getObjAttr( key)
+                      item_datatype = item_obj_attr['datatype_key']
+                      if item_datatype in _globals.DT_BLOBS:
+                        item_data = item[ key]
+                        if type( item_data) is dict:
+                          blob = _blobfields.createBlobField( self, item_datatype, item_data)
+                          item[ key] = blob
+            #-- Convert multilingual to monolingual attributes.
+            if obj_attr['multilang']==0 and \
+               type(value) is dict and \
+               len(value.keys()) == 1 and \
+               value.keys()[0] == self.getPrimaryLanguage():
+              value = value[value.keys()[0]]
+            xmlInitObjProperty(self,sTagName,value)
+          if self.dValueStack.size() > 0:
+            raise "Items on self.dValueStack=%s"%self.dValueStack
+        
+        #-- Simple Attributes (String, Integer, etc.)
+        else:
+          if value is not None:
+            _globals.writeBlock( self, "[xmlOnUnknownEndTag]: WARNING - Skip %s=%s"%(sTagName,str(value)))
+          value = cdata
+          #-- OPTIONS
+          if obj_attr.has_key('options'):
+            options = obj_attr['options']
+            if type(options) is list:
+              try:
+                i = options.index(int(value))
                 if i%2==1: value = options[i-1]
-              except: 
-                pass
-        xmlInitObjProperty(self,sTagName,value)
+              except:
+                try: 
+                  i = options.index(str(value))
+                  if i%2==1: value = options[i-1]
+                except:
+                  pass
+          xmlInitObjProperty(self,sTagName,value)
       
       # Clear value stack.
       self.dValueStack.clear()
@@ -359,16 +365,24 @@ def xmlOnUnknownEndTag(self, sTagName):
     #------------
     else:
       value = self.dTagStack.pop()
-      if value is None: value = {'cdata':''}
-      cdata = value.get('cdata','')
-      cdata += '<' + tag['name'] 
-      for attr_name in attrs.keys():
-        attr_value = attrs.get( attr_name)
-        cdata += ' ' + attr_name + '="' + attr_value + '"'
-      cdata += '>' + tag['cdata'] 
-      cdata += '</' + tag['name'] + '>'
-      value['cdata'] = cdata
-      self.dTagStack.push(value)
+      # ZMS < 2.11: titleshort => titlealt
+      if value is None and sTagName == 'titleshort' and 'titlealt' in self.getObjAttrs().keys():
+        sTagName = 'titlealt'
+        tag['name'] = sTagName
+        self.dTagStack.push( tag)
+        self.xmlOnUnknownEndTag( sTagName)
+      # ZMS >= 2.11
+      else:
+        if value is None: value = {'cdata':''}
+        cdata = value.get('cdata','')
+        cdata += '<' + tag['name'] 
+        for attr_name in attrs.keys():
+          attr_value = attrs.get( attr_name)
+          cdata += ' ' + attr_name + '="' + attr_value + '"'
+        cdata += '>' + tag['cdata'] 
+        cdata += '</' + tag['name'] + '>'
+        value['cdata'] = cdata
+        self.dTagStack.push(value)
     
     return 1 # accept matching end tag
   else:
@@ -376,16 +390,16 @@ def xmlOnUnknownEndTag(self, sTagName):
 
 
 """
-###############################################################################################
+################################################################################
 #
 #  Export
 #
-###############################################################################################
+################################################################################
 """
 
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 #  _xmllib.toCdata
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 def toCdata(self, s, xhtml=0):
   rtn = ''
   
@@ -446,15 +460,17 @@ def toCdata(self, s, xhtml=0):
 
   # Return Text in CDATA.
   elif s is not None:
+    # Hack for nested CDATA
+    s = re.compile( '\<\!\[CDATA\[(.*?)\]\]\>').sub( '<!{CDATA{\\1}}>', s)
     rtn = '<![CDATA[%s]]>'%s
 
   # Return.
   return rtn
 
 
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 #  _xmllib.toXml:
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 def toXml(self, value, xhtml=0, encoding='utf-8'):
   xml = []
   
@@ -468,6 +484,20 @@ def toXml(self, value, xhtml=0, encoding='utf-8'):
     elif isinstance(value,_blobfields.MyFile):
       xml.append( value.toXml( self))
     
+    # File (Zope-native)
+    elif isinstance(value,File):
+      tagname = 'data'
+      xml.append( '\n<%s'%tagname)
+      xml.append( ' content_type="%s"'%value.content_type)
+      xml.append( ' filename="%s"'%value.title)
+      xml.append( ' type="file"')
+      xml.append( '>')
+      if value.content_type.find( 'text/') == 0:
+        xml.append( '<![CDATA[%s]]>'%str(value.data))
+      else:
+        xml.append( _globals.bin2hex(value.data))
+      xml.append( '</%s>'%tagname)
+    
     # Dictionaries
     elif type(value) is dict:
       keys = value.keys()
@@ -485,7 +515,7 @@ def toXml(self, value, xhtml=0, encoding='utf-8'):
     # Tuples (DateTime)
     elif type(value) is tuple or type(value) is time.struct_time:
       try:
-        s_value = self.getLangFmtDate(value,self.getPrimaryLanguage(),'DATETIME_FMT')
+        s_value = self.getLangFmtDate(value,'eng','DATETIME_FMT')
         if len(s_value) > 0:
           xml.append(toCdata(self,s_value,-1))
       except:
@@ -506,9 +536,9 @@ def toXml(self, value, xhtml=0, encoding='utf-8'):
   return ''.join(xml)
 
 
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 #  _xmllib.getAttrToXml:
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 def getAttrToXml(self, base_path, data2hex, obj_attr, REQUEST):
   xml = ''
   
@@ -540,9 +570,8 @@ def getAttrToXml(self, base_path, data2hex, obj_attr, REQUEST):
     
     # XML.
     elif datatype == _globals.DT_XML or \
-         datatype == _globals.DT_URL or \
-	 datatype == _globals.DT_BOOLEAN or \
-	 datatype in _globals.DT_NUMBERS:
+         datatype == _globals.DT_BOOLEAN or \
+         datatype in _globals.DT_NUMBERS:
       xml += toXml( self, value, -1)
 
     # Others.
@@ -553,9 +582,9 @@ def getAttrToXml(self, base_path, data2hex, obj_attr, REQUEST):
   return xml
 
 
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 #  _xmllib.getObjPropertyToXml:
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 def getObjPropertyToXml(self, base_path, data2hex, obj_attr, REQUEST):
   xml = ''
   # Multi-Language Attributes.
@@ -576,13 +605,13 @@ def getObjPropertyToXml(self, base_path, data2hex, obj_attr, REQUEST):
   return xml
 
 
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 #  _xmllib.getObjToXml:
-# ---------------------------------------------------------------------------------------------
+# ------------------------------------------------------------------------------
 def getObjToXml(self, REQUEST, incl_embedded=False, deep=True, base_path='', data2hex=False):
   # Check Constraints.
   root = getattr( self, '__root__', None)
-  if root is not None or self.meta_type == 'ZMSTrashcan':
+  if root is not None:
     return ''
   ob = self
   if ob.meta_type != 'ZMS':
@@ -600,12 +629,10 @@ def getObjToXml(self, REQUEST, incl_embedded=False, deep=True, base_path='', dat
     xml.append(' id="%s"'%id)
   if ob.getParentNode() is not None and ob.getParentNode().meta_type == 'ZMSCustom':
     xml.append('\n id_prefix="%s"'%_globals.id_prefix(ob.id))
-  if ob.meta_type == 'ZMSCustom':
-    xml.append('\n meta_id="%s"'%ob.meta_id)
   xml.append('>')
   # Attributes.
   keys = ob.getObjAttrs().keys()
-  if ob.meta_type=='ZMSCustom' and ob.getType()=='ZMSRecordSet':
+  if ob.getType()=='ZMSRecordSet':
     keys = ['active',ob.getMetaobjAttrIds(ob.meta_id)[0]]
   for key in keys:
     obj_attr = ob.getObjAttr(key)
@@ -624,22 +651,22 @@ def getObjToXml(self, REQUEST, incl_embedded=False, deep=True, base_path='', dat
 
 
 """
-###############################################################################################
+################################################################################
 # class ParseError(Exception):
 #
 # General exception class to indicate parsing errors.
-###############################################################################################
+################################################################################
 """
 class ParseError(Exception): pass
 
 
 
 """
-###############################################################################################
+################################################################################
 # class XmlAttrBuilder:
 # 
 # Parser for complex Python-Attributes (dictionaries, lists).
-###############################################################################################
+################################################################################
 """
 class XmlAttrBuilder:
     "class XmlAttrBuilder"
@@ -647,17 +674,17 @@ class XmlAttrBuilder:
     ######## class variables ########
     iBufferSize=1028 * 32   # buffer size for XML file parsing
 
-    ###########################################################################################
+    ############################################################################
     # XmlAttrBuilder.__init__(self):
     #
     # Constructor.
-    ###########################################################################################
+    ############################################################################
     def __init__(self):
       """ XmlAttrBuilder.__init__ """
       pass
 
 
-    ###########################################################################################
+    ############################################################################
     # XmlAttrBuilder.parse(self, input):
     #
     # Parse a given XML document.
@@ -665,7 +692,7 @@ class XmlAttrBuilder:
     # IN:  input = XML document as string
     #            = XML document as file object
     # OUT: value or None, if nothing was parsed
-    ###########################################################################################
+    ############################################################################
     def parse(self, input, mediadbStorable=True):
       """ XmlAttrBuilder.parse """
       
@@ -691,7 +718,7 @@ class XmlAttrBuilder:
       #### parsing ####
       if _globals.debug( self): 
         _globals.writeLog( self, "#### parsing ####")
-      if type(input) is type(''):
+      if type(input) is str:
         # input is a string!
         rv = p.Parse(input, 1)
       else:
@@ -714,7 +741,7 @@ class XmlAttrBuilder:
       return self.dValueStack.pop()
 
 
-    ###########################################################################################
+    ############################################################################
     # XmlAttrBuilder.OnStartElement(self, name, attrs):
     #
     # Handler of XML-Parser: 
@@ -726,7 +753,7 @@ class XmlAttrBuilder:
     #
     # IN: name  = element name (=tag name)
     #     attrs = dictionary of element attributes
-    ###########################################################################################
+    ############################################################################
     def OnStartElement(self, sTagName, dTagAttrs):
       """ XmlAttrBuilder.OnStartElement """
       if _globals.debug( self):
@@ -746,14 +773,14 @@ class XmlAttrBuilder:
         self.dValueStack.push([])
 
 
-    ###########################################################################################
+    ############################################################################
     # XmlAttrBuilder.OnEndElement(self, name):
     #
     # Handler of XML-Parser: 
     # Called at the end of a XML element (resp. on occurence of a XML end tag).
     #
     # IN: name  = element name (=tag name)
-    ###########################################################################################
+    ############################################################################
     def OnEndElement(self, sTagName):
       """ XmlAttrBuilder.OnEndElement """
       if _globals.debug( self):
@@ -764,6 +791,8 @@ class XmlAttrBuilder:
       name = _globals.unencode( tag['name'])
       attrs = _globals.unencode( tag['attrs'])
       cdata = _globals.unencode( tag['cdata'])
+      # Hack for nested CDATA
+      cdata = re.compile( '\<\!\{CDATA\{(.*?)\}\}\>').sub( '<![CDATA[\\1]]>', cdata)
       
       #-- Zope < 2.9
       if type( attrs) is list:
@@ -779,7 +808,7 @@ class XmlAttrBuilder:
         if content_type.find('text/') == 0:
           data = cdata
         else:
-          data = _blobfields.hex2bin( cdata)
+          data = _globals.hex2bin( cdata)
         file = {'data':data,'filename':filename,'content_type':content_type}
         objtype = attrs.get('type')
         item = None
@@ -809,7 +838,7 @@ class XmlAttrBuilder:
         self.dValueStack.push(value)
 
 
-    ###########################################################################################
+    ############################################################################
     # XmlAttrBuilder.OnCharacterData(self, data):
     #
     # Handler of XML-Parser:
@@ -818,7 +847,7 @@ class XmlAttrBuilder:
     # CDATA block.
     #
     # IN: data = character data string
-    ###########################################################################################
+    ############################################################################
     def OnCharacterData(self, sData):
       """ XmlAttrBuilder.OnCharacterData """
       if _globals.debug( self):
@@ -831,29 +860,29 @@ class XmlAttrBuilder:
         self.dTagStack.push(tag)
 
 
-    ###########################################################################################
+    ############################################################################
     # XmlAttrBuilder.OnStartCData(self):
     #
     # Handler of XML-Parser:
     # Called at the start of a CDATA block (resp. on occurence of the "CDATA[" tag).
-    ###########################################################################################
+    ############################################################################
     def OnStartCData(self):
       """ XmlAttrBuilder.OnStartCData """
       self.bInCData = 1 
 
 
-    ###########################################################################################
+    ############################################################################
     # XmlAttrBuilder.OnEndCData(self):
     #
     # Handler of XML-Parser:
     # Called at the end of a CDATA block (resp. on occurence of the "]" tag).
-    ###########################################################################################
+    ############################################################################
     def OnEndCData(self):
       """ XmlAttrBuilder.OnEndCData """
       self.bInCData = 0
 
 
-    ###########################################################################################
+    ############################################################################
     # XmlAttrBuilder.OnProcessingInstruction(self, target, data):
     #
     # Handler of XML-Parser:
@@ -861,26 +890,26 @@ class XmlAttrBuilder:
     #
     # IN: target = target (processing instruction)
     #     data   = dictionary of data
-    ###########################################################################################
+    ############################################################################
     def OnProcessingInstruction(self, target, data):
       """ XmlAttrBuilder.OnProcessingInstruction """
       pass  # ignored
 
 
-    ###########################################################################################
+    ############################################################################
     # XmlAttrBuilder.OnComment(self, data):
     #
     # Handler of XML-Parser:
     # Called on occurence of a comment.
     #
     # IN: data = comment string
-    ###########################################################################################
+    ############################################################################
     def OnComment(self, data):
       """ XmlAttrBuilder.OnComment """
       pass  # ignored
 
 
-    ###########################################################################################
+    ############################################################################
     # XmlAttrBuilder.OnStartNamespaceDecl(self, prefix, uri):
     #
     # Handler of XML-Parser:
@@ -888,20 +917,20 @@ class XmlAttrBuilder:
     #
     # IN: prefix = prefix of namespace
     #     uri    = namespace identifier
-    ###########################################################################################
+    ############################################################################
     def OnStartNamespaceDecl(self, prefix, uri):
       """ XmlAttrBuilder.OnStartNamespaceDecl """
       pass  # ignored
 
 
-    ###########################################################################################
+    ############################################################################
     # XmlAttrBuilder.OnEndNamespaceDecl(self, prefix):
     #
     # Handler of XML-Parser:
     # Called at the end of a namespace declaration.
     #
     # IN: prefix = prefix of namespace
-    ###########################################################################################
+    ############################################################################
     def OnEndNamespaceDecl(self, prefix):
       """ XmlAttrBuilder.OnEndNamespaceDecl """
       pass  # ignored
@@ -909,11 +938,11 @@ class XmlAttrBuilder:
 
 
 """
-###############################################################################################
+################################################################################
 # class XmlBuilder:
 # 
 # Parser for custom xml.
-###############################################################################################
+################################################################################
 """
 class XmlBuilder:
     "class XmlBuilder"
@@ -921,17 +950,17 @@ class XmlBuilder:
     ######## class variables ########
     iBufferSize=1028 * 32   # buffer size for XML file parsing
 
-    ###########################################################################################
+    ############################################################################
     # XmlBuilder.__init__(self):
     #
     # Constructor.
-    ###########################################################################################
+    ############################################################################
     def __init__(self):
       """ XmlBuilder.__init__ """
       pass
 
 
-    ###########################################################################################
+    ############################################################################
     # XmlBuilder.parse(self, input):
     #
     # Parse a given XML document.
@@ -939,7 +968,7 @@ class XmlBuilder:
     # IN:  input = XML document as string
     #            = XML document as file object
     # OUT: value or None, if nothing was parsed
-    ###########################################################################################
+    ############################################################################
     def parse(self, input):
         """ XmlBuilder.parse """
         
@@ -964,7 +993,7 @@ class XmlBuilder:
         #### parsing ####
         if _globals.debug( self): 
           _globals.writeLog( self, "#### parsing ####")
-        if type(input) is type('string'):
+        if type(input) is str:
           # input is a string!
           rv = p.Parse(input, 1)
         else:
@@ -987,7 +1016,7 @@ class XmlBuilder:
         return self.dTagStack.pop()['tags']
 
 
-    ###########################################################################################
+    ############################################################################
     # XmlBuilder.OnStartElement(self, name, attrs):
     #
     # Handler of XML-Parser: 
@@ -999,7 +1028,7 @@ class XmlBuilder:
     #
     # IN: name  = element name (=tag name)
     #     attrs = dictionary of element attributes
-    ###########################################################################################
+    ############################################################################
     def OnStartElement(self, sTagName, dTagAttrs):
       """ XmlBuilder.OnStartElement """
       if _globals.debug( self):
@@ -1009,14 +1038,14 @@ class XmlBuilder:
       self.dTagStack.push(tag)
 
 
-    ###########################################################################################
+    ############################################################################
     # XmlBuilder.OnEndElement(self, name):
     #
     # Handler of XML-Parser: 
     # Called at the end of a XML element (resp. on occurence of a XML end tag).
     #
     # IN: name  = element name (=tag name)
-    ###########################################################################################
+    ############################################################################
     def OnEndElement(self, sTagName):
       """ XmlBuilder.OnEndElement """
       if _globals.debug( self):
@@ -1050,7 +1079,7 @@ class XmlBuilder:
       self.dTagStack.push(parent)
 
 
-    ###########################################################################################
+    ############################################################################
     # XmlBuilder.OnCharacterData(self, data):
     #
     # Handler of XML-Parser:
@@ -1059,7 +1088,7 @@ class XmlBuilder:
     # CDATA block.
     #
     # IN: data = character data string
-    ###########################################################################################
+    ############################################################################
     def OnCharacterData(self, sData):
       """ XmlBuilder.OnCharacterData """
       if _globals.debug( self):
@@ -1070,29 +1099,29 @@ class XmlBuilder:
       self.dTagStack.push(tag)
 
 
-    ###########################################################################################
+    ############################################################################
     # XmlBuilder.OnStartCData(self):
     #
     # Handler of XML-Parser:
     # Called at the start of a CDATA block (resp. on occurence of the "CDATA[" tag).
-    ###########################################################################################
+    ############################################################################
     def OnStartCData(self):
       """ XmlBuilder.OnStartCData """
       self.bInCData=1
 
 
-    ###########################################################################################
+    ############################################################################
     # XmlBuilder.OnEndCData(self):
     #
     # Handler of XML-Parser:
     # Called at the end of a CDATA block (resp. on occurence of the "]" tag).
-    ###########################################################################################
+    ############################################################################
     def OnEndCData(self):
       """ XmlBuilder.OnEndCData """
       self.bInCData=0
 
 
-    ###########################################################################################
+    ############################################################################
     # XmlBuilder.OnProcessingInstruction(self, target, data):
     #
     # Handler of XML-Parser:
@@ -1100,26 +1129,26 @@ class XmlBuilder:
     #
     # IN: target = target (processing instruction)
     #     data   = dictionary of data
-    ###########################################################################################
+    ############################################################################
     def OnProcessingInstruction(self, target, data):
       """ XmlBuilder.OnProcessingInstruction """
       pass  # ignored
 
 
-    ###########################################################################################
+    ############################################################################
     # XmlBuilder.OnComment(self, data):
     #
     # Handler of XML-Parser:
     # Called on occurence of a comment.
     #
     # IN: data = comment string
-    ###########################################################################################
+    ############################################################################
     def OnComment(self, data):
       """ XmlBuilder.OnComment """
       pass  # ignored
 
 
-    ###########################################################################################
+    ############################################################################
     # XmlBuilder.OnStartNamespaceDecl(self, prefix, uri):
     #
     # Handler of XML-Parser:
@@ -1127,20 +1156,20 @@ class XmlBuilder:
     #
     # IN: prefix = prefix of namespace
     #     uri    = namespace identifier
-    ###########################################################################################
+    ############################################################################
     def OnStartNamespaceDecl(self, prefix, uri):
       """ XmlBuilder.OnStartNamespaceDecl """
       pass  # ignored
 
 
-    ###########################################################################################
+    ############################################################################
     # XmlBuilder.OnEndNamespaceDecl(self, prefix):
     #
     # Handler of XML-Parser:
     # Called at the end of a namespace declaration.
     #
     # IN: prefix = prefix of namespace
-    ###########################################################################################
+    ############################################################################
     def OnEndNamespaceDecl(self, prefix):
       """ XmlBuilder.OnEndNamespaceDecl """
       pass  # ignored
