@@ -2,12 +2,10 @@
 # _zcatalogmanager.py
 #
 # $Id: _zcatalogmanager.py,v 1.8 2004/11/30 20:03:17 zmsdev Exp $
-# $Name:  $
+# $Name:$
 # $Author: zmsdev $
 # $Revision: 1.8 $
 #
-# Implementation of classes ZCatalogItem and ZCatalogManager (see below).
-# 
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
 # as published by the Free Software Foundation; either version 2
@@ -26,6 +24,7 @@
 # Imports.
 import new
 import re
+import sys
 from Products.ZCatalog import ZCatalog, CatalogAwareness
 # Product Imports.
 import _globals
@@ -40,7 +39,7 @@ class Empty: pass
 # ------------------------------------------------------------------------------
 #  _zcatalogmanager.zcat_meta_types
 # ------------------------------------------------------------------------------
-zcat_meta_types = ['ZMS','ZMSFolder','ZMSDocument','ZMSCustom','ZMSSysFolder','ZMSFile']
+zcat_meta_types = ['ZMS','ZMSCustom']
 
 
 # ------------------------------------------------------------------------------
@@ -51,7 +50,7 @@ zcat_meta_types = ['ZMS','ZMSFolder','ZMSDocument','ZMSCustom','ZMSSysFolder','Z
 def search_string(v):
   s = ''
   if v is not None:
-    if type(v) is type('') and len(v) > 0:
+    if type(v) is str and len(v) > 0:
       s += v + ' '
     elif type(v) is list:
       for i in v:
@@ -81,14 +80,14 @@ def search_quote(s, maxlen=255, tag='&middot;'):
 # ------------------------------------------------------------------------------
 #  _zcatalogmanager.addVocabulary:
 # ------------------------------------------------------------------------------
-def addVocabulary(cat):
+def addVocabulary( self, cat):
   from Products.ZCatalog import Vocabulary
-
+  
   #-- Remove Default-Vocabulary
   ids = cat.objectIds( ['Vocabulary'])
   if len( ids) > 0:
     cat.manage_delObjects( ids)
-      
+  
   #-- Add ISO 8859-1 Vocabulary
   # see ZCatalogs with Umlauts
   # http://www.zope.org/Members/strobl/HowTos/Iszcatalog
@@ -100,24 +99,33 @@ def addVocabulary(cat):
 # ------------------------------------------------------------------------------
 #  _zcatalogmanager.addLexicon:
 # ------------------------------------------------------------------------------
-def addLexicon(cat):
-  elem = []
-  wordSplitter = Empty()
-  wordSplitter.group = 'Word Splitter'
-  wordSplitter.name = 'HTML aware splitter'
-  caseNormalizer = Empty()
-  caseNormalizer.group = 'Case Normalizer'
-  caseNormalizer.name = 'Case Normalizer'
-  stopWords = Empty()
-  stopWords.group = 'Stop Words'
-  stopWords.name = 'Remove listed and single char words'
-  elem.append(wordSplitter)
-  elem.append(caseNormalizer)
-  elem.append(stopWords)
-  try:      
-    cat.manage_addProduct['ZCTextIndex'].manage_addLexicon('Lexicon', 'Default lexicon', elem)
-  except:
-    pass
+def addLexicon( self, cat):
+  
+  #-- Remove Lexicon
+  ids = cat.objectIds( ['ZCTextIndex Lexicon'])
+  if len( ids) > 0:
+    cat.manage_delObjects( ids)
+  
+  #-- Add Lexicon
+  index_type = self.getConfProperty('ZCatalog.TextIndexType','ZCTextIndex')
+  if index_type == 'ZCTextIndex':
+    elem = []
+    wordSplitter = Empty()
+    wordSplitter.group = 'Word Splitter'
+    wordSplitter.name = 'HTML aware splitter'
+    caseNormalizer = Empty()
+    caseNormalizer.group = 'Case Normalizer'
+    caseNormalizer.name = 'Case Normalizer'
+    stopWords = Empty()
+    stopWords.group = 'Stop Words'
+    stopWords.name = 'Remove listed and single char words'
+    elem.append(wordSplitter)
+    elem.append(caseNormalizer)
+    elem.append(stopWords)
+    try:
+      cat.manage_addProduct['ZCTextIndex'].manage_addLexicon('Lexicon', 'Default lexicon', elem)
+    except:
+      pass
 
 
 ################################################################################
@@ -197,13 +205,28 @@ class ZCatalogItem(CatalogAwareness.CatalogAware):
             s = s.replace( capital, letter)
       except ( UnicodeDecodeError, UnicodeEncodeError):
         _globals.writeException(self,"[search_encode]")
+        v = str(sys.exc_value)
+        STR_POSITION = ' position '
+        i = v.find(STR_POSITION)
+        if i > 0:
+          v = v[i+len(STR_POSITION):]
+          if v.find('-') > 0:
+            l = int( v[:v.find('-')])
+            h = int( v[v.find('-')+1:v.find(':')])
+          else:
+            l = int( v[:v.find(':')])
+            h = l
+          ln = max( l - 20, 0)
+          hn = min( h + 20, len(s))
+          print ">>>>>",s[ln:hn]
+          print ">>>>>"," "*(l-ln)+"^"*(h-l+1)
       return s
 
 
     # --------------------------------------------------------------------------
-    #	ZCatalogItem.getCatalogNavUrl:
+    #  ZCatalogItem.getCatalogNavUrl:
     #
-    #	Returns catalog-navigation url.
+    #  Returns catalog-navigation url.
     # --------------------------------------------------------------------------
     def getCatalogNavUrl(self, REQUEST):
       return self.url_inherit_params(REQUEST['URL'],REQUEST,['qs'])
@@ -216,23 +239,30 @@ class ZCatalogItem(CatalogAwareness.CatalogAware):
     # --------------------------------------------------------------------------
     def catalogText(self, REQUEST):
       v = ''
-      if self.isVisible(REQUEST):
+      # Custom hook.
+      if 'catalogText' in self.getMetaobjAttrIds(self.meta_id):
+        value = self.getObjProperty('catalogText',REQUEST)
+        value = search_string(value)
+        v += value
+      else:
         for key in filter( lambda x: x.find('_') != 0, self.getObjAttrs().keys()):
           obj_attr = self.getObjAttr(key)
           if obj_attr['xml']:
             datatype = obj_attr['datatype_key']
             if datatype in _globals.DT_STRINGS or \
-               datatype == _globals.DT_DICTIONARY or \
+               datatype == _globals.DT_DICT or \
                datatype == _globals.DT_LIST:
               value = self.getObjAttrValue(obj_attr,REQUEST)
               value = search_string(value)
-              # Increase weight of title-attributes!
-              if obj_attr['key'] in ['title','titlealt','titleshort']:
+              # Increase weight of attributes!
+              if obj_attr['id'].find('title') == 0:
+                value = value * 5
+              elif obj_attr['id'].find('attr_dc_') == 0:
                 value = value * 3
               v += value
-        for ob in self.getChildNodes():
-          if not ob.isCatalogItem():
-            v += ob.catalogText(REQUEST)
+      for ob in self.getChildNodes():
+        if not ob.isCatalogItem():
+          v += ob.catalogText(REQUEST)
       return v
 
 
@@ -243,14 +273,13 @@ class ZCatalogItem(CatalogAwareness.CatalogAware):
     # --------------------------------------------------------------------------
     def catalogData(self, REQUEST):
       source = ''
-      if self.isVisible( REQUEST):
-        key = self.txng_get_key()
-        if key is not None:
-          file = self.getObjProperty( key, REQUEST)
-          if file is not None:
-            source = file.getData()
-            mimetype = file.getContentType()
-            encoding = 'utf-8'
+      key = self.txng_get_key()
+      if key is not None:
+        file = self.getObjProperty( key, REQUEST)
+        if file is not None:
+          source = file.getData()
+          mimetype = file.getContentType()
+          encoding = 'utf-8'
       return source
 
 
@@ -308,7 +337,7 @@ class ZCatalogItem(CatalogAwareness.CatalogAware):
     # --------------------------------------------------------------------------
     def isCatalogItem(self):
       b = False
-      b = b or (self.isPage() or self.meta_type == 'ZMSFile')
+      b = b or (self.isPage() or self.meta_id == 'ZMSFile')
       b = b or (self.getConfProperty('ZCatalog.TextIndexNG',0)==1 and self.txng_get_key() is not None)
       return b
 
@@ -386,10 +415,10 @@ class ZCatalogManager:
         zcatalog = obs[ 0]
       
       #-- Add vocabulary
-      addVocabulary( zcatalog)
+      addVocabulary( self, zcatalog)
       
       #-- Add lexicon
-      addLexicon( zcatalog)
+      addLexicon( self, zcatalog)
       
       #-- Clear catalog
       zcatalog.manage_catalogClear()
@@ -420,11 +449,13 @@ class ZCatalogManager:
         message += "Index ["
         index_name = 'zcat_text_%s'%lang
         setattr( ZCatalogItem, index_name, new.function(ZCatalogItem.zcat_text.func_code, {}, index_name, (lang,)))
-        index_type = 'ZCTextIndex'
-        index_extras = Empty()
-        index_extras.doc_attr = index_name
-        index_extras.index_type = 'Okapi BM25 Rank'
-        index_extras.lexicon_id = 'Lexicon'            
+        index_type = self.getConfProperty('ZCatalog.TextIndexType','ZCTextIndex')
+        index_extras = None
+        if index_type == 'ZCTextIndex':
+          index_extras = Empty()
+          index_extras.doc_attr = index_name
+          index_extras.index_type = 'Okapi BM25 Rank'
+          index_extras.lexicon_id = 'Lexicon'
         zcatalog.manage_addColumn(index_name)
         zcatalog.manage_addIndex(index_name,index_type,index_extras)
         message += index_name
@@ -438,7 +469,7 @@ class ZCatalogManager:
           if index_type == 'ZCTextIndex':
             index_extras.doc_attr = index_name
             index_extras.index_type = 'Okapi BM25 Rank'
-            index_extras.lexicon_id = 'Lexicon'            
+            index_extras.lexicon_id = 'Lexicon'
           else:
             index_extras.default_encoding = 'utf-8'
             index_extras.indexed_fields = index_name
@@ -545,16 +576,16 @@ class ZCatalogManager:
     #  Submits query to catalog.
     # --------------------------------------------------------------------------
     def submitCatalogQuery(self, search_query, search_order_by, search_meta_types, search_clients, REQUEST):
-
+      
       #-- Initialize local variables.
       message = ''
       rtn = []
-    
+      
       #-- Process clients.
       if self.getConfProperty('ZCatalog.portalClients',1) == 1 and search_clients == 1:
         for portalClient in self.getPortalClients():
           rtn.extend(portalClient.submitCatalogQuery( search_query, search_order_by, search_meta_types, search_clients, REQUEST))
-          
+      
       #-- Numbers are not cataloged.
       lnum = []
       qs = ''
@@ -591,6 +622,8 @@ class ZCatalogManager:
         order_by = 'time'
       
       #-- Process results.
+      abs_url = self.absolute_url()
+      aq_abs_url = self.aq_absolute_url()
       results = []
       for item in items:
         # Get object by path.
@@ -599,11 +632,12 @@ class ZCatalogManager:
         ob = self.getCatalogPathObject( path)
         # Uncatalog object.
         uncatalog = ob is None
+        append = ob is not None
         if not uncatalog:
           for path_ob in ob.breadcrumbs_obj_path():
             if not uncatalog:
               uncatalog = uncatalog or path_ob == self.getTrashcan()
-              uncatalog = uncatalog or not path_ob.isVisible(REQUEST)
+              append = append and path_ob.isVisible(REQUEST)
         if uncatalog:
           try:
             zcatalog.uncatalog_object(path)
@@ -611,16 +645,13 @@ class ZCatalogManager:
             pass
           ob = None
         # Check for valid result.
-        append = ob is not None and ob not in map(lambda x: x[1]['ob'], results)
+        append = append and ob not in map(lambda x: x[1]['ob'], results)
         if append:
           # Handle Pages.
           append = ob.isCatalogItem()
           # Handle Meta-Types.
           if len(search_meta_types) > 0:
-            if ob.meta_type == 'ZMSCustom':
-              append = append and ob.meta_id in search_meta_types
-            else:
-              append = append and ob.meta_type in search_meta_types
+            append = append and ob.meta_id in search_meta_types
           # Handle Numbers.
           zctext = getattr(ob,'zcat_text_%s'%lang,'')
           for num in lnum:
@@ -646,19 +677,17 @@ class ZCatalogManager:
             sum_url = 'index_%s.html'%lang
             if REQUEST.get('preview','')=='preview':
               sum_url = self.url_append_params(sum_url,{'preview':'preview'})
-            if ob.meta_type=='ZMSCustom':
-              sum_url = self.url_append_params(sum_url,{'ZMS_PARAM0':'search','ZMS_PARAM1':search_query})
           result = {}
           result['ob'] = ob
           result['score'] = sum_score
           result['time'] = sum_time
-          result['url'] = ob.getDeclUrl(REQUEST) + '/' + sum_url
+          result['url'] = ob.getDeclUrl(REQUEST).replace(abs_url,aq_abs_url) + '/' + sum_url
           results.append((result[order_by],result))
       
       #-- Sort objects.
       results.sort()
       results.reverse()
-
+      
       #-- Append objects.
       rtn.extend(map(lambda ob: ob[1],results))
       
