@@ -2,12 +2,10 @@
 # _zreferableitem.py
 #
 # $Id: _zreferableitem.py,v 1.9 2004/11/30 20:03:17 zmsdev Exp $
-# $Name:  $
+# $Name:$
 # $Author: zmsdev $
 # $Revision: 1.9 $
 #
-# Implementation of classes ZReferableItem (see below).
-# 
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
 # as published by the Free Software Foundation; either version 2
@@ -50,14 +48,14 @@ def getMedlineLink(url):
 #  _zreferableitem.isMailLink:
 # ------------------------------------------------------------------------------
 def isMailLink(url):
-  rtn = url.lower().find('mailto:') == 0
+  rtn = type(url) is str and url.lower().find('mailto:') == 0
   return rtn
 
 # ------------------------------------------------------------------------------
 #  _zreferableitem.isInternalLink:
 # ------------------------------------------------------------------------------
 def isInternalLink(url):
-  rtn = type(url) is type('') and url.startswith('{$') and url.endswith('}')
+  rtn = type(url) is str and url.startswith('{$') and url.endswith('}')
   return rtn
 
 # ------------------------------------------------------------------------------
@@ -88,8 +86,8 @@ class ZReferableItem:
 
   # Management Interface.
   # ---------------------
-  manage_RefForm = HTMLFile('dtml/zmslinkelement/manage_refform', globals())
-  manage_browse_objs = browse_objs = HTMLFile('dtml/zmslinkelement/browse_objs', globals()) 
+  manage_RefForm = HTMLFile('dtml/ZMSLinkElement/manage_refform', globals())
+  manage_browse_objs = f_browse_objs = HTMLFile('dtml/ZMSLinkElement/f_browse_objs', globals()) 
 
 
   # ----------------------------------------------------------------------------
@@ -432,18 +430,17 @@ class ZReferableItem:
   #
   #  Resolves internal/external links and returns URL.
   # ----------------------------------------------------------------------------
-  def getLinkUrl( self, url, REQUEST={}): 
+  def getLinkUrl( self, url, REQUEST=None):
+    REQUEST = _globals.nvl( REQUEST, self.REQUEST)
     if isInternalLink(url):
       ob = self.getLinkObj(url,REQUEST)
-      if ob is not None:
-        if ob.meta_type == 'ZMSFile':
-          file = ob.getFile(REQUEST)
-          if file is not None:
-            return file.getHref(REQUEST)
-        else:
-          return ob.getHref2IndexHtml(REQUEST)
+      if ob is None:
+        index_html = './index_%s.html?op=not_found&url=%s'%(REQUEST.get('lang',self.getPrimaryLanguage()),url)
       else:
-        return './index_%s.html?op=not_found&url=%s'%(REQUEST.get('lang',self.getPrimaryLanguage()),url)
+        index_html = ob.getObjProperty('getHref2IndexHtml',REQUEST)
+        if index_html == '':
+          index_html = ob.getHref2IndexHtml(REQUEST)
+      return index_html
     elif isMailLink (url): 
       prefix = 'mailto:'
       return prefix + self.encrypt_ordtype(url[len(prefix):])
@@ -454,11 +451,12 @@ class ZReferableItem:
   #
   #  Resolves internal/external links and returns Html.
   # ----------------------------------------------------------------------------
-  def getLinkHtml( self, url, html='<a href="%s">&raquo;</a>', REQUEST={}):
+  def getLinkHtml( self, url, html='<a href="%s">&raquo;</a>', REQUEST=None):
+    REQUEST = _globals.nvl( REQUEST, self.REQUEST)
     s = ''
     ob = self
     while ob is not None:
-      if html in ob.getMetaobjIds():
+      if html in ob.getMetaobjIds( sort=0):
         metaObj = ob.getMetaobj( html) 
         for metaObjAttr in metaObj['attrs']:
           if metaObjAttr['type'] == 'method' and \
@@ -489,17 +487,17 @@ class ZReferableItem:
     t0 = time.time()
     
     obs = {}
-    map( lambda x: operator.setitem(obs, x.absolute_url(), x), _globals.objectTree( self, True))
+    map( lambda x: operator.setitem(obs, x.base_url(), x), _globals.objectTree( self, True))
     message += 'Load object-tree (in '+str(int((time.time()-t0)*100.0)/100.0)+' secs.)<br/>'
     abs_urls = obs.keys()
     abs_urls.sort()
+    map( lambda x: delattr( obs[x], 'ref_by'), filter( lambda x: hasattr( obs[x], 'ref_by'), abs_urls))
     langs = self.getLangIds()
-    
     for abs_url in abs_urls:
-      ob  = obs[ abs_url]
+      ob = obs[ abs_url]
       
       # Process recordset.
-      if ob.meta_type == 'ZMSCustom' and ob.getType()=='ZMSRecordSet':
+      if ob.getType()=='ZMSRecordSet':
         key = ob.getMetaobjAttrIds(ob.meta_id)[0]
         obj_attr = ob.getObjAttr(key)
         for lang in langs:
@@ -510,7 +508,7 @@ class ZReferableItem:
               for k in r.keys():
                 v = r[k]
                 o = v
-                if type(v) is type(''):
+                if type(v) is str:
                   l = v.split('{$')
                   if l > 1:
                     m = [l[0]]
@@ -529,7 +527,9 @@ class ZReferableItem:
                       if len( f) == 0:
                         ref = '__%s__'%ref
                       else:
-                        ref = ob.getRefObjPath( obs[f[0]])[2:-1]
+                        target = obs[f[0]]
+                        ref = ob.getRefObjPath( target)[2:-1]
+                        setattr( target, 'ref_by', getattr( target, 'ref_by', []) + [ target.getRefObjPath( ob)])
                       if ref.startswith('__') and ref.endswith('__'):
                         message += '<a href="%s/manage_main" target="_blank">%s(%s).%s[%i]=%s</a><br/>'%(ob.absolute_url(),ob.absolute_url(),ob.meta_type,k,c,ref)
                       m.append(ref+i[i.find('}'):])
@@ -538,7 +538,7 @@ class ZReferableItem:
                       r[k] = v
               c += 1
       
-      # Process object. 
+      # Process object.
       else:
         for key in ob.getObjAttrs().keys():
           obj_attr = ob.getObjAttr(key)
@@ -548,7 +548,7 @@ class ZReferableItem:
               for obj_vers in ob.getObjVersions():
                 v = _objattrs.getobjattr(ob,obj_vers,obj_attr,lang)
                 o = v
-                if type(v) is type(''):
+                if type(v) is str:
                   l = v.split('{$')
                   if l > 1:
                     m = [l[0]]
@@ -567,14 +567,16 @@ class ZReferableItem:
                       if len( f) == 0:
                         ref = '__%s__'%ref
                       else:
-                        ref = ob.getRefObjPath( obs[f[0]])[2:-1]
+                        target = obs[f[0]]
+                        ref = ob.getRefObjPath( target)[2:-1]
+                        setattr( target, 'ref_by', getattr( target, 'ref_by', []) + [ target.getRefObjPath( ob)])
                       if ref.startswith('__') and ref.endswith('__'):
                         message += '<a href="%s/manage_main" target="_blank">%s(%s).%s=%s</a><br/>'%(ob.absolute_url(),ob.absolute_url(),ob.meta_type,key,ref)
                       m.append(ref+i[i.find('}'):])
                     v = '{$'.join(m)
                     if v != o:
                       _objattrs.setobjattr(ob,obj_vers,obj_attr,v,lang)
-
+    
     if ob_id is not None:
       # Return with desired object.
       home = ob_id.split('@')[0]
