@@ -2,12 +2,10 @@
 # zmslinkelement.py
 #
 # $Id: zmslinkelement.py,v 1.7 2004/11/23 23:04:47 zmsdev Exp $
-# $Name:  $
+# $Name:$
 # $Author: zmsdev $
 # $Revision: 1.7 $
 #
-# Implementation of class ZMSLinkElement (see below).
-#
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
 # as published by the Free Software Foundation; either version 2
@@ -24,7 +22,7 @@
 ################################################################################
 
 # Imports.
-from Globals import HTML, HTMLFile
+from Globals import HTMLFile
 import sys
 import urllib
 # Product Imports.
@@ -32,7 +30,6 @@ from zmscontainerobject import ZMSContainerObject
 from zmsobject import ZMSObject
 from zmsproxyobject import ZMSProxyObject
 import _globals
-import _metadata
 import _zreferableitem
 
 
@@ -57,23 +54,23 @@ def initZMSLinkElement(oItem, REQUEST):
 def setZMSLinkElement(oItem, title, url, description, REQUEST):
   lang = REQUEST['lang']
   
-  if title != oItem.getObjProperty('titleshort',REQUEST) or \
+  if title != oItem.getObjProperty('titlealt',REQUEST) or \
      title != oItem.getObjProperty('title',REQUEST) or \
      url != oItem.getObjProperty('attr_ref',REQUEST) or \
      description != oItem.getObjProperty('attr_dc_description',REQUEST):
-
+    
     type = 'new'
     if _zreferableitem.isInternalLink(url):
       type = 'replace'
-
+    
     ##### Object State ####
     oItem.setObjStateModified(REQUEST)
-
+    
     ##### Set Metadata ####
     oItem.setObjProperty('attr_dc_description',description,lang)
-
+    
     ##### Set Properties ####
-    oItem.setObjProperty('titleshort',title,lang)
+    oItem.setObjProperty('titlealt',title,lang)
     oItem.setObjProperty('title',title,lang)
     oItem.setObjProperty('attr_ref',url,lang)
     oItem.setObjProperty('attr_type',type,lang)
@@ -93,32 +90,32 @@ def addZMSLinkElement(self, title, url, description, REQUEST):
   type = 'new'
   if _zreferableitem.isInternalLink(url):
     type = 'replace'
-
+  
   ##### Create ####
   id_prefix = _globals.id_prefix(REQUEST.get('id','e'))
   obj = ZMSLinkElement(self.getNewId(id_prefix))
   self._setObject(obj.id, obj)
-
+  
   obj = getattr(self,obj.id)
-
+  
   ##### Object State ####
   obj.setObjStateNew(REQUEST)
-
+  
   ##### Init Metadata (Important: DC.Coverage!) ####
   obj.setObjProperty("attr_dc_coverage","local."+lang,lang)
   obj.setObjProperty("attr_dc_description",description,lang)
-
+  
   ##### Init Properties ####
-  obj.setObjProperty("titleshort",title,lang)
+  obj.setObjProperty("titlealt",title,lang)
   obj.setObjProperty("title",title,lang)
   obj.setObjProperty("active",1,lang)
   obj.setObjProperty("attr_ref",url,lang)
   obj.setObjProperty("attr_type",type,lang)
   initZMSLinkElement(obj,REQUEST)
-    
+  
   ##### VersionManager ####
   obj.onChangeObj(REQUEST)
-
+  
   ##### Normalize Sort-IDs ####
   self.normalizeSortIds(id_prefix)
   
@@ -136,64 +133,65 @@ class ConstraintViolation(Exception): pass
 
 
 ################################################################################
-# Constructor(s):
-#
-# Form and method to add new object.
+################################################################################
+###
+###  Constructor
+###
+################################################################################
 ################################################################################
 manage_addZMSLinkElementForm = HTMLFile('manage_addzmslinkelementform', globals()) 
-def manage_addZMSLinkElement(self, lang, _sort_id, REQUEST):
+def manage_addZMSLinkElement(self, lang, _sort_id, REQUEST, RESPONSE):
   """ manage_addZMSLinkElement """
+  meta_id = 'ZMSLinkElement'
   
-  ##### Create ####
-  id_prefix = _globals.id_prefix(REQUEST.get('id','e'))
-  obj = ZMSLinkElement(self.getNewId(id_prefix),_sort_id+1)
-  self._setObject(obj.id, obj)
-  
-  obj = getattr(self,obj.id)
-
-  ##### Object State ####
-  obj.setObjStateNew(REQUEST)
-
-  ##### Init Metadata (Important: DC.Coverage!) ####
-  obj.setMetadata(lang,REQUEST)
-
-  ##### Init Properties ####
-  obj.setReqProperty('titleshort',REQUEST)
-  obj.setReqProperty('title',REQUEST)
-  obj.setReqProperty('active',REQUEST)
-  obj.setReqProperty('attr_active_start',REQUEST)
-  obj.setReqProperty('attr_active_end',REQUEST)
-  obj.setReqProperty('attr_ref',REQUEST)
-  obj.setReqProperty('attr_type',REQUEST)
-  initZMSLinkElement(obj,REQUEST)
+  if REQUEST['btn'] == self.getZMILangStr('BTN_INSERT'):
     
-  ##### VersionManager ####
-  obj.onChangeObj(REQUEST)
-            
-  ##### Normalize Sort-IDs ####
-  self.normalizeSortIds(id_prefix)
-
-  # Return with message.        
-  if REQUEST.RESPONSE:
+    ##### Create ####
+    id_prefix = _globals.id_prefix(REQUEST.get('id','e'))
+    obj = ZMSLinkElement(self.getNewId(id_prefix),_sort_id+1)
+    self._setObject(obj.id, obj)
+    
+    redirect_self = bool( REQUEST.get('redirect_self',0)) or REQUEST.get('btn','') == ''
+    for attr in self.getMetaobj(meta_id)['attrs']:
+      attr_type = attr['type']
+      redirect_self = redirect_self or attr_type in self.getMetaobjIds()+['*']
+    redirect_self = redirect_self and not REQUEST.get('btn','') in  [ self.getZMILangStr('BTN_CANCEL'), self.getZMILangStr('BTN_BACK')]
+    
+    obj = getattr(self,obj.id)
+    ##### Object State ####
+    obj.setObjStateNew(REQUEST)
+    ##### Init Coverage ####
+    obj.setReqProperty('attr_dc_coverage',REQUEST)
+    ##### Init Properties ####
+    obj.manage_changeProperties(lang,REQUEST,RESPONSE)
+    initZMSLinkElement(obj,REQUEST)
+    
+    ##### VersionManager ####
+    obj.onChangeObj(REQUEST)
+    
+    # Return with message.
     message = self.getZMILangStr('MSG_INSERTED')%obj.display_type(REQUEST)
-    REQUEST.RESPONSE.redirect('%s/%s/manage_main?lang=%s&manage_tabs_message=%s'%(self.absolute_url(),obj.id,lang,urllib.quote(message)))
-        
+    if redirect_self:
+      RESPONSE.redirect('%s/%s/manage_main?lang=%s&manage_tabs_message=%s'%(self.absolute_url(),obj.id,lang,urllib.quote(message)))
+    else:
+      RESPONSE.redirect('%s/manage_main?lang=%s&manage_tabs_message=%s#_%s'%(self.absolute_url(),lang,urllib.quote(message),obj.id))
+  
+  else:
+    RESPONSE.redirect('%s/manage_main?lang=%s'%(self.absolute_url(),lang))
+
 
 ################################################################################
 ################################################################################
-###   
-###   C l a s s
-###   
+###
+###  Class
+###
 ################################################################################
 ################################################################################
-
-class ZMSLinkElement(ZMSContainerObject, _metadata.Metadata):
+class ZMSLinkElement(ZMSContainerObject):
 
     # Properties.
     # -----------
-    meta_type = "ZMSLinkElement"
-    icon = "misc_/zms/zmslinkcontainer.gif"
-    icon_disabled = "misc_/zms/zmslinkcontainer_disabled.gif"
+    meta_type = meta_id = "ZMSLinkElement"
 
     # Management Options.
     # -------------------
@@ -206,9 +204,9 @@ class ZMSLinkElement(ZMSContainerObject, _metadata.Metadata):
     # Management Permissions.
     # -----------------------
     __authorPermissions__ = (
-		'manage','manage_main','manage_workspace',
+		'manage','manage_main','manage_workspace','manage_checkout',
 		'manage_changeProperties',
-		'manage_moveObjUp','manage_moveObjDown',
+		'manage_moveObjUp','manage_moveObjDown','manage_moveObjToPos',
 		'manage_cutObjects','manage_copyObjects','manage_pasteObjs',
 		'manage_userForm','manage_user',
 		)
@@ -216,45 +214,16 @@ class ZMSLinkElement(ZMSContainerObject, _metadata.Metadata):
 		('ZMS Author', __authorPermissions__),
 		)
 
-    # Properties.
-    # -----------
-    __obj_attrs__ = {
-        # Changed by
-        'created_uid':{'datatype':'string','multilang':False,'xml':False},
-        'created_dt':{'datatype':'datetime','multilang':False,'xml':False},
-        'change_uid':{'datatype':'string','multilang':True,'xml':False,'lang_inherit':False},
-        'change_dt':{'datatype':'datetime','multilang':True,'xml':False,'lang_inherit':False},
-       # Version info
-        'master_version':{'datatype':'int','xml':False,'default':0},
-        'major_version':{'datatype':'int','xml':False,'default':0},
-        'minor_version':{'datatype':'int','xml':False,'default':0},
-        # Properties
-        'active':{'datatype':'boolean'},
-        'attr_active_start':{'datatype':'datetime','multilang':True},
-        'attr_active_end':{'datatype':'datetime','multilang':True},
-        'titleshort':{'datatype':'string','multilang':True,'size':20},
-        'title':{'datatype':'string','multilang':True,'size':40},
-        'attr_ref':{'datatype':'url'},
-        'attr_type':{'datatype':'string','type':'select','options':['replace','replace','new','new','embed','embed','recursive','recursive']},
-        # Meta-Data
-        'attr_dc_coverage':{'datatype':'string'},
-        # Meta-Dictionaries        
-        '$metadict':{'datatype':'MetaDict'},
-    }
-
-
     # Management Interface.
     # ---------------------
-    manage_main = HTMLFile('dtml/zmslinkelement/manage_main', globals())
+    manage_main = HTMLFile('dtml/ZMSObject/manage_main', globals())
 
 
-    """
     ############################################################################
     #
     #   Constructor
     #
     ############################################################################
-    """
 
     # --------------------------------------------------------------------------
     #  ZMSLinkElement.getSelf:
@@ -272,7 +241,7 @@ class ZMSLinkElement(ZMSContainerObject, _metadata.Metadata):
     #  ZMSLinkElement.getEmbedType: 
     # --------------------------------------------------------------------------
     def getEmbedType(self):
-      rtn = self.getObjAttrValue( self.getObjAttr( 'attr_type'), self.REQUEST) 
+      rtn = self.getObjAttrValue( self.getObjAttr( 'attr_type'), self.REQUEST)
       return rtn
 
 
@@ -280,7 +249,7 @@ class ZMSLinkElement(ZMSContainerObject, _metadata.Metadata):
     #  ZMSLinkElement.isEmbedded: 
     # --------------------------------------------------------------------------
     def isEmbedded(self, REQUEST):
-      rtn = self.getEmbedType() in [ 'embed', 'recursive']
+      rtn = self.getEmbedType() in [ 'embed', 'recursive', 'remote']
       return rtn
 
 
@@ -292,44 +261,11 @@ class ZMSLinkElement(ZMSContainerObject, _metadata.Metadata):
       return rtn
 
 
-    # --------------------------------------------------------------------------
-    #  ZMSLinkElement.getLinkList: 
-    #
-    #  Overrides getLinkList of zmscontainerobject.ZMSContainerObject.
-    # --------------------------------------------------------------------------
-    def getLinkList(self, REQUEST, allow_none=0):
-      rtn = []
-      append = self.isVisible(REQUEST) and not self.isResource(REQUEST)
-      if append:
-        if self.isPage() and self.isEmbedded(REQUEST):
-          for el in ZMSContainerObject.getLinkList(self,REQUEST,allow_none):
-            if not el in rtn:
-              rtn.append(el)
-          ref_obj = self.getRefObj()
-          if isinstance(ref_obj,ZMSContainerObject):
-            for el in ZMSContainerObject.getLinkList(ref_obj,REQUEST,allow_none):
-              if not el in rtn:
-                rtn.append(el)
-        elif not self.isEmbedded(REQUEST):
-          ref = self.getObjProperty('attr_ref',REQUEST)
-          dct = {}
-          dct['internal'] = _zreferableitem.isInternalLink(ref)
-          dct['src'] = self
-          dct['dst'] = self.getLinkObj(ref,REQUEST)
-          dct['url'] = self.getLinkUrl(ref,REQUEST)
-          dct['title'] = self.getTitle(REQUEST)
-          dct['description'] = _globals.nvl(self.getObjProperty('attr_dc_description',REQUEST),'')
-          rtn.extend([dct])
-      return rtn
-
-
-    """
     ############################################################################
     ###
     ###   Properties
     ###
     ############################################################################
-    """
 
     ############################################################################
     #  ZMSLinkElement.manage_changeProperties: 
@@ -360,9 +296,6 @@ class ZMSLinkElement(ZMSContainerObject, _metadata.Metadata):
             if obj_attr['xml']:
               self.setReqProperty(key,REQUEST)
           
-          ##### Metadata ####
-          self.setMetadata(lang,REQUEST)
-          
           ##### VersionManager ####
           self.onChangeObj(REQUEST)
           
@@ -384,7 +317,7 @@ class ZMSLinkElement(ZMSContainerObject, _metadata.Metadata):
     # --------------------------------------------------------------------------
     #  ZMSLinkElement.getRef:
     # --------------------------------------------------------------------------
-    def getRef(self): 
+    def getRef(self):
       coverage = self.getDCCoverage()
       req = {'preview':'preview','lang':coverage[coverage.find('.')+1:]}
       ref = self.getObjAttrValue( self.getObjAttr( 'attr_ref'), req) 
@@ -394,7 +327,7 @@ class ZMSLinkElement(ZMSContainerObject, _metadata.Metadata):
     # --------------------------------------------------------------------------
     #  ZMSLinkElement.getRefObj:
     # --------------------------------------------------------------------------
-    def getRefObj(self): 
+    def getRefObj(self):
       coverage = self.getDCCoverage()
       req = {'preview':'preview','lang':coverage[coverage.find('.')+1:]}
       ref_obj = self.getLinkObj( self.getRef(), req)
@@ -405,6 +338,30 @@ class ZMSLinkElement(ZMSContainerObject, _metadata.Metadata):
       return ref_obj
 
 
+    # --------------------------------------------------------------------------
+    #  ZMSLinkElement.getRemoteObj:
+    # --------------------------------------------------------------------------
+    def getRemoteObj(self):
+      
+      REQUEST = self.REQUEST
+      lang = REQUEST['lang']
+      #-- [ReqBuff]: Fetch buffered value from Http-Request.
+      reqBuffId = 'getRemoteObj'
+      try:
+        value = self.fetchReqBuff( reqBuffId, REQUEST)
+        return value
+      except:
+        value = None
+        ref = self.getRef()
+        try:
+          value = self.http_import( ref + '/ajaxGetNode?lang=%s'%lang)
+          value = self.xmlParse( value)
+        except:
+          _globals.writeException(self,'[getRemoteObj]: can\'t embed from remote: ref=%s'%ref)
+        #-- [ReqBuff]: Returns value and stores it in buffer of Http-Request.
+        return self.storeReqBuff( reqBuffId, value, REQUEST)
+
+
     # --------------------------------------------------------------------------
     #  ZMSLinkElement.isMetaType:
     # --------------------------------------------------------------------------
@@ -420,7 +377,7 @@ class ZMSLinkElement(ZMSContainerObject, _metadata.Metadata):
             if not (self.NORESOLVEREF == meta_type or (type(meta_type) is list and self.NORESOLVEREF in meta_type)):
               b = b or ref_obj.isMetaType(meta_type,REQUEST)
       return b
-      
+
     def isMetaType(self, meta_type, REQUEST={'preview':'preview'}):
       proxy = self.getProxy()
       rtn = self.isMetaTypePROXY( proxy, meta_type, REQUEST)
@@ -452,8 +409,13 @@ class ZMSLinkElement(ZMSContainerObject, _metadata.Metadata):
       else:
         rtn = getattr( self, 'aq_parent', getattr( self, 'base', None))
       return rtn
-    
+
+    getParentNode__roles__ = None
     def getParentNode(self):
+      """
+      The parent of this node. 
+      All nodes except root may have a parent.
+      """
       proxy = self.getProxy()
       rtn = self.getParentNodePROXY( proxy)
       return rtn
@@ -466,23 +428,30 @@ class ZMSLinkElement(ZMSContainerObject, _metadata.Metadata):
       if proxy != self and proxy is not None and self.isEmbeddedRecursive( REQUEST):
         rtn = proxy.getTitlealt( REQUEST)
       else:
-        rtn = self.getObjProperty('titleshort',REQUEST)
+        rtn = self.getObjProperty('titlealt',REQUEST)
         if len(rtn) == 0:
           ref_obj = self.getRefObj()
           if ref_obj is None:
             rtn = ZMSContainerObject.getTitlealt(self,REQUEST)
           else:
             rtn = ref_obj.getTitlealt(REQUEST)
+      return rtn
+
+    def getTitlealt(self, REQUEST):
+      rtn = ''
+      if self.getEmbedType() == 'remote':
+        remote_obj = self.getRemoteObj()
+        if type( remote_obj) is list:
+          for node in self.xmlNodeSet( remote_obj, 'titlealt'):
+            rtn = node['cdata']
+      else:
+        proxy = self.getProxy()
+        rtn = self.getTitlealtPROXY( proxy, REQUEST)
       if len(rtn) == 0:
         rtn = self.display_type(REQUEST)
       return rtn
 
-    def getTitlealt(self, REQUEST):
-      proxy = self.getProxy()
-      rtn = self.getTitlealtPROXY( proxy, REQUEST)
-      return rtn
 
-      
     # --------------------------------------------------------------------------
     #  ZMSLinkElement.getTitle:
     # --------------------------------------------------------------------------
@@ -497,54 +466,71 @@ class ZMSLinkElement(ZMSContainerObject, _metadata.Metadata):
             rtn = ZMSContainerObject.getTitle(self,REQUEST)
           else:
             rtn = ref_obj.getTitle(REQUEST)
+      return rtn
+
+    def getTitle(self, REQUEST):
+      rtn = ''
+      if self.getEmbedType() == 'remote':
+        remote_obj = self.getRemoteObj()
+        if type( remote_obj) is list:
+          for node in self.xmlNodeSet( remote_obj, 'title'):
+            rtn = node['cdata']
+      else:
+        proxy = self.getProxy()
+        rtn = self.getTitlePROXY( proxy, REQUEST)
       if len(rtn) == 0:
         rtn = self.display_type(REQUEST)
       return rtn
 
-    def getTitle(self, REQUEST):
-      proxy = self.getProxy()
-      rtn = self.getTitlePROXY( proxy, REQUEST)
-      return rtn
 
-      
     # --------------------------------------------------------------------------
     #  ZMSLinkElement.display_icon:
     # --------------------------------------------------------------------------
-    def display_icon(self, REQUEST, meta_type=''): 
+    def display_icon(self, REQUEST, meta_type=None, key='icon'): 
       ref_obj = self.getRefObj()
       if ref_obj is None or not self.isEmbedded(REQUEST):
         ref_obj = self
-      if len(meta_type) == 0:
+      if meta_type is None:
         if self.isActive(REQUEST):
-          return ref_obj.icon
+          return ZMSObject.display_icon(ref_obj,REQUEST,key=key)
         else:
-          return ref_obj.icon_disabled
+          return ZMSObject.display_icon(ref_obj,REQUEST,key='icon_disabled')
       else:
-        return ref_obj.display_icon(REQUEST,meta_type)
+        return ZMSObject.display_icon(ref_obj,REQUEST,meta_type,key)
+
 
-    
     # --------------------------------------------------------------------------
     #  ZMSLinkElement.isActive:
     # --------------------------------------------------------------------------
     def isActive(self, REQUEST):
       active = ZMSContainerObject.isActive(self,REQUEST) 
-      ref_obj = self.getRefObj()
-      if ref_obj is not None:
-          active = active and ref_obj.isActive(REQUEST)
+      if self.getEmbedType() == 'remote':
+        remote_obj = self.getRemoteObj()
+        if type( remote_obj) is list:
+          rtnVal = remote_obj[1]['attrs']['active'] in ['1','True']
+      else:
+        ref_obj = self.getRefObj()
+        if ref_obj is not None:
+            active = active and ref_obj.isActive(REQUEST)
       return active
 
-    
+
     # --------------------------------------------------------------------------
     #  ZMSLinkElement.isPage
     # --------------------------------------------------------------------------
     def isPage(self):
       rtnVal = False
-      req = {'lang':self.getPrimaryLanguage(),'preview':'preview'}
-      embedded = self.isEmbedded(req)
-      ref_obj = self.getRefObj()
-      rtnVal = rtnVal or (ref_obj is None and embedded)
-      if ref_obj is not None:
-        rtnVal = rtnVal or (ref_obj.isPage() and embedded)
+      if self.getEmbedType() == 'remote':
+        remote_obj = self.getRemoteObj()
+        if type( remote_obj) is list:
+          rtnVal = remote_obj[1]['attrs']['is_page'] in ['1','True']
+      else:
+        req = {'lang':self.getPrimaryLanguage(),'preview':'preview'}
+        embedded = self.isEmbedded(req)
+        ref_obj = self.getRefObj()
+        rtnVal = rtnVal or (ref_obj is None and embedded)
+        if ref_obj is not None:
+          rtnVal = rtnVal or (ref_obj.isPage() and embedded)
       return rtnVal
 
 
@@ -553,9 +539,17 @@ class ZMSLinkElement(ZMSContainerObject, _metadata.Metadata):
     # --------------------------------------------------------------------------
     def isPageElement(self): 
       rtnVal = False
-      ref_obj = self.getRefObj()
-      if ref_obj is not None:
-        rtnVal = rtnVal or ref_obj.isPageElement()
+      if self.getEmbedType() == 'remote':
+        remote_obj = self.getRemoteObj()
+        if type( remote_obj) is list:
+          rtnVal = remote_obj[1]['attrs']['is_pageelement'] in ['1','True']
+      else:
+        if self.isEmbedded( self.REQUEST):
+          ref_obj = self.getRefObj()
+          if ref_obj is not None:
+            rtnVal = rtnVal or ref_obj.isPageElement()
+        elif self.getObjProperty('align',self.REQUEST) not in ['','NONE']:
+          rtnVal = rtnVal or True
       return rtnVal
 
 
@@ -570,7 +564,7 @@ class ZMSLinkElement(ZMSContainerObject, _metadata.Metadata):
         rtn = proxy.getType()
       else:
         ref_obj = self.getRefObj()
-        if ref_obj is not None and ref_obj.meta_type=='ZMSCustom':
+        if ref_obj is not None:
            rtn = ref_obj.getType()
       return rtn
     
@@ -585,14 +579,20 @@ class ZMSLinkElement(ZMSContainerObject, _metadata.Metadata):
     #
     #  Overrides getObjProperty of _objattrs.ObjAttrs. 
     # --------------------------------------------------------------------------
-    def getObjPropertyPROXY(self, proxy, key, REQUEST={}, default=None): 
+    def getObjPropertyPROXY(self, proxy, key, REQUEST={}, default=None):
       obj_attr = proxy.getObjAttr( key)
       value = proxy.getObjAttrValue( obj_attr, REQUEST) 
       return value
 
     def getObjProperty(self, key, REQUEST={}, default=None): 
       value = self.getObjPropertyPROXY( self, key, REQUEST, default)
-      if key not in ['attr_ref','attr_dc_coverage'] and (value is None or len(str(value)) == 0 or value == 0):
+      # First exit...
+      if (value is None or len(str(value)) == 0 or value == 0) and \
+        key in self.getMetaobjAttrIds( self.meta_id):
+        value = ZMSObject.getObjProperty( self, key, REQUEST, default)
+      # Second exit...
+      if (value is None or len(str(value)) == 0 or value == 0) and \
+        key not in ['attr_ref','attr_dc_coverage','work_dt','work_uid']:
         recursive = self.isEmbeddedRecursive( REQUEST)
         if recursive:
           proxy = self.getProxy()
@@ -602,57 +602,12 @@ class ZMSLinkElement(ZMSContainerObject, _metadata.Metadata):
           ref_obj = self.getRefObj()
           if ref_obj != self and ref_obj is not None:
             value = self.getObjPropertyPROXY( ref_obj, key, REQUEST, default)
+          # Last exit...
+          if (value is None or len(str(value)) == 0 or value == 0):
+            value = ZMSObject.getObjProperty( self, key, REQUEST, default)
       return value
 
 
-    # --------------------------------------------------------------------------
-    #  ZMSLinkElement.getFormat
-    #
-    #  Includes getFormat of zmsgraphic.ZMSGraphic and zmstextarea.ZMSTextarea.
-    # --------------------------------------------------------------------------
-    def getFormat(self,REQUEST): 
-      return self.getObjProperty('format',REQUEST)
-
-
-    # --------------------------------------------------------------------------
-    #  ZMSLinkElement.getTeaserElements:
-    #
-    #  Overrides getTeaserElements of zmscontainerobject.ZMSContainerObject.
-    # --------------------------------------------------------------------------
-    def getTeaserElementsPROXY(self, proxy, REQUEST, current=None):
-      rtn = []
-      recursive = self.isEmbeddedRecursive( REQUEST)
-      if proxy != self and proxy is not None and recursive:
-        for el in ZMSContainerObject.getTeaserElements( proxy, REQUEST): 
-          if not el in rtn:
-            rtn.append(el)
-      else:
-        if self.isPage():
-          for el in ZMSContainerObject.getTeaserElements( self, REQUEST):
-            if not el in rtn:
-              rtn.append(el)
-          ref_obj = self.getRefObj()
-          if isinstance(ref_obj,ZMSContainerObject):
-            for el in ZMSContainerObject.getTeaserElements( ref_obj, REQUEST):
-              if not el in rtn:
-                rtn.append(el)
-      return rtn
-
-    def getTeaserElements(self, REQUEST, current=None):
-      rtn = []
-      embedded = self.isEmbedded( REQUEST)
-      if embedded:
-        if self.isPage():
-          for el in ZMSContainerObject.getTeaserElements( self.aq_parent, REQUEST, self):
-            if not el in rtn:
-              rtn.append(el)
-      proxy = self.getProxy()
-      for el in self.getTeaserElementsPROXY( proxy, REQUEST, current):
-        if not el in rtn:
-          rtn.append(el)
-      return rtn
-
-
     # --------------------------------------------------------------------------
     #  ZMSLinkElement.getNavItems:
     #
@@ -697,13 +652,11 @@ class ZMSLinkElement(ZMSContainerObject, _metadata.Metadata):
       return rtn
 
 
-    """
     ############################################################################
     ###
     ###  HTML-Presentation
     ###
     ############################################################################
-    """
 
     # --------------------------------------------------------------------------
     #  ZMSLinkElement.getHref2IndexHtml:
@@ -722,73 +675,58 @@ class ZMSLinkElement(ZMSContainerObject, _metadata.Metadata):
 
 
     # --------------------------------------------------------------------------
-    #	ZMSLinkElement._getBodyContent:
+    #  ZMSLinkElement._getBodyContent:
     #
-    #	HTML presentation of Linkelement. 
+    #  HTML presentation of link-element.
     # --------------------------------------------------------------------------
     def _getBodyContent(self, REQUEST):
       rtn = ''
-      proxy = self.getProxy()
-      if proxy != self and proxy is not None and self.isEmbeddedRecursive( self.REQUEST):
-        rtn += proxy._getBodyContent(REQUEST)
-      elif proxy == self and proxy is not None and self.isEmbedded( REQUEST):
-        ref_obj = self.getRefObj()
-        if ref_obj is not None:
-          rtn += ref_obj._getBodyContent(REQUEST)
+      if self.getEmbedType() == 'remote':
+        ref = self.getObjProperty('attr_ref',REQUEST)
+        try:
+          rtn += self.http_import( ref+'/getBodyContent')
+        except:
+          rtn += _globals.writeException(self,'[_getBodyContent]: can\'t embed from remote: ref=%s'%ref)
+      else:
+        proxy = self.getProxy()
+        if proxy != self and proxy is not None and self.isEmbeddedRecursive( self.REQUEST):
+          rtn += proxy._getBodyContent(REQUEST)
+        elif proxy == self and proxy is not None and self.isEmbedded( REQUEST):
+          ref_obj = self.getRefObj()
+          if ref_obj is not None:
+            rtn += ref_obj._getBodyContent( REQUEST)
+        else:
+          rtn += self._getBodyContent2( REQUEST)
       return rtn
 
 
     # --------------------------------------------------------------------------
-    #	ZMSLinkElement.renderShort:
-    #
-    #	Renders short presentation of Linkelement.
+    #  ZMSLinkElement.renderShort:
     # --------------------------------------------------------------------------
     def renderShort(self, REQUEST):
-      s = ''
-      ref = self.getObjProperty('attr_ref',REQUEST)
+      """
+      Renders short presentation of link-element.
+      """
+      rtn = ''
       ref_obj = self.getRefObj()
-      target = ''
-      img_src = 'external_link.gif'
-      if _zreferableitem.isInternalLink(ref):
-        if ref_obj is None:
-          img_src = 'internal_link_broken.gif'
-          ref = 'javascript:alert(\'Broken Link >%s<!\');'%(ref)
-        else:
-          img_src = 'internal_link.gif'
-          if self.isPreviewRequest(REQUEST):
-            ref = '%s/manage_main?lang=%s'%(ref_obj.absolute_url(),REQUEST['lang'])
-          else:
-            ref = ref_obj.getHref2IndexHtml(REQUEST)
-      elif _zreferableitem.isMailLink(ref):
-        img_src = 'mail_link.gif'
+      ref = self.getObjProperty('attr_ref',REQUEST)
       
-      if self.isEmbedded(REQUEST):
-        render = ''
+      if self.getEmbedType() == 'remote':
+        try:
+          rtn += self.http_import( ref+'/renderShort')
+        except:
+          rtn += _globals.writeException(self,'[renderShort]: can\'t embed from remote: ref=%s'%ref)
+      
+      elif self.isEmbedded(REQUEST) and ref_obj is not None:
         if ref_obj is None or ref_obj.isPage():
-          render += ZMSContainerObject.renderShort(self,REQUEST)
+          rtn += ZMSContainerObject.renderShort(self,REQUEST)
         else:
-          render += ref_obj.renderShort(REQUEST)
-        s += render
+          rtn += ref_obj.renderShort(REQUEST)
       
       else:
-        title = self.getTitle(REQUEST)
-        medline = title.lower()=='medline'
-        if medline:
-          title = '<img src="%spubmed_small.gif" title="Medline" border="0"/>'%(self.MISC_ZMS)
-          ref = _zreferableitem.getMedlineLink(ref)
-        if self.getObjProperty('attr_type',REQUEST) == 'new':
-          target = ' target="_blank"'
-        description = self.getObjProperty('attr_dc_description',REQUEST)
-        s += '<div class="%s">\n'%self.meta_type
-        s += '<div class="title">\n'
-        s += '<img src="%s%s" title="" border="0" align="absmiddle">\n'%(self.MISC_ZMS,img_src)
-        s += '<a href="%s"%s>%s</a>\n'%(ref,target,title)
-        s += '</div>\n'
-        if description is not None and len(str(description)) > 0:
-          s += '<div class="text">%s</div>\n'%description
-        s += '</div>\n'
+          rtn += self._getBodyContent( REQUEST)
       
-      return s
+      return rtn
 
 
     # --------------------------------------------------------------------------
@@ -806,27 +744,25 @@ class ZMSLinkElement(ZMSContainerObject, _metadata.Metadata):
       return s
 
 
-    """
     ############################################################################
-    ###  
+    ###
     ###  DOM-Methods
-    ### 
+    ###
     ############################################################################
-    """
 
     # --------------------------------------------------------------------------
     #  ZMSLinkElement.breadcrumbs_obj_path:
     # --------------------------------------------------------------------------
-    def breadcrumbs_obj_pathPROXY(self, proxy):
+    def breadcrumbs_obj_pathPROXY(self, proxy, portalMaster=True):
       if proxy != self and proxy is not None and self.isEmbeddedRecursive( self.REQUEST):
         rtn = proxy.breadcrumbs_obj_path()
       else:
-        rtn = ZMSObject.breadcrumbs_obj_path( proxy)
+        rtn = ZMSObject.breadcrumbs_obj_path( proxy, portalMaster)
       return rtn
     
-    def breadcrumbs_obj_path(self):
+    def breadcrumbs_obj_path(self, portalMaster=True):
       proxy = self.getProxy()
-      rtn = self.breadcrumbs_obj_pathPROXY( proxy)
+      rtn = self.breadcrumbs_obj_pathPROXY( proxy, portalMaster)
       return rtn
 
 
@@ -902,13 +838,12 @@ class ZMSLinkElement(ZMSContainerObject, _metadata.Metadata):
       rtn = self.getChildNodesPROXY( proxy, REQUEST, meta_types)
       return rtn
 
-    """
-    ############################################################################    
-    #
-    #   Printable
-    #
-    ############################################################################    
-    """
+
+    ############################################################################
+    ###
+    ###  Printable
+    ###
+    ############################################################################
 
     # --------------------------------------------------------------------------
     #  ZMSLinkElement.printHtml:
