@@ -2,12 +2,10 @@
 # zmsproxyobject.py
 #
 # $Id: zmsproxyobject.py,v 1.7 2004/11/23 23:04:47 zmsdev Exp $
-# $Name:  $
+# $Name:$
 # $Author: zmsdev $
 # $Revision: 1.7 $
 #
-# Implementation of class ZMSProxyObject (see below).
-#
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
 # as published by the Free Software Foundation; either version 2
@@ -25,20 +23,23 @@
 
 # Imports.
 from types import StringTypes
+from zope.interface import implements
 # Product Imports.
-from zmsobject import ZMSObject
+import IZMSMetamodelProvider, IZMSFormatProvider
 from zmscontainerobject import ZMSContainerObject
 
 
 ################################################################################
 ################################################################################
-###   
-###   C l a s s
-###   
+###
+###  Class
+###
 ################################################################################
 ################################################################################
-
 class ZMSProxyObject(ZMSContainerObject):
+    implements(
+      IZMSMetamodelProvider.IZMSMetamodelProvider,
+      IZMSFormatProvider.IZMSFormatProvider)
 
 
     # --------------------------------------------------------------------------
@@ -50,8 +51,7 @@ class ZMSProxyObject(ZMSContainerObject):
       self.base = base
       self.url_base = url_base
       self.meta_type = proxy.meta_type
-      if self.meta_type == 'ZMSCustom':
-        self.meta_id = proxy.meta_id
+      self.meta_id = proxy.meta_id
       self.dGlobalAttrs = proxy.dGlobalAttrs
       self.REQUEST = proxy.REQUEST
       self.id = id
@@ -86,27 +86,18 @@ class ZMSProxyObject(ZMSContainerObject):
     # --------------------------------------------------------------------------
     #  ZMSProxyObject.absolute_url
     # --------------------------------------------------------------------------
-    def absolute_url(self):
+    def absolute_url(self, relative=0):
       url_base = self.url_base
       rtn = url_base
       return rtn
 
 
-    # --------------------------------------------------------------------------
-    #  ZMSProxyObject.getMetaobjIds:
-    # --------------------------------------------------------------------------
-    def getMetaobjIds(self, sort=1):
-      proxy = self.proxy
-      rtn = proxy.getMetaobjIds( sort)
-      return rtn
-
-
     # --------------------------------------------------------------------------
     #  ZMSProxyObject.getMetaobjAttrIds:
     # --------------------------------------------------------------------------
-    def getMetaobjAttrIds(self, meta_id):
+    def getMetaobjAttrIds(self, meta_id, types=[]):
       proxy = self.proxy
-      rtn = proxy.getMetaobjAttrIds( meta_id)
+      rtn = proxy.getMetaobjAttrIds( meta_id, types)
       return rtn
 
 
@@ -252,22 +243,6 @@ class ZMSProxyObject(ZMSContainerObject):
       return rtn
 
 
-    # --------------------------------------------------------------------------
-    #  ZMSProxyObject.getTeaserElements:
-    # --------------------------------------------------------------------------
-    def getTeaserElements(self, REQUEST, current=None):
-      rtn = []
-      recursive = self.recursive
-      if recursive:
-        proxy = self.proxy
-        req = self.__request__( REQUEST)
-        if hasattr( proxy, 'getTeaserElementsPROXY'):
-          rtn = proxy.getTeaserElementsPROXY( proxy, req, current)
-        else:
-          rtn = proxy.getTeaserElements( req, current)
-      return rtn
-
-
     # --------------------------------------------------------------------------
     #  ZMSProxyObject.getChildNodes:
     # --------------------------------------------------------------------------
@@ -285,11 +260,11 @@ class ZMSProxyObject(ZMSContainerObject):
 
 
     # --------------------------------------------------------------------------
-    #  ZMSProxyObject.getLevelnfc
+    #  ZMSProxyObject.getSecNo
     # --------------------------------------------------------------------------
-    def getLevelnfc(self, REQUEST):
+    def getSecNo(self):
       proxy = self.proxy
-      rtn = proxy.getLevelnfc( REQUEST)
+      rtn = proxy.getSecNo( )
       return rtn
 
 
@@ -325,7 +300,12 @@ class ZMSProxyObject(ZMSContainerObject):
     # --------------------------------------------------------------------------
     #  ZMSProxyObject.getParentNode
     # --------------------------------------------------------------------------
+    getParentNode__roles__ = None
     def getParentNode(self):
+      """
+      The parent of this node. 
+      All nodes except root may have a parent.
+      """
       rtn = self.base
       return rtn
 
@@ -335,10 +315,7 @@ class ZMSProxyObject(ZMSContainerObject):
     # --------------------------------------------------------------------------
     def getType(self):
       proxy = self.proxy
-      if proxy.meta_type == 'ZMSCustom':
-        rtn = proxy.getType( )
-      else:
-        rtn = None
+      rtn = proxy.getType()
       return rtn
 
 
@@ -407,28 +384,27 @@ class ZMSProxyObject(ZMSContainerObject):
       req = self.__request__( REQUEST)
       rtn = proxy._getBodyContent( req)
       return rtn
-    
-    
-    # --------------------------------------------------------------------------
-    #  ZMSProxyObject.getBodyContent:
-    # --------------------------------------------------------------------------
-    def getBodyContent(self, REQUEST):
+
+    def getBodyContent(self, REQUEST, forced=False):
+      """
+      HTML presentation in body-content. 
+      """
       proxy = self.proxy
       req = self.__request__( REQUEST)
       rtn = proxy.getBodyContent( req)
       return rtn
 
-    
+
     # --------------------------------------------------------------------------
     #  ZMSProxyObject.display_icon:
     # --------------------------------------------------------------------------
-    def display_icon(self, REQUEST, meta_type=''): 
+    def display_icon(self, REQUEST, meta_type=None, key='icon'): 
       proxy = self.proxy
       req = self.__request__( REQUEST)
-      rtn = proxy.display_icon( req, meta_type)
+      rtn = proxy.display_icon( req, meta_type, key)
       return rtn
-    
-    
+
+
     # --------------------------------------------------------------------------
     #  ZMSProxyObject.getLangIds:
     # --------------------------------------------------------------------------
@@ -465,24 +441,6 @@ class ZMSProxyObject(ZMSContainerObject):
       return rtn
 
 
-    # --------------------------------------------------------------------------
-    #  ZMSProxyObject.getMetadictAttrs:
-    # --------------------------------------------------------------------------
-    def getMetadictAttrs(self, meta_type=None):
-      proxy = self.proxy
-      rtn = proxy.getMetadictAttrs( meta_type)
-      return rtn
-
-
-    # --------------------------------------------------------------------------
-    #  ZMSProxyObject.getMetadictAttr:
-    # --------------------------------------------------------------------------
-    def getMetadictAttr(self, key):
-      proxy = self.proxy
-      rtn = proxy.getMetadictAttr( key)
-      return rtn
-
-
     # --------------------------------------------------------------------------
     #  ZMSProxyObject.getPrimaryLanguage:
     # --------------------------------------------------------------------------
@@ -515,16 +473,16 @@ class ZMSProxyObject(ZMSContainerObject):
     # --------------------------------------------------------------------------
     #  ZMSProxyObject.breadcrumbs_obj_path:
     # --------------------------------------------------------------------------
-    def breadcrumbs_obj_path(self):
+    def breadcrumbs_obj_path(self, portalMaster=True):
       proxy = self.proxy
       base = self.base
       rtn = base.breadcrumbs_obj_path( ) + [ self]
       recursive = self.recursive
       if proxy is not None and proxy is ZMSProxyObject:
         if hasattr( proxy, 'breadcrumbs_obj_pathPROXY'):
-          rtn = proxy.breadcrumbs_obj_pathPROXY( proxy)
+          rtn = proxy.breadcrumbs_obj_pathPROXY( proxy, portalMaster)
         else:
-          rtn = proxy.breadcrumbs_obj_path()
+          rtn = proxy.breadcrumbs_obj_path(portalMaster)
       return rtn
 
 
@@ -547,7 +505,16 @@ class ZMSProxyObject(ZMSContainerObject):
     # --------------------------------------------------------------------------
     def getHref2IndexHtml(self, REQUEST, deep=1): 
       proxy = self.proxy
-      rtn = ZMSObject.getHref2IndexHtml( self, REQUEST, deep)
+      rtn = ZMSContainerObject.getHref2IndexHtml( self, REQUEST, deep)
+      return rtn
+
+
+    # --------------------------------------------------------------------------
+    #  ZMSProxyObject.getStylesheet:
+    # --------------------------------------------------------------------------
+    def getStylesheet(self, id=None):
+      base = self.base
+      rtn = base.getStylesheet( id)
       return rtn
 
 
@@ -559,4 +526,45 @@ class ZMSProxyObject(ZMSContainerObject):
       rtn = proxy.printHtml( level, sectionizer, REQUEST, deep)
       return rtn
 
+
+    ############################################################################
+    ###
+    ###   Interface IZMSMetamodelProvider: delegate to proxy
+    ###
+    ############################################################################
+
+    def getMetaobjId(self, name):
+      return self.proxy.getMetaobjId( name)
+
+    def getMetaobjIds(self, sort=1, excl_ids=[]):
+      return self.proxy.getMetaobjIds( sort, excl_ids)
+
+    def getMetaobj(self, id):
+      return self.proxy.getMetaobj( id)
+
+    def getMetaobjAttrIds(self, meta_id, types=[]):
+      return self.proxy.getMetaobjAttrIds( meta_id, types)
+
+    def getMetaobjAttr(self, meta_id, key):
+      return self.proxy.getMetaobjAttr( meta_id, key)
+
+
+    ############################################################################
+    ###
+    ###   Interface IZMSFormatProvider: delegate to proxy
+    ###
+    ############################################################################
+
+    def getTextFormatDefault(self):
+      return self.proxy.getTextFormatDefault()
+
+    def getTextFormat(self, id, REQUEST):
+      return self.proxy.getTextFormat(id, REQUEST)
+
+    def getTextFormats(self, REQUEST):
+      return self.proxy.getTextFormats(REQUEST)
+
+    def getCharFormats(self):
+      return self.proxy.getCharFormats()
+
 ################################################################################
