@@ -1,13 +1,11 @@
 ################################################################################
 # zmstrashcan.py
 #
-# $Id: zmstrashcan.py,v 1.5 2004/11/23 23:19:31 zmsdev Exp $
-# $Name:  $
-# $Author: zmsdev $
-# $Revision: 1.5 $
+# $Id:$
+# $Name:$
+# $Author:$
+# $Revision:$
 #
-# Implementation of classes ZMSTrashcan (see below).
-# 
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
 # as published by the Free Software Foundation; either version 2
@@ -38,7 +36,7 @@ import _globals
 ################################################################################
 ################################################################################
 ###
-###   C l a s s
+###   Class
 ###
 ################################################################################
 ################################################################################
@@ -46,10 +44,10 @@ class ZMSTrashcan(ZMSContainerObject):
 
     # Properties.
     # -----------
-    meta_type = "ZMSTrashcan"
-    icon = "misc_/zms/zmstrashcan.gif"
-    icon_disabled = "misc_/zms/zmstrashcan_disabled.gif"
-    
+    meta_type = meta_id = "ZMSTrashcan"
+    icon = "misc_/zms/ZMSTrashcan.gif"
+    icon_disabled = "misc_/zms/ZMSTrashcan_disabled.gif"
+
     # Management Options.
     # -------------------
     manage_options = ( 
@@ -70,18 +68,49 @@ class ZMSTrashcan(ZMSContainerObject):
 
     # Management Interface.
     # ---------------------
-    manage_properties = HTMLFile('dtml/zmstrashcan/manage_properties', globals())
+    manage_properties = HTMLFile('dtml/ZMSTrashcan/manage_properties', globals())
 
-    # Properties.
-    # -----------
-    __obj_attrs__ = {}
 
     """
-    ############################################################################    
+    ############################################################################
     ###
-    ###   P r o p e r t i e s
+    ###   Constructor
     ###
-    ############################################################################    
+    ############################################################################
+    """
+
+    ############################################################################
+    #  ZMSTrashcan.__init__: 
+    #
+    #  Constructor (initialise a new instance of ZMSTrashcan).
+    ############################################################################
+    def __init__(self):
+      """ ZMSTrashcan.__init__ """
+      id = 'trashcan'
+      sort_id = 0
+      ZMSContainerObject.__init__(self,id,sort_id)
+
+
+    # --------------------------------------------------------------------------
+    #  ZMSTrashcan.display_icon:
+    #
+    #  @param REQUEST
+    # --------------------------------------------------------------------------
+    def display_icon(self, REQUEST, meta_type=None, key='icon'):
+      obj_type = meta_type
+      if obj_type is None:
+        if not self.isActive(REQUEST):
+          key = 'icon_disabled'
+        obj_type = self.meta_id
+      return getattr(self,key)
+
+
+    """
+    ############################################################################
+    ###
+    ###   Properties
+    ###
+    ############################################################################
     """
 
     ############################################################################
@@ -136,7 +165,7 @@ class ZMSTrashcan(ZMSContainerObject):
         self.manage_delObjects(ids=ids)
         #-- Update time-stamp.
         setattr(self,'last_garbage_collection',now)
-    
+
 
     # --------------------------------------------------------------------------
     #  ZMSTrashcan._verifyObjectPaste:
@@ -151,25 +180,35 @@ class ZMSTrashcan(ZMSContainerObject):
     #
     #  Overrides getDCCoverage of ZMSObject.
     # --------------------------------------------------------------------------
-    def getDCCoverage(self, REQUEST={}): 
+    def getDCCoverage(self, REQUEST={}):
       return 'global.'+self.getPrimaryLanguage()
 
     # --------------------------------------------------------------------------
     #  ZMSTrashcan.isActive
     # --------------------------------------------------------------------------
-    def isActive(self, REQUEST): 
+    def isActive(self, REQUEST):
       return len(self.getChildNodes(REQUEST))>0
 
+    # --------------------------------------------------------------------------
+    #  ZMSTrashcan.isPage
+    # --------------------------------------------------------------------------
+    def isPage(self):
+      return False
+
+    # --------------------------------------------------------------------------
+    #  ZMSTrashcan.getObjProperty
+    # --------------------------------------------------------------------------
+    def getObjProperty(self, key, REQUEST={}, par=None):
+      return ''
+
     # --------------------------------------------------------------------------
     #  ZMSTrashcan.getTitle
     # --------------------------------------------------------------------------
-    def getTitle(self, REQUEST): 
+    def getTitle(self, REQUEST):
+      try:
+        x = self.display_type(REQUEST) + " (" + str(len(self.getChildNodes(REQUEST))) + " " + self.getLangStr('ATTR_OBJECTS',REQUEST['lang']) + ")"
+      except:
+        print self.display_type(REQUEST)
       return self.display_type(REQUEST) + " (" + str(len(self.getChildNodes(REQUEST))) + " " + self.getLangStr('ATTR_OBJECTS',REQUEST['lang']) + ")"
 
-    # --------------------------------------------------------------------------
-    #  ZMSTrashcan.getTitlealt
-    # --------------------------------------------------------------------------
-    def getTitlealt(self, REQUEST): 
-      return self.display_type(REQUEST)
-
 ################################################################################
